#include "precomp.h"

static volatile BOOL gs_blInDebugMode;
static volatile BOOL gs_blTargetVoltageDetected;
static volatile BYTE gs_bStatus;
static volatile unsigned int gs_wChipId;
static xdata BYTE gs_bDebugConfig;
static const CHIPINFO code* gs_pChipInfo;

static void AdcIsr(void) interrupt INTERRUPT_ADC0_EOC using 1
{
    AD0INT = 0;
    g_wTargetVoltage.i = (unsigned int)((ADC0 * 3693UL) / 1023U);
}

static void WindowCompareIsr(void) interrupt INTERRUPT_ADC0_WINDOW
{
    static BYTE cnt;

    if(!ADC0LTL)
    {
        cnt = 25;
        ADC0LTH = 0x02;
        ADC0LTL = 0x24;
    }
    else if(ADC0GTL != 0xFF)
    {
        if(ADC0 >= 0x022F)
        {
            if(!--cnt)
            {
                gs_blTargetVoltageDetected = TRUE;
                gs_bStatus |= (BYTE)TargetVoltagePresent;

                if(DbgItf_EnterDebugMode())
                {
                    DbgItf_LeaveDebugMode();
                    ADC0GTH = 0x03;
                    ADC0GTL = 0xFF;
                }
            }
        }
        else
        {
            ADC0LTH = 0x00;
            ADC0LTL = 0x00;
        }
    }
    else
    {
        ADC0GTH = 0x02;
        ADC0GTL = 0x2F;
        ADC0LTH = 0x00;
        ADC0LTL = 0x00;
        gs_wChipId = 0;
        gs_blTargetVoltageDetected = FALSE;
        gs_blInDebugMode = FALSE;
        gs_bStatus &= ~((BYTE)TargetVoltagePresent | (BYTE)DebugMode);
        LED1 = 1;
    }

    AD0WINT = 0;
}

static void DbgItf_WaitTicks(BYTE bNumTicks)
{
    while(bNumTicks--)
    {
        _nop_();
        _nop_();
    }
}

static void DbgItf_SetDC(void)
{
    _nop_();
    DC = 1;
    _nop_();
}

static void DbgItf_ClearDC(void)
{
    _nop_();
    DC = 0;
    _nop_();
}

static void DbgItf_PulseClock()
{
    DbgItf_SetDC();
    DbgItf_ClearDC();
}

static void DbgItf_OutByte(BYTE b)
{
    BYTE num;

    for(num = 8; num; --num)
    {
        DbgItf_SetDC();
        DD_DAT = b & 0x80;
        // DbgItf_ClearDC();
        DC = 0;
        b <<= 1;
    }

    _nop_();
    DbgItf_ClearDC();
}

static BYTE DbgItf_InByte(void)
{
    BYTE bData = 0;
    BYTE num;

    for(num = 8; num; --num)
    {
        DbgItf_SetDC();
        bData <<= 1;

        if(DD_DAT)
            bData |= 1;

        DC = 0;
    }

    _nop_();
    DbgItf_ClearDC();
    return(bData);
}

static BOOL DbgItf_WaitForResponseReady(void)
{
    BYTE b = 0xFF;

    do
    {
        if(!DD_DAT)
            return(TRUE);

        DbgItf_InByte();
    } while(b--);

    return(FALSE);
}

static BOOL DbgItf_GetResponseByte(BYTE* pb)
{
    if(!DbgItf_WaitForResponseReady())
        return(FALSE);

    *pb = DbgItf_InByte();
    return(TRUE);
}

static BOOL DbgItf_ReadWord(BYTE cmd, unsigned int* pw)
{
    BYTE bl, bh;

    if(!gs_blInDebugMode)
        return(FALSE);

    DbgItf_WaitTicks(2);
    DbgItf_OutByte(cmd);
    DD_DAT = 1;
    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DD_DAT = 1;
    DbgItf_WaitTicks(10);

    if(!DbgItf_WaitForResponseReady())
    {
        DD_DIR = 1;
        P0MDOUT |= 0x40;
        return(FALSE);
    }

    bh = DbgItf_InByte();
    DbgItf_WaitTicks(2);
    bl = DbgItf_InByte();
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    *pw = MAKEWORD(bl, bh);
    return(TRUE);
}

static BOOL DbgItf_ReadByte(BYTE cmd, unsigned char* pb)
{
    BOOL ret;

    if(!gs_blInDebugMode)
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(cmd);
    DD_DAT = 1;
    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(pb);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

static BOOL DbgItf_WriteByte(BYTE cmd, unsigned char b, unsigned char* result)
{
    BOOL ret;

    if(!gs_blInDebugMode)
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(cmd);
    DbgItf_WaitTicks(3);
    DbgItf_OutByte(b);
    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

static BOOL DbgItf_WriteCommand(BYTE cmd, unsigned char* result)
{
    BOOL ret;

    if(!gs_blInDebugMode)
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(cmd);
    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

static const CHIPINFO code* DbgItf_ChipId2ChipInfo(BYTE bChipId)
{
    static code const CHIPINFO s_chipinfo[] =
    {
        /* bChipId, bDbgStatusXorMask, bFlashWordSize, bFWT, bBrkpntEnableBit */
        { UnknownChip, 0x00, 2, 0x10, 3 },
        { CC1110, 0x00, 2, 0x11, 2 },
        { CC1111, 0x00, 2, 0x10, 2 },
        { CC2430, 0x00, 4, 0x15, 2 },
        { CC2431, 0x00, 4, 0x15, 2 },
        { CC2510, 0x00, 2, 0x11, 2 },
        { CC2511, 0x00, 2, 0x10, 2 },
        { CC2530, 0x90, 4, 0x00, 3 },
        { CC2531, 0x90, 4, 0x00, 3 },
        { CC2540, 0x90, 4, 0x00, 3 },
        { CC2541, 0x90, 4, 0x00, 3 },
        { CC2543, 0x90, 4, 0x00, 3 },
        { CC2544, 0x90, 4, 0x00, 3 },
        { CC2545, 0x90, 4, 0x00, 3 }
    };

    const CHIPINFO code* pci = &s_chipinfo[_countof(s_chipinfo) - 1];
    
    do
    {
        if(pci->bChipId == bChipId)
            break;

        --pci;
    } while(pci->bChipId != UnknownChip);

    return(pci);
}

void DbgItf_Initialize(void)
{
    DD_DIR = 1;
    DD_DAT = 0;
    DC = 0;
    gs_blInDebugMode = TRUE;
    DbgItf_LeaveDebugMode();
}

void DbgItf_SetSpeed(unsigned int wSpeed)
{
    TMR2RLL = LOBYTE(wSpeed);
    TMR2RLH = HIBYTE(wSpeed);
}

BOOL DbgItf_EnterDebugMode(void)
{
    if(!gs_blInDebugMode && gs_blTargetVoltageDetected)
    {
        gs_blInDebugMode = TRUE;
        DbgItf_LeaveDebugMode();
        DbgItf_WaitTicks(2);
        RESET_N = 0;
        DbgItf_WaitTicks(255);
        DbgItf_PulseClock();
        DbgItf_PulseClock();
        DbgItf_WaitTicks(255);
        DbgItf_ClearDC();
        RESET_N = 1;
        DbgItf_WaitTicks((TMR2RL <= DBGITF_SPEED_SLOW) ? 100 : 25);
        gs_blInDebugMode = TRUE;

        if(!DbgItf_ReadWord(DBGITF_CMD_GET_CHIP_ID, &gs_wChipId))
        {
            DbgItf_LeaveDebugMode();
        }
        else if((gs_wChipId == 0xFFFF) || (!gs_wChipId))
        {
            gs_wChipId = 0;
            DbgItf_LeaveDebugMode();
        }
        else
        {
            gs_bStatus |= (BYTE)DebugMode;
            LED1 = 0;
            gs_pChipInfo = DbgItf_ChipId2ChipInfo(HIBYTE(gs_wChipId));
        }
    }

    return(gs_blInDebugMode);
}

void DbgItf_LeaveDebugMode(void)
{
    if(gs_blInDebugMode)
    {
        LED1 = 1;
        gs_blInDebugMode = FALSE;
        gs_bStatus &= ~((BYTE)DebugMode);
        RESET_N = 0;
        DbgItf_WaitTicks((TMR2RL <= DBGITF_SPEED_SLOW) ? 250 : 30);
        RESET_N = 1;
    }
}

unsigned int DbgItf_GetChipId(void)
{
    return(gs_wChipId);
}

BYTE DbgItf_GetStatus(void)
{
    return(gs_bStatus);
}

BOOL DbgItf_ReadDebugConfig(BYTE* pConfig)
{
    return(DbgItf_ReadByte(DBGITF_CMD_RD_CONFIG, pConfig));
}

BOOL DbgItf_WriteDebugConfig(BYTE config)
{
    BYTE result;

    if((!DbgItf_WriteByte(DBGITF_CMD_WR_CONFIG, config, &result)))
        return(FALSE);

    gs_bDebugConfig = config;
    return(TRUE);
}

BYTE DbgItf_GetDebugConfig(void)
{
    return(gs_bDebugConfig);
}

BOOL DbgItf_ReadDebugStatus(BYTE* pStatus)
{
    if(!DbgItf_ReadByte(DBGITF_CMD_READ_STATUS, pStatus))
        return(FALSE);

    *pStatus ^= gs_pChipInfo->bDbgStatusXorMask;
    return(TRUE);
}

BOOL DbgItf_Resume(void)
{
    BYTE result;
    return(DbgItf_WriteCommand(DBGITF_CMD_RESUME, &result));
}

BOOL DbgItf_Halt(void)
{
    BYTE result;
    return(DbgItf_WriteCommand(DBGITF_CMD_HALT, &result));
}

BOOL DbgItf_ChipErase(void)
{
    BYTE result;

    if(!DbgItf_WriteCommand(DBGITF_CMD_CHIP_ERASE, &result))
        return(FALSE);

    while(DbgItf_ReadDebugStatus(&result))
    {
        if(result & CHIP_ERASE_DONE)
            return(TRUE);

        _nop_();
    }

    return(FALSE);
}

BOOL DbgItf_DebugInstr(BYTE cnt, BYTE byt1, BYTE byt2, BYTE byt3, BYTE* result)
{
    BOOL ret;

    if((!gs_blInDebugMode) || (!cnt) || (cnt > 3))
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(DBGITF_CMD_DEBUG_INSTR | cnt);
    DbgItf_WaitTicks(1);
    DbgItf_OutByte(byt1);

    if(--cnt)
    {
        DbgItf_OutByte(byt2);

        if(--cnt)
            DbgItf_OutByte(byt3);
    }

    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DD_DAT = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

BOOL DbgItf_StepReplace(BYTE cnt, BYTE byt1, BYTE byt2, BYTE byt3, BYTE* result)
{
    BOOL ret;

    if((!gs_blInDebugMode) || (!cnt) || (cnt > 3))
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(DBGITF_CMD_STEP_REPLACE | cnt);
    DbgItf_WaitTicks(1);
    DbgItf_OutByte(byt1);

    if(--cnt)
    {
        DbgItf_OutByte(byt2);

        if(--cnt)
            DbgItf_OutByte(byt3);
    }

    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(16);
    ret = DbgItf_GetResponseByte(result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

BOOL DbgItf_StepInstr(BYTE* result)
{
    return(DbgItf_WriteCommand(DBGITF_CMD_STEP_INSTR, result));
}

BOOL DbgItf_SetHwBreakpoint(BYTE brkpntNumber, BOOL enable, unsigned long address)
{
    BYTE result;
    BOOL ret;

    if((!gs_blInDebugMode) || (brkpntNumber > 3) || (address > 0x0003FFFF))
        return(FALSE);

    DbgItf_WaitTicks(1);
    DbgItf_OutByte(DBGITF_CMD_SET_HW_BRKPNT);
    DbgItf_OutByte(((BYTE)(brkpntNumber << 3)) | ((enable) ? (1 << gs_pChipInfo->bBrkpntEnableBit) : 0x00) | (BYTE)(GetHiword(address) & 0x03));
    DbgItf_OutByte(HIBYTE(address));
    DbgItf_OutByte(LOBYTE(address));
    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(&result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}

BOOL DbgItf_GetPc(unsigned short* pc)
{
    return(DbgItf_ReadWord(DBGITF_CMD_GET_PC, pc));
}

BOOL DbgItf_GetBm(BYTE* pBank)
{
    return(DbgItf_ReadByte(DBGITF_CMD_GET_BM, pBank));
}

const CHIPINFO* DbgItf_GetChipInfo(void)
{
    return(gs_pChipInfo);
}

BOOL DbgItf_BurstWrite(unsigned int len, const BYTE* pData)
{
    BYTE result;
    BOOL ret;

    if(!gs_blInDebugMode)
        return(FALSE);

    DbgItf_OutByte(DBGITF_CMD_BURST_WRITE | HIBYTE(len));
    DbgItf_OutByte(LOBYTE(len));

    while(len--)
        DbgItf_OutByte(*pData++);

    P0MDOUT &= ~0x40;
    DD_DIR = 0;
    DbgItf_WaitTicks(10);
    ret = DbgItf_GetResponseByte(&result);
    DD_DIR = 1;
    P0MDOUT |= 0x40;
    return(ret);
}
