#ifndef DEBUGITF_H
#define DEBUGITF_H

#define DBGITF_SPEED_SLOW           (65536 - 240)
#define DBGITF_SPEED_MEDIUM         (65536 - 48)
#define DBGITF_SPEED_FAST           (65536 - 27)

#define DBGITF_CMD_CHIP_ERASE       0x14
#define DBGITF_CMD_WR_CONFIG        0x1D
#define DBGITF_CMD_RD_CONFIG        0x24
#define DBGITF_CMD_GET_PC           0x28
#define DBGITF_CMD_READ_STATUS      0x34
#define DBGITF_CMD_SET_HW_BRKPNT    0x3B
#define DBGITF_CMD_HALT             0x44
#define DBGITF_CMD_RESUME           0x4C
#define DBGITF_CMD_DEBUG_INSTR      0x54
#define DBGITF_CMD_STEP_INSTR       0x5C
#define DBGITF_CMD_GET_BM           0x60
#define DBGITF_CMD_STEP_REPLACE     0x64
#define DBGITF_CMD_GET_CHIP_ID      0x68
#define DBGITF_CMD_BURST_WRITE      0x80

// Status bits as returned by DBGITF_CMD_READ_STATUS
#define CHIP_ERASE_DONE             0x80
#define PCON_IDLE                   0x40
#define CPU_HALTED                  0x20
#define POWER_MODE_0                0x10
#define HALT_STATUS                 0x08
#define DEBUG_LOCKED                0x04
#define OSCILLATOR_STABLE           0x02
#define STACK_OVERFLOW              0x01

enum ChipId
{
    UnknownChip = 0x00,
    CC1110 = 0x01,
    CC1111 = 0x11,
    CC2430 = 0x85,
    CC2431 = 0x89,
    CC2510 = 0x81,
    CC2511 = 0x91,
    CC2530 = 0xA5,
    CC2531 = 0xB5,
    CC2533 = 0x95,
    CC2540 = 0x8D,
    CC2541 = 0x41,
    CC2543 = 0x43,
    CC2544 = 0x44,
    CC2545 = 0x45
};

enum DebugConfigFlags
{
    SoftPowerMode = 0x20,
    TimersOff = 0x08,
    DmaPause = 0x04,
    TimerSuspend = 0x02
};

enum StatusFlags
{
    TargetVoltagePresent = 0x01,
    DebugMode = 0x02
};

typedef struct tagCHIPINFO
{
    BYTE bChipId;
    BYTE bDbgStatusXorMask;
    BYTE bFlashWordSize;
    BYTE bFWT;
    BYTE bBrkpntEnableBit;
} CHIPINFO;

void DbgItf_Initialize(void);
void DbgItf_SetSpeed(unsigned int wSpeed);
BOOL DbgItf_EnterDebugMode(void);
void DbgItf_LeaveDebugMode(void);
unsigned int DbgItf_GetChipId(void);
BYTE DbgItf_GetStatus(void);
BOOL DbgItf_ReadDebugConfig(BYTE* pConfig);
BOOL DbgItf_WriteDebugConfig(BYTE config);
BOOL DbgItf_ReadDebugStatus(BYTE* pStatus);
BOOL DbgItf_Resume(void);
BOOL DbgItf_Halt(void);
BOOL DbgItf_ChipErase(void);
BOOL DbgItf_DebugInstr(BYTE cnt, BYTE byt1, BYTE byt2, BYTE byt3, BYTE* result);
BOOL DbgItf_StepReplace(BYTE cnt, BYTE byt1, BYTE byt2, BYTE byt3, BYTE* result);
BOOL DbgItf_StepInstr(BYTE* result);
BOOL DbgItf_SetHwBreakpoint(BYTE brkpntNumber, BOOL enable, unsigned long address);
BOOL DbgItf_GetPc(unsigned short* pc);
BOOL DbgItf_GetBm(BYTE* pBank);
const CHIPINFO* DbgItf_GetChipInfo(void);
BYTE DbgItf_GetDebugConfig(void);
BOOL DbgItf_BurstWrite(unsigned int len, const BYTE* pData);

#define DbgItf_IsInDebugMode()  (DbgItf_GetStatus() & DebugMode)

#endif
