#ifndef USB_DESCRIPTOR_H
#define USB_DESCRIPTOR_H

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    WORD bcdUSB;
    BYTE bDeviceClass;
    BYTE bDeviceSubClass;
    BYTE bDeviceProtocol;
    BYTE bMaxPacketSize0;
    WORD idVendor;
    WORD idProduct;
    WORD bcdDevice;
    BYTE iManufacturer;
    BYTE iProduct;
    BYTE iSerialNumber;
    BYTE bNumConfigurations;
} DEVICE_DESCRIPTOR;

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    WORD wTotalLength;
    BYTE bNumInterfaces;
    BYTE bConfigurationValue;
    BYTE iConfiguration;
    BYTE bmAttributes;
    BYTE bMaxPower;
} CONFIGURATION_DESCRIPTOR;

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    BYTE bInterfaceNumber;
    BYTE bAlternateSetting;
    BYTE bNumEndpoints;
    BYTE bInterfaceClass;
    BYTE bInterfaceSubClass;
    BYTE bInterfaceProtocol;
    BYTE iInterface;
} INTERFACE_DESCRIPTOR;

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    BYTE bEndpointAddress;
    BYTE bmAttributes;
    WORD wMaxPacketSize;
    BYTE bInterval;
} ENDPOINT_DESCRIPTOR;

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    WORD pString[1];
} STRING_DESCRIPTOR;

typedef struct
{
    BYTE bLength;
    BYTE bDescriptorType;
    WORD bscHID;
    BYTE bCountryCode;
    BYTE bNumDescriptors;
    BYTE bRDescriptorType;
    WORD wDescriptorLength;
} HID_DESCRIPTOR;

typedef struct
{
    CONFIGURATION_DESCRIPTOR    cfgDescriptor;
    INTERFACE_DESCRIPTOR        itfDescriptor;
    HID_DESCRIPTOR              hidDescriptor;
    ENDPOINT_DESCRIPTOR         epDescriptor[2];
} CONFIGURATION_DESCRIPTOR_COMPLETE;

//-----------------------------
// SETUP Packet Type Definition
//-----------------------------
typedef struct
{
    unsigned char bmRequestType;        // Request recipient, type, and dir.
    unsigned char bRequest;             // Specific standard request number
    WORD wValue;                        // varies according to request
    WORD wIndex;                        // varies according to request
    WORD wLength;                       // Number of bytes to transfer
} setup_buffer;                        // End of SETUP Packet Type

extern code const DEVICE_DESCRIPTOR code gc_DeviceDescriptor;
extern code const CONFIGURATION_DESCRIPTOR_COMPLETE gc_ConfigurationDescriptor;
extern code const BYTE code gc_ReportDescriptor[];
extern code const BYTE code* const gc_StringDescriptors[];

#endif
