#include "precomp.h"

// Word-size == 16, no burst-write
static code const BYTE gs_bFlashLoader16[] =
{
    0x00,               //          nop
    0x00,               //          nop
    0xE5, 0xBE,         //  loop1:  mov a, sleep
    0x30, 0xE5, 0xFB,   //          jnb acc.5, loop1
    0x75, 0xAE, 0x02,   //          mov fctl, #002h
    0xE6,               //  loop2:  mov a, @r0
    0xF5, 0xAF,         //          mov fwdata, a
    0x08,               //          inc r0
    0xE6,               //          mov a, @r0
    0xF5, 0xAF,         //          mov fwdata, a
    0x08,               //          inc r0
    0xE5, 0xAE,         //  loop3:  mov a, fctl
    0x20, 0xE6, 0xFB,   //          jb acc.6, loop3
    0xD9, 0xF1,         //          djnz r1, loop2
    0x75, 0xAE, 0x00,   //          mov fctl, #000h
    0xA5                //          trap
};

// Word-size == 32, burst-write
static code const BYTE gs_dmacfg32b[16] =
{
    // Debug Interface -> Buffer
    0x62,   // src[15:8]    - HIBYTE(DBGDATA)
    0x60,   // src[7:0]     - LOBYTE(DBGDATA)
    0x00,   // dest[15:8]   - HIBYTE(sizeof(gs_dmacfg32b)
    0x10,   // dest[7:0]    - LOBYTE(sizeof(gs_dmacfg32b))
    0x00,   // len[12:8]    - HIBYTE(sizeof(REPORT4.bData) + 6)
    0x44,   // len[7:0]     - LOBYTE(sizeof(REPORT4.bData) + 6)
    0x1F,   // trig[4:0]    - DBG_BW
    0x11,   // destinc[1:0] - 1 Byte
    // Buffer 0 -> Flash controller (Channel 3)
    0x00,   // src[15:8]    - HIBYTE(sizeof(gs_dmacfg32b)
    0x10,   // src[7:0]     - LOBYTE(sizeof(gs_dmacfg32b))
    0x62,   // src[15:8]    - HIBYTE(FWDATA)
    0x73,   // src[7:0]     - LOBYTE(FWDATA)
    0x00,   // len[12:8]    - HIBYTE(sizeof(REPORT4.bData) + 6)
    0x44,   // len[7:0]     - LOBYTE(sizeof(REPORT4.bData) + 6)
    0x12,   // trig[4:0]    - FLASH
    0x42,   // srcinc[1:0]  - 1 Byte
};

static xdata void (code * pfnRestore)(void);

static xdata union
{
    BYTE w16[sizeof(gs_bFlashLoader16) + (62 + 2) + 6];
    BYTE w32b[sizeof(gs_dmacfg32b) + (62 + 6) + 5];
} gs_PreservedMemory;

void Memory_ReadData(unsigned char address, unsigned char len, unsigned char* pBuffer)
{
    BYTE acc;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop

    while(len--)
    {
        if(address == 0xE0)
            *pBuffer = acc;
        else
            DbgItf_DebugInstr(2, 0xE5, address, 0x00, pBuffer); // mov A, address

        ++pBuffer;
        ++address;
    }

    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &acc);                // mov A, #acc
}

void Memory_ReadIData(unsigned char address, unsigned char len, unsigned char* pBuffer)
{
    BYTE acc, r0, addressOfR0, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(1, 0xE8, 0x00, 0x00, &r0);                // mov a, r0
    DbgItf_DebugInstr(2, 0xE5, 0xD0, 0x00, &addressOfR0);       // mov a, psw
    addressOfR0 &= 0x18;

    DbgItf_DebugInstr(2, 0x78, address, 0x00, &tmp);            // mov R0, #address

    while(len--)
    {
        if(address == addressOfR0)
            *pBuffer = r0;
        else
            DbgItf_DebugInstr(1, 0xE6, 0x00, 0x00, pBuffer);    // mov A, address

        ++pBuffer;
        ++address;
        DbgItf_DebugInstr(1, 0x08, 0x00, 0x00, &tmp);           // inc r0
    }

    DbgItf_DebugInstr(2, 0x78, r0, 0x00, &tmp);                 // mov R0, #r0
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

static void Memory_ReadCode_Flat(unsigned short address, unsigned char len, unsigned char* pBuffer)
{
    BYTE acc, dpl, dph, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(2, 0xE5, 0x82, 0x00, &dpl);               // mov a, dpl
    DbgItf_DebugInstr(2, 0xE5, 0x83, 0x00, &dph);               // mov a, dph
    DbgItf_DebugInstr(3, 0x90, HIBYTE(address), LOBYTE(address), &tmp);// mov dptr, #address

    while(len--)
    {
        DbgItf_DebugInstr(1, 0xE4, 0x00, 0x00, &tmp);           // clr a
        DbgItf_DebugInstr(1, 0x93, 0x00, 0x00, pBuffer++);      // movc a, @a+dptr
        DbgItf_DebugInstr(1, 0xA3, 0x00, 0x00, &tmp);           // inc dptr
    }

    DbgItf_DebugInstr(3, 0x90, dph, dpl, &tmp);                 // mov dptr, #(dph * 256 + dpl)
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

static void Memory_ReadCode_Banked(unsigned long address, unsigned char len, unsigned char* pBuffer)
{
    BYTE acc, dpl, dph, memctr, fmap, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(2, 0xE5, 0x82, 0x00, &dpl);               // mov a, dpl
    DbgItf_DebugInstr(2, 0xE5, 0x83, 0x00, &dph);               // mov a, dph
    DbgItf_DebugInstr(2, 0xE5, 0xC7, 0x00, &memctr);            // mov A, memctr
    DbgItf_DebugInstr(2, 0xE5, 0x9F, 0x00, &fmap);              // mov a, fmap
    DbgItf_DebugInstr(3, 0x90, HIBYTE(address) | 0x80, LOBYTE(address), &tmp);// mov dptr, #address
    DbgItf_DebugInstr(3, 0x75, 0xC7, 0x00, &tmp);               // mov memctr, #000h
    DbgItf_DebugInstr(3, 0x75, 0x9F, (GetHiword(address) << 1) + ((LOWORD(address) & 0x8000) ? 1 : 0), &tmp); // mov fmap, #bank

    while(len--)
    {
        DbgItf_DebugInstr(1, 0xE4, 0x00, 0x00, &tmp);           // clr a
        DbgItf_DebugInstr(1, 0x93, 0x00, 0x00, pBuffer++);      // movc a, @a+dptr
        DbgItf_DebugInstr(1, 0xA3, 0x00, 0x00, &tmp);           // inc dptr
    }

    DbgItf_DebugInstr(3, 0x90, dph, dpl, &tmp);                 // mov dptr, #(dph * 256 + dpl)
    DbgItf_DebugInstr(3, 0x75, 0xC7, memctr, &tmp);             // mov memctr, #memctr
    DbgItf_DebugInstr(3, 0x75, 0x9F, fmap, &tmp);               // mov fmap, #fmap
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

void Memory_ReadCode(unsigned long address, unsigned char len, unsigned char* pBuffer)
{
    idata BYTE len2;

    if(DbgItf_GetChipInfo()->bFlashWordSize <= 2)
    {
        Memory_ReadCode_Flat((unsigned short)address, len, pBuffer);
    }
    else if((LOWORD(address) & 0x8000) == ((LOWORD(address) + (len - 1)) & 0x8000))
    {
        Memory_ReadCode_Banked(address, len, pBuffer);
    }
    else
    {
        len2 = ((LOWORD(address) + len) & 0x7FFF);
        len -= len2;
        Memory_ReadCode_Banked(address, len, pBuffer);
        Memory_ReadCode_Banked(address + len, len2, pBuffer + len);
    }
}

void Memory_ReadXData(unsigned short address, unsigned char len, unsigned char* pBuffer)
{
    BYTE acc, dpl, dph, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(2, 0xE5, 0x82, 0x00, &dpl);               // mov a, dpl
    DbgItf_DebugInstr(2, 0xE5, 0x83, 0x00, &dph);               // mov a, dph
    DbgItf_DebugInstr(3, 0x90, HIBYTE(address), LOBYTE(address), &tmp);// mov dptr, #address

    while(len--)
    {
        DbgItf_DebugInstr(1, 0xE0, 0x00, 0x00, pBuffer++);      // movx a, @dptr
        DbgItf_DebugInstr(1, 0xA3, 0x00, 0x00, &tmp);           // inc dptr
    }

    DbgItf_DebugInstr(3, 0x90, dph, dpl, &tmp);                 // mov dptr, #(dph * 256 + dpl)
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

void Memory_WriteData(unsigned char address, unsigned char len, const unsigned char* pBuffer)
{
    BYTE tmp;

    while(len--)
        DbgItf_DebugInstr(3, 0x75, address++, *pBuffer++, &tmp);// mov direct, #data
}

void Memory_WriteIData(unsigned char address, unsigned char len, const unsigned char* pBuffer)
{
    BYTE tmp, acc, r0, addressOfR0;

    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(1, 0xE8, 0x00, 0x00, &r0);                // mov a, r0
    DbgItf_DebugInstr(2, 0xE5, 0xD0, 0x00, &addressOfR0);       // mov a, psw
    addressOfR0 &= 0x18;

    DbgItf_DebugInstr(2, 0x78, address, 0x00, &tmp);            // mov R0, #address

    while(len--)
    {
        if(address == addressOfR0)
            r0 = *pBuffer;
        else
            DbgItf_DebugInstr(2, 0x76, *pBuffer, 0x00, &tmp);   // mov @R0, #data

        ++pBuffer;
        ++address;
        DbgItf_DebugInstr(1, 0x08, 0x00, 0x00, &tmp);           // inc r0
    }

    DbgItf_DebugInstr(2, 0x78, r0, 0x00, &tmp);                 // mov R0, #r0
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

void Memory_WriteXData(unsigned short address, unsigned char len, const unsigned char* pBuffer)
{
    BYTE acc, dpl, dph, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop
    DbgItf_DebugInstr(2, 0xE5, 0x82, 0x00, &dpl);               // mov a, dpl
    DbgItf_DebugInstr(2, 0xE5, 0x83, 0x00, &dph);               // mov a, dph
    DbgItf_DebugInstr(3, 0x90, HIBYTE(address), LOBYTE(address), &tmp);// mov dptr, #address

    while(len--)
    {
        DbgItf_DebugInstr(2, 0x74, *pBuffer++, 0x00, &tmp);     // mov a, #data
        DbgItf_DebugInstr(1, 0xF0, 0x00, 0x00, &tmp);           // movx @dptr, a
        DbgItf_DebugInstr(1, 0xA3, 0x00, 0x00, &tmp);           // inc dptr
    }

    DbgItf_DebugInstr(3, 0x90, dph, dpl, &tmp);                 // mov dptr, #(dph * 256 + dpl)
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

static void Memory_WriteCode16(unsigned long address, unsigned char len, const unsigned char* pBuffer)
{
    BYTE acc, buffer_address, offset, cnt, tmp;
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop

    if(0 != (offset = (address & 1)))
        DbgItf_DebugInstr(3, 0x75, SIZEOF_FLASHLOADER16, 0xFF, &tmp);

    buffer_address = SIZEOF_FLASHLOADER16 + offset;

    for(cnt = len; cnt; --cnt)
        DbgItf_DebugInstr(3, 0x75, buffer_address++, *pBuffer++, &tmp);

    if((address + len) & 1)
    {
        ++len;
        DbgItf_DebugInstr(3, 0x75, buffer_address, 0xFF, &tmp);
    }

    address >>= 1;

    DbgItf_DebugInstr(3, 0x75, 0xAD, HIBYTE(address), &tmp);    // mov faddrh, #hi(address)
    DbgItf_DebugInstr(3, 0x75, 0xAC, LOBYTE(address), &tmp);    // mov faddrl, #lo(address)
    DbgItf_DebugInstr(2, 0x78, SIZEOF_FLASHLOADER16, 0x00, &tmp); // mov r0, #data
    DbgItf_DebugInstr(2, 0x79, (len + offset) >> 1, 0x00, &tmp);// mov r1, #data
    DbgItf_DebugInstr(3, 0x02, 0xFF, 0x02, &tmp);               // ljmp flash_loader
    DbgItf_Resume();

    for(address = 0xFFFF; address; --address)
    {
        if(!DbgItf_ReadDebugStatus(&tmp))
            break;

        if(tmp & CPU_HALTED)
            break;
    }

    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

static void Memory_WriteCode32b(unsigned long address, unsigned char len, const unsigned char* pBuffer)
{
    xdata BYTE buffer[62 + 6];
    xdata BYTE xdata * pSrc = buffer;
    xdata BYTE acc, tmp = (BYTE)address & 3;
    xdata unsigned short faddr;

    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);               // nop

    address >>= (DbgItf_GetChipInfo()->bFlashWordSize >> 1);
    faddr = IMPL_TO_LENDIAN((unsigned short)address);
    Memory_WriteXData(0x6271, 2, (BYTE*)&faddr);                // mov FADDRL, LOW faddr; mov FADDRH, HIGH faddr

    memset(buffer, -1, tmp);
    pSrc += tmp;
    memcpy(pSrc, pBuffer, len);
    memset(pSrc + len, -1, sizeof(buffer) - len - tmp);
    DbgItf_DebugInstr(3, 0x75, 0xD6, 0x02, &tmp);               // mov DMAARM, #002h
    DbgItf_BurstWrite(sizeof(buffer), buffer);
    DbgItf_DebugInstr(3, 0x75, 0xD6, 0x04, &tmp);               // mov DMAARM, #004h
    tmp = 0x06;
    Memory_WriteXData(0x6270, 1, &tmp);                         // mov FCTL, #006h

    for(tmp = 0xFF; tmp; --tmp)
    {
        Memory_ReadXData(0x6270, 1, &tmp);

        if(!(tmp & 0x80))
            break;
    }

    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);                // mov A, #acc
}

void Memory_WriteCode(unsigned long address, unsigned char len, const unsigned char* pBuffer)
{
    CHIPINFO code* pci = DbgItf_GetChipInfo();

    if(pci->bFlashWordSize == 2)
    {
        Memory_WriteCode16(address, len, pBuffer);
    }
    else if(!pci->bFWT)
    {
        Memory_WriteCode32b(address, len, pBuffer);
    }
    else
    {
        // CC2430 and CC2431 are not yet supported
    }
}

static BOOL Memory_WriteLockBits16(const BYTE* pLockBits)
{
    xdata BYTE cfg = DbgItf_GetDebugConfig();

    if(!DbgItf_WriteDebugConfig(cfg | 0x01))
        return(FALSE);

    Memory_WriteCode16(0x0000, 1, pLockBits);
    return(DbgItf_WriteDebugConfig(cfg));
}

BOOL Memory_WriteLockBits(const BYTE* pLockBits)
{
    CHIPINFO code* pci = DbgItf_GetChipInfo();

    if(pci->bFlashWordSize == 2)
    {
        return(Memory_WriteLockBits16(pLockBits));
    }
    else if(!pci->bFWT)
    {
    }
    else
    {
        // CC2430 and CC2431 are not yet supported
    }

    return(FALSE);
}

BOOL Memory_ReadLockBits16(BYTE* pLockBits)
{
    xdata BYTE cfg = DbgItf_GetDebugConfig();

    if(!DbgItf_WriteDebugConfig(cfg | 0x01))
        return(FALSE);

    Memory_ReadCode(0x0000, 1, pLockBits);
    return(DbgItf_WriteDebugConfig(cfg));
}

BOOL Memory_ReadLockBits(BYTE* pLockBits)
{
    CHIPINFO code* pci = DbgItf_GetChipInfo();

    if(pci->bFlashWordSize == 2)
    {
        return(Memory_ReadLockBits16(pLockBits));
    }
    else if(!pci->bFWT)
    {
    }
    else
    {
        // CC2430 and CC2431 are not yet supported
    }

    return(FALSE);
}

static BOOL IsTargetHalted(void)
{
    BYTE bStatus;

    if(!DbgItf_ReadDebugStatus(&bStatus))
        return(FALSE);

    return(bStatus & CPU_HALTED);
}

static BOOL Memory_Preserve16(void)
{
    BYTE acc, tmp;

    if(!IsTargetHalted())
        return(FALSE);

    Memory_ReadData(0, sizeof(gs_PreservedMemory.w16) - 6, gs_PreservedMemory.w16);
    Memory_WriteData(0xF002, sizeof(gs_bFlashLoader16) - 2, &gs_bFlashLoader16[2]);
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);   // nop
    DbgItf_DebugInstr(2, 0xE5, 0xA8, 0x00, &gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 1]); //mov a, ien0
    DbgItf_DebugInstr(2, 0xE5, 0xAB, 0x00, &gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 2]); //mov a, fwt
    DbgItf_DebugInstr(2, 0xE5, 0xC6, 0x00, &gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 3]); //mov a, clkcon
    DbgItf_DebugInstr(2, 0xE5, 0xD0, 0x00, &gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 4]); //mov a, psw
    DbgItf_GetPc((unsigned int*)&gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 6]);
    DbgItf_DebugInstr(3, 0x75, 0xA8, 0x00, &tmp);   // mov ien0, #000h
    DbgItf_DebugInstr(3, 0x75, 0xC6, 0xF9, &tmp);   // mov clkcon, #0F9h
    DbgItf_DebugInstr(3, 0x75, 0xD0, 0x00, &tmp);   // mov psw, #000h
    DbgItf_DebugInstr(3, 0x75, 0xAB, DbgItf_GetChipInfo()->bFWT, &tmp); // mov fwt, #xx
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);    // mov a, #data
    return(TRUE);
}

static void Memory_Restore16(void)
{
    BYTE tmp;
    unsigned int pc;

    Memory_WriteData(0, sizeof(gs_PreservedMemory.w16) - 6, gs_PreservedMemory.w16);
    DbgItf_DebugInstr(3, 0x75, 0xD0, gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 4], &tmp); // mov psw, #data
    DbgItf_DebugInstr(3, 0x75, 0xC6, gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 3], &tmp); // mov clkcon, #data
    DbgItf_DebugInstr(3, 0x75, 0xAB, gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 2], &tmp); // mov fwt, #data
    DbgItf_DebugInstr(3, 0x75, 0xA8, gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 1], &tmp); // mov ien0, #data
    pc = *(unsigned int*)&gs_PreservedMemory.w16[_countof(gs_PreservedMemory.w16) - 6];
    DbgItf_DebugInstr(3, 0x02, LOBYTE(pc), HIBYTE(pc), &tmp);   // ljmp addr16
}

static BOOL Memory_Preserve32b(void)
{
    BYTE acc, tmp, cnt = 0xFF;

    if(!IsTargetHalted())
        return(FALSE);

    Memory_ReadXData(0, sizeof(gs_PreservedMemory.w32b) - 5, gs_PreservedMemory.w32b);
    Memory_WriteXData(0, sizeof(gs_dmacfg32b), gs_dmacfg32b);
    DbgItf_ReadDebugConfig(&gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 1]);
    DbgItf_WriteDebugConfig(SoftPowerMode | TimerSuspend);
    DbgItf_DebugInstr(1, 0x00, 0x00, 0x00, &acc);       // nop
    DbgItf_DebugInstr(2, 0xE5, 0xD2, 0x00, &gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 2]); //mov a, DMA1CFGH
    DbgItf_DebugInstr(2, 0xE5, 0xD3, 0x00, &gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 3]); //mov a, DMA1CFGL
    DbgItf_DebugInstr(2, 0xE5, 0xD6, 0x00, &gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 4]); //mov a, DMAARM
    DbgItf_DebugInstr(2, 0xE5, 0xC6, 0x00, &gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 5]); //mov a, CLKCONCMD
    DbgItf_DebugInstr(3, 0x75, 0xC6, 0x80, &tmp);       // mov CLKCONCMD, #080h
    tmp = 0;

    while(DbgItf_DebugInstr(2, 0xE5, 0x9E, 0x00, &tmp)) // mov A, CLKCONSTA
    {
        if(tmp == 0x80)
            break;

        if(!--cnt)
            break;
    }

    DbgItf_DebugInstr(3, 0x75, 0xD6, 0x86, &tmp);       // mov DMAARM, #081h
    DbgItf_DebugInstr(3, 0x75, 0xD2, 0x00, &tmp);       // mov DMA1CFGL, #LOBYTE(&gs_dmacfg32b[0])
    DbgItf_DebugInstr(3, 0x75, 0xD3, 0x00, &tmp);       // mov DMA1CFGH, #HIBYTE(&gs_dmacfg32b[0])
    DbgItf_DebugInstr(2, 0x74, acc, 0x00, &tmp);        // mov a, #data
    return(TRUE);
}

static void Memory_Restore32b(void)
{
    BYTE tmp;

    DbgItf_DebugInstr(3, 0x75, 0xD6, 0x86, &tmp);       // mov DMAARM, #086h
    Memory_WriteXData(0, sizeof(gs_PreservedMemory.w32b) - 5, gs_PreservedMemory.w32b);
    DbgItf_DebugInstr(3, 0x75, 0xD2, gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 2], &tmp); //mov DMA1CFGH, #data
    DbgItf_DebugInstr(3, 0x75, 0xD3, gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 3], &tmp); //mov DMA1CFGL, #data
    DbgItf_DebugInstr(3, 0x75, 0xD6, gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 4], &tmp); //mov DMAARM, #data
    DbgItf_DebugInstr(3, 0x75, 0xC6, gs_PreservedMemory.w32b[_countof(gs_PreservedMemory.w32b) - 5], &tmp); //mov CLKCONCMD, #data
}

BOOL Memory_Preserve(void)
{
    CHIPINFO code* pci = DbgItf_GetChipInfo();

    if(pci->bFlashWordSize == 2)
    {
        pfnRestore = Memory_Restore16;
        return(Memory_Preserve16());
    }

    if(pci->bFWT)
    {
        // CC2430 and CC2431 are not yet supported
        return(FALSE);
    }

    pfnRestore = Memory_Restore32b;
    return(Memory_Preserve32b());
}

void Memory_Restore(void)
{
    if(pfnRestore)
    {
        pfnRestore();
        pfnRestore = NULL;
    }
}
