#ifndef USB_H
#define USB_H

// Descriptor Types
#define USB_DT_DEVICE                           0x01
#define USB_DT_CONFIGURATION                    0x02
#define USB_DT_STRING                           0x03
#define USB_DT_INTERFACE                        0x04
#define USB_DT_ENDPOINT                         0x05
#define USB_DT_DEVICE_QUALIFIER                 0x06
#define USB_DT_OTHER_SPEED_CONFIGURATION        0x07
#define USB_DT_INTERFACE_POWER                  0x08
#define USB_DT_OTG                              0x09
#define USB_DT_DEBUG                            0x0A
#define USB_DT_INTERFACE_ASSOCIATION            0x0B
#define USB_DT_SECURITY                         0x0C
#define USB_DT_KEY                              0x0D
#define USB_DT_ENCRYPTION_TYPE                  0x0E
#define USB_DT_BOS                              0x0F
#define USB_DT_DEVICE_CAPABILITY                0x10
#define USB_DT_WIRELESS_ENDPOINT_COMPANION      0x11
#define USB_DT_HID                              0x21
#define USB_DT_DFU_FUNCTIONAL                   0x21
#define USB_DT_REPORT                           0x22
#define USB_DT_PHYSICAL                         0x23
#define USB_DT_CS_INTERFACE                     0x24
#define USB_DT_CS_ENDPOINT                      0x25

// Descriptor SubTypes
#define USB_DT_SUBTYPE_HEADER                   0x00
#define USB_DT_SUBTYPE_CALLMANAGEMENT           0x01
#define USB_DT_SUBTYPE_ACM                      0x02
#define USB_DT_SUBTYPE_DLM                      0x03
#define USB_DT_SUBTYPE_UNION                    0x06
#define USB_DT_SUBTYPE_DMM                      0x14
#define USB_DT_SUBTYPE_OBEX                     0x15

// Class Codes
#define USB_CLASS_AUDIO                         0x01
#define USB_CLASS_COMMUNICATIONDEVICE           0x02
#define USB_CLASS_HID                           0x03
#define USB_CLASS_PHYSICAL                      0x05
#define USB_CLASS_IMAGE                         0x06
#define USB_CLASS_PRINTER                       0x07
#define USB_CLASS_MASSSTORAGE                   0x08
#define USB_CLASS_HUB                           0x09
#define USB_CLASS_DATA                          0x0A
#define USB_CLASS_SMARTCARD                     0x0B
#define USB_CLASS_CONTENTSECURITY               0x0D
#define USB_CLASS_VIDEO                         0x0E
#define USB_CLASS_PERSONALHEALTHCARE            0x0F
#define USB_CLASS_DIAGNOSTIC                    0xDC
#define USB_CLASS_WIRELESSCONTROLLER            0xE0
#define USB_CLASS_MISCELLANEOUS                 0xEF
#define USB_CLASS_APPSPECIFIC                   0xFE
#define USB_CLASS_VENDORSPECIFIC                0xFF

// SubClass Codes
#define USB_SUBCLASS_UNDEFINED                  0x00
#define USB_SUBCLASS_DFU                        0x01
#define USB_SUBCLASS_DLM                        0x01
#define USB_SUBCLASS_AUDIOCONTROL               0x01
#define USB_SUBCLASS_ACM                        0x02
#define USB_SUBCLASS_AUDIOSTREAMING             0x02
#define USB_SUBCLASS_COMMON                     0x02
#define USB_SUBCLASS_MIDISTREAMING              0x03
#define USB_SUBCLASS_TCM                        0x03

// Protocol Codes
#define USB_PROTOCOL_IAD                        0x01

// Request Types
#define USB_REQTYPE_STANDARD                    0x00
#define USB_REQTYPE_CLASS                       0x20
#define USB_REQTYPE_VENDOR                      0x40
#define USB_REQTYPE_RESERVED                    0x60
#define USB_REQTYPE_MASK                        0x60

// Request Codes (Standard)
#define USB_STDREQ_GET_STATUS                   0x00
#define USB_STDREQ_CLEAR_FEATURE                0x01
#define USB_STDREQ_SET_FEATURE                  0x03
#define USB_STDREQ_SET_ADDRESS                  0x05
#define USB_STDREQ_GET_DESCRIPTOR               0x06
#define USB_STDREQ_SET_DESCRIPTOR               0x07
#define USB_STDREQ_GET_CONFIGURATION            0x08
#define USB_STDREQ_SET_CONFIGURATION            0x09
#define USB_STDREQ_GET_INTERFACE                0x0A
#define USB_STDREQ_SET_INTERFACE                0x0B
#define USB_STDREQ_SYNCH_FRAME                  0x0C

// Request Codes (HID)
#define USB_HIDREQ_GET_REPORT                   0x01
#define USB_HIDREQ_GET_IDLE                     0x02
#define USB_HIDREQ_GET_PROTOCOL                 0x03
#define USB_HIDREQ_SET_REPORT                   0x09
#define USB_HIDREQ_SET_IDLE                     0x0A
#define USB_HIDREQ_SET_PROTOCOL                 0x0B

// Request Codes (CDC)
#define USB_CDCREQ_SEND_ENCAPSULATED_COMMAND    0x00
#define USB_CDCREQ_GET_ENCAPSULATED_RESPONSE    0x01
#define USB_CDCREQ_SET_COMM_FEATURE             0x02
#define USB_CDCREQ_GET_COMM_FEATURE             0x03
#define USB_CDCREQ_CLEAR_COMM_FEATURE           0x04
#define USB_CDCREQ_SET_AUX_LINE_STATE           0x10
#define USB_CDCREQ_SET_HOOK_STATE               0x11
#define USB_CDCREQ_PULSE_SETUP                  0x12
#define USB_CDCREQ_SEND_PULSE                   0x13
#define USB_CDCREQ_SET_PULSE_TIME               0x14
#define USB_CDCREQ_RING_AUX_JACK                0x15
#define USB_CDCREQ_SET_LINE_CODING              0x20
#define USB_CDCREQ_GET_LINE_CODING              0x21
#define USB_CDCREQ_SET_CONTROL_LINE_STATE       0x22
#define USB_CDCREQ_SEND_BREAK                   0x23
#define USB_CDCREQ_SET_RINGER_PARMS             0x30
#define USB_CDCREQ_GET_RINGER_PARMS             0x31
#define USB_CDCREQ_SET_OPERATION_PARMS          0x32
#define USB_CDCREQ_GET_OPERATION_PARMS          0x33
#define USB_CDCREQ_SET_LINE_PARMS               0x34
#define USB_CDCREQ_GET_LINE_PARMS               0x35
#define USB_CDCREQ_DIAL_DIGITS                  0x36

// Report Types
#define USB_REPTYPE_INPUT                       0x01
#define USB_REPTYPE_OUTPUT                      0x02
#define USB_REPTYPE_FEATURE                     0x03

// Device States
#define USB_STATE_ATTACHED                      0x01
#define USB_STATE_POWERED                       0x02
#define USB_STATE_DEFAULT                       0x03
#define USB_STATE_ADDRESS                       0x04
#define USB_STATE_CONFIGURED                    0x05
#define USB_STATE_SUSPENDED                     0x06

// Standard Feature Selectors
#define USB_FEATURE_ENDPOINT_HALT               0x00
#define USB_FEATURE_DEVICE_REMOTE_WAKEUP        0x01
#define USB_FEATURE_TEST_MODE                   0x02

// Feature Selector
#define USB_ENDPOINT_HALT                       0x00
#define USB_DEVICE_REMOTE_WAKEUP                0x01
#define USB_TEST_MODE                           0x02

typedef struct
{
    BYTE    bmRequestType;
    BYTE    bRequest;
    BYTE    wValueL;
    BYTE    wValueH;
    BYTE    wIndexL;
    BYTE    wIndexH;
    BYTE    wLengthL;
    BYTE    wLengthH;
} SETUP_PACKET;

typedef struct
{
    BYTE    bChipId;
    BYTE    bVersion;
    BYTE    bStatus;
    BYTE    bTargetVoltageL;
    BYTE    bTargetVoltageH;
} REPORT1;

#define CMDID_DEBUGMODE         0x01    /* param1 = 0xFFFF -> Enter, param1 = 0x0000 -> Leave                                                               */
#define CMDID_DEBUGCONFIG       0x02    /* param1 = 0x00xx -> xx = config data                                                                              */
#define CMDID_RESUME            0x03    /* ** no parameters **                                                                                              */
#define CMDID_HALT              0x04    /* ** no parameters **                                                                                              */
#define CMDID_CHIP_ERASE        0x05    /* ** no parameters                                                                                                 */
#define CMDID_DEBUG_INSTR       0x06    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3                                                  */
#define CMDID_STEP_REPLACE      0x07    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3, param2 high -> number of bytes used             */
#define CMDID_STEP_INSTR        0x08    /* ** no parameters **                                                                                              */
#define CMDID_SET_HW_BRKPNT     0x09    /* param1 -> Breakpoint address[0..15], param2 low -> Breakpoint address[16..23], param2 high -> Breakpoint Info    */

typedef struct
{
    BYTE    bCmd;
    WORD    param1;
    WORD    param2;
} REPORT2;

typedef struct
{
    BYTE            bSegment;
    unsigned long   uAddress;
    unsigned long   uLen;
    BYTE            blRead;
} REPORT3;

typedef struct
{
    BYTE    bLength;
    BYTE    bData[62];
} REPORT4;

typedef struct
{
    BYTE    bInDbgMode;
    BYTE    bConfig;
    BYTE    bStatus;
    BYTE    bResult;
} REPORT5;

typedef struct
{
    unsigned short pc;
} REPORT6;

typedef struct
{
    BYTE    bLockBits[1];
    BYTE    bReserved[15];
} REPORT7;

typedef struct
{
    BYTE    bBank;
} REPORT8;

typedef struct
{
    BYTE    bReportID;
    union
    {
        REPORT1 report1;
        REPORT2 report2;
        REPORT3 report3;
        REPORT4 report4;
        REPORT5 report5;
        REPORT6 report6;
        REPORT7 report7;
        REPORT8 report8;
    } dat;
} IO_REPORT;

#endif
