#include "precomp.h"

extern HWND hMfrm;

CAgdiProgress::CAgdiProgress(long lAlreadyDone, long lActualSize, pCBF pCbFunc)
    : m_lAlreadyDone(lAlreadyDone), m_lActualSize(lActualSize), m_pCbFunc(pCbFunc), m_pTaskbarList(NULL)
{
    if(FAILED(CoCreateInstance(CLSID_TaskbarList, NULL, CLSCTX_SERVER, __uuidof(ITaskbarList3), (void**)&m_pTaskbarList)))
        m_pTaskbarList = NULL;
}

CAgdiProgress::~CAgdiProgress()
{
    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NOPROGRESS);
        m_pTaskbarList->Release();
    }
}

STDMETHODIMP
CAgdiProgress::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04Progress))
            *ppvObject = static_cast<ISmartICE04Progress*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CAgdiProgress::AddRef()
{
    return(2);
}

STDMETHODIMP_(ULONG)
CAgdiProgress::Release()
{
    return(1);
}

STDMETHODIMP
CAgdiProgress::Start()
{
    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NORMAL);
        m_pTaskbarList->SetProgressValue(hMfrm, 0, 100);
    }

    return(S_OK);
}

STDMETHODIMP
CAgdiProgress::Stop()
{
    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressState(hMfrm, TBPF_NOPROGRESS);

    return(S_OK);
}

STDMETHODIMP
CAgdiProgress::SetProgress(long lCompleted, long lTotal)
{
    OIL ProgressBar;
    ProgressBar.Job   = PROGRESS_SETPOS;
    ProgressBar.pos   = min(100, (lCompleted + m_lAlreadyDone) * 100 / max(m_lActualSize, lTotal));
    ProgressBar.low   = 0;
    ProgressBar.hig   = 100;
    ProgressBar.label = NULL;
    pCbFunc(AG_CB_PROGRESS, &ProgressBar);

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressValue(hMfrm, static_cast<unsigned long>(lCompleted), static_cast<unsigned long>(lTotal));

    return(S_OK);
}

STDMETHODIMP
CAgdiProgress::get_Canceled(VARIANT_BOOL* pblCanceled)
{
    if(pblCanceled)
        *pblCanceled = VARIANT_FALSE;

    return(S_OK);
}
