#include "precomp.h"

HRESULT
CBinaryFile::Load(LPCWSTR filename, SAFEARRAY **ppsa)
{
    ULARGE_INTEGER  siz;
    SAFEARRAYBOUND  sab;
    HRESULT         hr;
    HANDLE          hFile;
    DWORD           dw;
    void*           pData;

    if(INVALID_HANDLE_VALUE == (hFile = CreateFileW(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL)))
        return(HRESULT_FROM_WIN32(GetLastError()));

    if(INVALID_FILE_SIZE == (siz.LowPart = GetFileSize(hFile, &siz.HighPart)))
    {
        hr = HRESULT_FROM_WIN32(GetLastError());
    }
    else if(siz.HighPart)
    {
        hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
    }
    else
    {
        hr = S_OK;
    }

    if(SUCCEEDED(hr))
    {
        sab.lLbound = 0;
        sab.cElements = siz.LowPart;

        if(NULL == (*ppsa = SafeArrayCreate(VT_UI1, 1, &sab)))
        {
            hr = E_OUTOFMEMORY;
        }
        else
        {
            if(SUCCEEDED((hr = SafeArrayAccessData(*ppsa, &pData))))
            {
                if(!ReadFile(hFile, pData, siz.LowPart, &dw, NULL))
                {
                    hr = HRESULT_FROM_WIN32(GetLastError());
                }
                else if(dw != siz.LowPart)
                {
                    hr = E_FAIL;
                }

                SafeArrayUnaccessData(*ppsa);
            }

            if(FAILED(hr))
            {
                SafeArrayDestroy(*ppsa);
                *ppsa = NULL;
            }
        }
    }

    CloseHandle(hFile);
    return(hr);
}