#include "precomp.h"

CClassFactory::CClassFactory() throw()
    : m_lRefCount(1)
{
    ::LockServer();
}

CClassFactory::~CClassFactory() throw()
{
    ::UnlockServer();
}

STDMETHODIMP
CClassFactory::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IClassFactory))
            *ppvObject = static_cast<IClassFactory*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CClassFactory::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CClassFactory::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CClassFactory::LockServer(THIS_ BOOL fLock)
{
    if(fLock)
        ::LockServer();
    else
        ::UnlockServer();

    return(S_OK);
}
