#include "precomp.h"

CComException::CComException(IDispatch *pSender, DISPID dispid, HRESULT hr /* = E_FAIL */)
    : m_pSender(pSender), m_dispid(dispid), m_hr(hr)
{
    assert(m_pSender);
}

CComException::~CComException()
{
    if(FAILED(m_hr))
        Raise();
}

void
CComException::Raise(void)
{
    IProvideClassInfo*  pPCI        = NULL;
    ICreateErrorInfo*   pCreateErr  = NULL;
    IErrorInfo*         pError      = NULL;
    ITypeInfo*          pInfo       = NULL;
    TYPEATTR*           ptattr      = NULL;
    LPOLESTR            pSource     = NULL;
    LPWSTR              lpErrorMsg;
    DWORD               dwHelpContext;
    DWORD               dwMessageLen = 0;
    TCHAR               szError[1024];
    BSTR                bstrHelpFile;

    if((HRESULT_FACILITY(m_hr) == FACILITY_ITF) && (HRESULT_CODE(m_hr) >= 0x0200))
    {
        CThreadLocale loc(MAKELCID(MAKELANGID(PRIMARYLANGID(CThreadLocale::getLANGID()), SUBLANG_DEFAULT), SORT_DEFAULT));
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_HMODULE, g_hInst, m_hr, 0, szError, _countof(szError), NULL);
    }
    else
    {
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM, NULL, m_hr, 0, szError, _countof(szError), NULL);
    }

    if(!dwMessageLen)
    {
        #ifdef _DLL
          #ifdef UNICODE
            swprintf(szError, _countof(szError) - 1, L"Unknown error: 0x%08X", m_hr);
            szError[_countof(szError) - 1] = L'\0';
          #else
            sprintf(szError, "Unknown error: 0x%08X", m_hr);
          #endif
        #else
          wsprintf(szError, TEXT("Unknown error: 0x%08X"), m_hr);
        #endif
    }

    #ifdef UNICODE
      lpErrorMsg = szError;
    #else
      WCHAR wszError[_countof(szError)];
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szError, -1, wszError, _countof(wszError));
      lpErrorMsg = wszError;
    #endif

    if(SUCCEEDED(CreateErrorInfo(&pCreateErr)))
    {
        pCreateErr->SetDescription(lpErrorMsg);

        if(SUCCEEDED(m_pSender->QueryInterface(IID_IProvideClassInfo, (void**)&pPCI)))
        {
            if(SUCCEEDED(pPCI->GetClassInfo(&pInfo)))
            {
                if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
                {
                    if(SUCCEEDED(ProgIDFromCLSID(ptattr->guid, &pSource)))
                    {
                        pCreateErr->SetSource(pSource);
                        CoTaskMemFree(pSource);
                    }
                    pInfo->ReleaseTypeAttr(ptattr);
                    ptattr = NULL;
                }
                pInfo->Release();
                pInfo = NULL;
            }
            pPCI->Release();
        }

        if(SUCCEEDED(m_pSender->GetTypeInfo(0, CThreadLocale::getLCID(), &pInfo)))
        {
            if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
            {
                pCreateErr->SetGUID(ptattr->guid);
                pInfo->ReleaseTypeAttr(ptattr);
            }

            if(SUCCEEDED(pInfo->GetDocumentation(m_dispid, NULL, NULL, &dwHelpContext, &bstrHelpFile)))
            {
                pCreateErr->SetHelpContext(dwHelpContext);
                pCreateErr->SetHelpFile(bstrHelpFile);
                SysFreeString(bstrHelpFile);
            }

            pInfo->Release();
        }

        if(SUCCEEDED(pCreateErr->QueryInterface(IID_IErrorInfo, (void**)&pError)))
        {
            SetErrorInfo(0, pError);
            pError->Release();
        }

        pCreateErr->Release();
    }
}
