#include "precomp.h"

bool operator==(const CONNECTDATA& cd, DWORD dwCookie) { return(dwCookie == cd.dwCookie); }

CConnectionPoint::CConnectionPoint() throw()
{
}

CConnectionPoint::~CConnectionPoint() throw()
{
    CONNECTDATA cd;

    while(m_Connections.Remove(0, &cd))
        cd.pUnk->Release();
}

STDMETHODIMP
CConnectionPoint::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IConnectionPoint))
            *ppvObject = static_cast<IConnectionPoint*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CConnectionPoint::AddRef()
{
    return(m_pCPC->AddRef());
}

STDMETHODIMP_(ULONG)
CConnectionPoint::Release()
{
    return(m_pCPC->Release());
}

STDMETHODIMP
CConnectionPoint::GetConnectionPointContainer(IConnectionPointContainer** ppCPC)
{
    return(m_pCPC->QueryInterface(IID_IConnectionPointContainer, (void**)ppCPC));
}

STDMETHODIMP
CConnectionPoint::Advise(IUnknown* pUnk, DWORD* pdwCookie)
{
    CONNECTDATA data;
    HRESULT     hr;
    DWORD       dwIndex;
    IID         iid;

    if(IsBadWritePtrAuto(pdwCookie))
        return(E_POINTER);

    *pdwCookie = 0;

    if(FAILED((hr = GetConnectionInterface(&iid))))
        return(hr);

    for(dwIndex = 1; m_Connections.Get(dwIndex - 1, &data); ++dwIndex)
    {
        if(!dwIndex)
            return(CONNECT_E_ADVISELIMIT);

        if(data.dwCookie > dwIndex)
            break;
    }

    data.pUnk = NULL;
    data.dwCookie = dwIndex;

    if(SUCCEEDED((hr = TryConnect(pUnk, iid, (void**)&data.pUnk))))
    {
        if(!m_Connections.Insert(dwIndex - 1, data))
        {
            hr = E_OUTOFMEMORY;
            data.pUnk->Release();
        }
        else
        {
            *pdwCookie = data.dwCookie;
        }
    }

    return(hr);
}

STDMETHODIMP
CConnectionPoint::Unadvise(DWORD dwCookie)
{
    CONNECTDATA cd;
    UINT_PTR    idx;

    if(m_Connections.Find(dwCookie, &cd, &idx))
    {
        m_Connections.Remove(idx);
        cd.pUnk->Release();
        return(S_OK);
    }

    return(CONNECT_E_NOCONNECTION);
}

STDMETHODIMP
CConnectionPoint::EnumConnections(IEnumConnections** ppEnum)
{
    CEnumConnections*   pEnum;
    HRESULT             hr;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumConnections(this)))
        return(E_OUTOFMEMORY);

    hr = pEnum->QueryInterface(IID_IEnumConnections, (void**)ppEnum);
    pEnum->Release();
    return(hr);
}

HRESULT
CConnectionPoint::GetConnectData(UINT_PTR uIndex, CONNECTDATA* pData) throw()
{
    if(!m_Connections.Get(uIndex, pData))
        return(HRESULT_FROM_WIN32(GetLastError()));

    pData->pUnk->AddRef();
    return(S_OK);
}

HRESULT
CConnectionPoint::TryConnect(IUnknown* pUnk, REFIID riid, void** ppvObject) throw()
{
    __try
    {
        if(FAILED(pUnk->QueryInterface(riid, ppvObject)))
            return(CONNECT_E_CANNOTCONNECT);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_POINTER);
    }

    return((*ppvObject) ? S_OK : E_UNEXPECTED);
}