#include "precomp.h"

CDeviceList::CDeviceList() throw()
    : m_hWnd(NULL)
{
    m_Sink.SetParent(this);
    m_Sink.Connect();
}

CDeviceList::~CDeviceList() throw()
{
}

HRESULT
CDeviceList::DoModal(HWND hWndParent)
{
    CDeviceList dlg;
    CActCtx ctx(IDR_MANIFEST);
    INT_PTR n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DEVICES), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CDeviceList::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    HIMAGELIST himl;
    LVCOLUMN lvc;
    RECT rc;
    TCHAR sz[64];
    unsigned idx;
    HWND hWndLV;

    m_hWnd = hWnd;
    hWndLV = GetDlgItem(hWnd, IDC_DEVICES);

    ZeroMemory(&lvc, sizeof(lvc));
    GetClientRect(hWndLV, &rc);
    lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT;
    lvc.fmt = LVCFMT_LEFT;
    lvc.cx = ((rc.right - rc.left) - GetSystemMetrics(SM_CXVSCROLL)) / 4;
    lvc.pszText = sz;

    for(idx = IDS_TARGETVOLTAGE; idx >= IDS_DEVICE; --idx) 
        if(LoadString(g_hInst, idx, sz, _countof(sz)))
            ListView_InsertColumn(hWndLV, 0, &lvc);

    if(NULL != (himl = ImageList_Create(GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), ILC_COLOR32 | ILC_MASK, 1, 63)))
        ListView_SetImageList(hWndLV, himl, LVSIL_SMALL);

    m_Sink.EnumDevices();
    SetWindowTheme(hWndLV, L"Explorer", NULL);
    ListView_SetExtendedListViewStyle(hWndLV, LVS_EX_FULLROWSELECT | LVS_EX_DOUBLEBUFFER);
    return(TRUE);
}

INT_PTR
CDeviceList::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    LVITEMA lvi;

    switch(LOWORD(wParam))
    {
        case IDOK:
            if(!IsWindowEnabled(GetDlgItem(hWnd, IDOK)))
                break;

            if(0 > (lvi.iItem = static_cast<int>(SendDlgItemMessage(hWnd, IDC_DEVICES, LVM_GETNEXTITEM, static_cast<WPARAM>(-1), LVNI_ALL | LVNI_SELECTED))))
                break;

            lvi.iSubItem = 0;
            lvi.cchTextMax = _countof(MonConf.idDevice);
            lvi.pszText = MonConf.idDevice;
            SendDlgItemMessage(hWnd, IDC_DEVICES, LVM_GETITEMTEXTA, static_cast<WPARAM>(lvi.iItem), reinterpret_cast<LPARAM>(&lvi));
        case IDCANCEL:
            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR
CDeviceList::OnWmNotify(HWND hWnd, WPARAM, LPARAM lParam)
{
    NMLISTVIEW* pnmlv = reinterpret_cast<NMLISTVIEW*>(lParam);

    switch(reinterpret_cast<NMHDR*>(lParam)->code)
    {
        case LVN_ITEMCHANGING:
            if((pnmlv->uChanged & LVIF_STATE) && (pnmlv->uOldState & LVIS_SELECTED) && (!(pnmlv->uNewState & LVIS_SELECTED)))
                EnableWindow(GetDlgItem(hWnd, IDOK), FALSE);

            return(TRUE);
        case LVN_ITEMCHANGED:
            if((pnmlv->uChanged & LVIF_STATE) && (!(pnmlv->uOldState & LVIS_SELECTED)) && (pnmlv->uNewState & LVIS_SELECTED))
                EnableWindow(GetDlgItem(hWnd, IDOK), TRUE);

            return(TRUE);
        case LVN_ITEMACTIVATE:
            PostMessage(hWnd, WM_COMMAND, MAKEWPARAM(IDOK, BN_CLICKED), reinterpret_cast<LPARAM>(GetDlgItem(hWnd, IDOK)));
            return(TRUE);
        case LVN_DELETEITEM:
            reinterpret_cast<CSmartICE04EventsSink*>(pnmlv->lParam)->Disconnect();
            reinterpret_cast<CSmartICE04EventsSink*>(pnmlv->lParam)->Release();
            return(TRUE);
        default:
            break;
    }

    return(FALSE);
}

INT_PTR CALLBACK
CDeviceList::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
#pragma warning(push)
#pragma warning(disable: 4820)
    static const struct
    {
        UINT uMsg;
        INT_PTR (CDeviceList::*pfn)(HWND hWnd, WPARAM wParam, LPARAM lParam);
    }
    map[] =
    {
        {   WM_NOTIFY,          &OnWmNotify     },
        {   WM_COMMAND,         &OnWmCommand    }
    };
#pragma warning(pop)

    for(unsigned n = 0; n < _countof(map); ++n)
        if(map[n].uMsg == uMsg)
            return((reinterpret_cast<CDeviceList*>(GetWindowLongPtr(hWnd, DWLP_USER))->*map[n].pfn)(hWnd, wParam, lParam));

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CDeviceList*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}

void
CDeviceList::AddDevice(ISmartICE04* pDevice)
{
    CSmartICE04EventsSink* pSink;
    LVFINDINFO lvfi;
    LVITEM lvi;
    VARIANT_BOOL vbl;
    enum ChipId id;
    long voltage;
    BSTR sernum = NULL;
    HWND hWndLV = GetDlgItem(m_hWnd, IDC_DEVICES);

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    lvfi.flags = LVFI_STRING;
    lvfi.psz = sernum;

    if(0 > ListView_FindItem(hWndLV, -1, &lvfi))
    {
        if(NULL != (pSink = new (std::nothrow) CSmartICE04EventsSink(this)))
        {
            pSink->ConnectTo(pDevice);
            lvi.mask = LVIF_IMAGE | LVIF_PARAM | LVIF_TEXT;
            lvi.iItem = MAXLONG;
            lvi.iSubItem = 0;
            lvi.pszText = sernum;
            lvi.iImage = AddIcon(pDevice);
            lvi.lParam = reinterpret_cast<LPARAM>(pSink);

            if(0 <= ListView_InsertItem(hWndLV, &lvi))
            {
                pDevice->get_IsInDebugMode(&vbl);
                pSink->OnDebugModeChanged(pDevice, vbl);
                pDevice->get_ChipId(&id);
                pSink->OnTargetChanged(pDevice, id);
                pDevice->get_TargetVoltage(&voltage);
                pSink->OnTargetVoltageChanged(pDevice, (voltage >= 1800) ? VARIANT_TRUE : VARIANT_FALSE);
            }
            else
            {
                pSink->Disconnect();
                pSink->Release();
            }
        }
    }

    SysFreeString(sernum);
}

void
CDeviceList::RemoveDevice(ISmartICE04* pDevice)
{
    int iItem;

    if(0 <= (iItem = FindDevice(pDevice)))
        ListView_DeleteItem(GetDlgItem(m_hWnd, IDC_DEVICES), iItem);
}

void
CDeviceList::SetTargetVoltage(ISmartICE04* pDevice, LPCTSTR pszVoltage)
{
    SetSubItemText(pDevice, 3, pszVoltage);
}

void
CDeviceList::SetTarget(ISmartICE04* pDevice, LPCTSTR pszTarget)
{
    SetSubItemText(pDevice, 2, pszTarget);
}

void
CDeviceList::SetDebugMode(ISmartICE04* pDevice, LPCTSTR pszDebugMode)
{
    SetSubItemText(pDevice, 1, pszDebugMode);
}

void
CDeviceList::SetSubItemText(ISmartICE04* pDevice, int iSubItem, LPCTSTR psz)
{
    LVITEM lvi;

    if(0 > (lvi.iItem = FindDevice(pDevice)))
        return;

    lvi.mask = LVIF_TEXT;
    lvi.iSubItem = iSubItem;
    lvi.pszText = const_cast<LPTSTR>(psz);
    ListView_SetItem(GetDlgItem(m_hWnd, IDC_DEVICES), &lvi);
}

int
CDeviceList::FindDevice(ISmartICE04* pDevice) const
{
    LVFINDINFO lvfi;
    BSTR bstr = NULL;
    int idx;

    if(FAILED(pDevice->get_SerialNumber(&bstr)))
        return(-1);

    lvfi.flags = LVFI_STRING;
    lvfi.psz = bstr;
    idx = ListView_FindItem(GetDlgItem(m_hWnd, IDC_DEVICES), -1, &lvfi);
    SysFreeString(bstr);
    return(idx);
}

int
CDeviceList::AddIcon(ISmartICE04* pDevice)
{
    HIMAGELIST  himl;
    IDispatch*  pDispatch;
    IPicture2*  pPicture;
    ICONINDEX   ii;
    HICON       hIcon;
    short       pictype;
    long        id;

    if(FAILED(pDevice->get_ProductId(&id)))
        return(-1);

    if(m_IconIndex.Find(id, &ii))
        return(ii.idx);

    if(NULL == (himl = ListView_GetImageList(GetDlgItem(m_hWnd, IDC_DEVICES), LVSIL_SMALL)))
        return(-1);

    pDispatch = NULL;

    if(FAILED(pDevice->get_SmallIcon(&pDispatch)))
        return(-1);

    ii.id = id;
    ii.idx = -1;

    if(pDispatch)
    {
        pPicture = NULL;

        if(FAILED(pDispatch->QueryInterface(IID_IPicture2, (void**)&pPicture)))
        {
            pPicture = NULL;

            if(FAILED(pDispatch->QueryInterface(IID_IPicture, (void**)&pPicture)))
                pPicture = NULL;
        }

        if(pPicture)
        {
            if(SUCCEEDED(pPicture->get_Type(&pictype)))
            {
                if(pictype == PICTYPE_ICON)
                {
                    hIcon = NULL;

                    if(SUCCEEDED(pPicture->get_Handle(reinterpret_cast<HHANDLE*>(&hIcon))))
                    {
                        if(0 <= (ii.idx = ImageList_AddIcon(himl, hIcon)))
                        {
                            m_IconIndex.Add(ii);
                        }
                    }
                }
            }
            pPicture->Release();
        }
        pDispatch->Release();
    }

    return(ii.idx);
}
