#include "precomp.h"

CDownloadSetup::CDownloadSetup() throw()
{
    ZeroMemory(&m_ptMinTrackSize, sizeof(m_ptMinTrackSize));
}

CDownloadSetup::~CDownloadSetup() throw()
{
}

HRESULT
CDownloadSetup::DoModal(HWND hWndParent)
{
    CDownloadSetup  dlg;
    CActCtx         ctx(IDR_MANIFEST);
    INT_PTR         n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_DOWNLOADSETUP), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CDownloadSetup::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    RECT rc;

    GetWindowRect(hWnd, &rc);
    m_ptMinTrackSize.x = rc.right - rc.left;
    m_ptMinTrackSize.y = rc.bottom - rc.top;
    CheckDlgButton(hWnd, IDC_FLASH_ERASE, static_cast<UINT>((MonConf.Opt & FLASH_ERASE) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_FLASH_PROGRAM, static_cast<UINT>((MonConf.Opt & FLASH_PROGRAM) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_FLASH_VERIFY, static_cast<UINT>((MonConf.Opt & FLASH_VERIFY) ? BST_CHECKED : BST_UNCHECKED));
    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDCANCEL:
            OnWmInitDialog(hWnd, 0, reinterpret_cast<LPARAM>(this));
        case IDOK:
            MonConf.Opt &= ~(FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY);

            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_ERASE))
                MonConf.Opt |= FLASH_ERASE;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_PROGRAM))
                MonConf.Opt |= FLASH_PROGRAM;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_FLASH_VERIFY))
                MonConf.Opt |= FLASH_VERIFY;

            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmGetMinMaxInfo(HWND, WPARAM, LPARAM lParam)
{
    reinterpret_cast<MINMAXINFO*>(lParam)->ptMinTrackSize = m_ptMinTrackSize;
    return(TRUE);
}

INT_PTR
CDownloadSetup::OnWmSize(HWND hWnd, WPARAM, LPARAM lParam)
{
    POINT   pt;
    HWND    hCtl;
    RECT    rc = { 0 };
    int     nID;

    for(nID = IDC_FLASH_VERIFY; nID >= IDC_DOWNLOAD_SETUP_GROUP; --nID)
    {
        hCtl = GetDlgItem(hWnd, nID);
        GetWindowRect(hCtl, &rc);
        ScreenToClient(hWnd, reinterpret_cast<POINT*>(&rc.left));
        ScreenToClient(hWnd, reinterpret_cast<POINT*>(&rc.right));
        MoveWindow(hCtl, rc.left, rc.top, LOWORD(lParam) - (2 * rc.left), rc.bottom - rc.top, TRUE);
    }

    pt.x = rc.left / 2;
    pt.y = rc.top;

    hCtl = GetDlgItem(hWnd, IDOK);
    GetWindowRect(hCtl, &rc);
    MoveWindow(hCtl, ((LOWORD(lParam) / 2) - pt.x) - (rc.right - rc.left),
                     (HIWORD(lParam) - pt.y - (rc.bottom - rc.top)),
                     (rc.right - rc.left),
                     (rc.bottom - rc.top), FALSE);
    InvalidateRect(hWnd, NULL, TRUE);
    
    hCtl = GetDlgItem(hWnd, IDCANCEL);
    GetWindowRect(hCtl, &rc);
    MoveWindow(hCtl, ((LOWORD(lParam) / 2) + pt.x),
                     (HIWORD(lParam) - pt.y - (rc.bottom - rc.top)),
                     (rc.right - rc.left),
                     (rc.bottom - rc.top), FALSE);
    InvalidateRect(hWnd, NULL, TRUE);

    return(TRUE);
}

INT_PTR CALLBACK
CDownloadSetup::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
#pragma warning(push)
#pragma warning(disable: 4820)
    static const struct
    {
        UINT uMsg;
        INT_PTR (CDownloadSetup::*pfn)(HWND hWnd, WPARAM wParam, LPARAM lParam);
    }
    map[] =
    {
        {   WM_SIZE,            &OnWmSize           },
        {   WM_GETMINMAXINFO,   &OnWmGetMinMaxInfo  },
        {   WM_COMMAND,         &OnWmCommand        }
    };
#pragma warning(pop)

    for(unsigned n = 0; n < _countof(map); ++n)
        if(map[n].uMsg == uMsg)
            return((reinterpret_cast<CDownloadSetup*>(GetWindowLongPtr(hWnd, DWLP_USER))->*map[n].pfn)(hWnd, wParam, lParam));

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CDownloadSetup*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}