#include "precomp.h"

CEnumConnections::CEnumConnections(CConnectionPoint* pItf) throw()
    : m_lCursor(0), m_lRefCount(1), m_pItf(pItf)
{
    m_pItf->AddRef();
}

CEnumConnections::~CEnumConnections() throw()
{
    m_pItf->Release();
}

STDMETHODIMP
CEnumConnections::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IEnumConnections))
            *ppvObject = static_cast<IEnumConnections*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CEnumConnections::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CEnumConnections::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CEnumConnections::Next(THIS_ ULONG cConnections, CONNECTDATA* pData, ULONG* pcFetched)
{
    ULONG_PTR   lCursor;
    ULONG       ulTemp;
    BOOL        bl;

    if(FALSE == (bl = IsBadWritePtrAuto(pcFetched)))
        *pcFetched = 0;

#ifdef _DEBUG
    if(IsBadWritePtr(pData, (cConnections * sizeof(*pData))))
#else
    if(!pData)
#endif
        return(E_POINTER);

    ZeroMemory(pData, (cConnections * sizeof(*pData)));

    if(bl)
    {
        if((cConnections > 1) || (pcFetched))
            return(E_INVALIDARG);

        ulTemp = 0;
        pcFetched = &ulTemp;
    }

    for(bl = FALSE; ((*pcFetched < cConnections) && (!bl)); ++(*pcFetched))
    {
        do
        {
            if(pData[*pcFetched].pUnk)
            {
                pData[*pcFetched].pUnk->Release();
                pData[*pcFetched].pUnk = NULL;
                pData[*pcFetched].dwCookie = 0;
            }

#ifndef _MSC_VER
            lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            lCursor = m_lCursor;
#endif

            if(S_OK != m_pItf->GetConnectData(lCursor, &pData[*pcFetched]))
            {
                --(*pcFetched);
                bl = TRUE;
                break;
            }

        } while(InterlockedCompareExchangeRelease(&m_lCursor, lCursor + 1, lCursor) != lCursor);
    }

    return((cConnections == *pcFetched) ? S_OK : S_FALSE);
}

STDMETHODIMP
CEnumConnections::Skip(THIS_ ULONG cConnections)
{
    CONNECTDATA conn;
    ULONG_PTR   lCursor;

    while(cConnections--)
    {
        do
        {
#ifndef _MSC_VER
            lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            lCursor = m_lCursor;
#endif

            if(FAILED(m_pItf->GetConnectData(lCursor, &conn)))
                return(S_FALSE);

            conn.pUnk->Release();

        } while(InterlockedCompareExchangeRelease(&m_lCursor, lCursor + 1, lCursor) != lCursor);
    }

    return(S_OK);
}

STDMETHODIMP
CEnumConnections::Reset(THIS)
{
    InterlockedExchange(&m_lCursor, 0);
    return(S_OK);
}

STDMETHODIMP
CEnumConnections::Clone(THIS_ IEnumConnections** ppEnum)
{
    CEnumConnections*  pEnum;
    HRESULT             hr;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumConnections(m_pItf)))
        return(E_OUTOFMEMORY);

#ifndef _MSC_VER
    pEnum->m_lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, SIZE_MAX, SIZE_MAX);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    pEnum->m_lCursor = m_lCursor;
#endif

    hr = pEnum->QueryInterface(IID_IEnumConnections, (void**)ppEnum);
    pEnum->Release();
    return(hr);
}
