#include "precomp.h"

CEnumVARIANT::CEnumVARIANT(CSmartICE04Devices* pDevices) throw()
    : m_pDevices(pDevices), m_lRefCount(1), m_lCursor(0)
{
    m_pDevices->AddRef();
}

CEnumVARIANT::~CEnumVARIANT() throw()
{
    m_pDevices->Release();
}

STDMETHODIMP
CEnumVARIANT::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = static_cast<IUnknown*>(this);
        else if(IsEqualIID(riid, IID_IEnumVARIANT))
            *ppvObject = static_cast<IEnumVARIANT*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CEnumVARIANT::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CEnumVARIANT::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CEnumVARIANT::Next(THIS_ ULONG celt, VARIANT* rgVar, ULONG* pCeltFetched)
{
    VARIANT varIndex;
    HRESULT hr;
    ULONG   ulTemp;
    BOOL    bl;

    if(FALSE == (bl = IsBadWritePtrAuto(pCeltFetched)))
        *pCeltFetched = 0;

#ifdef _DEBUG
    if(IsBadWritePtr(rgVar, (sizeof(*rgVar) * celt)))
#else
    if(!rgVar)
#endif
        return(E_INVALIDARG);

    for(ulTemp = 0; ulTemp < celt; ++ulTemp)
        VariantInit(&rgVar[ulTemp]);

    if(bl)
    {
        if((celt > 1) || (pCeltFetched))
            return(E_INVALIDARG);

        ulTemp = 0;
        pCeltFetched = &ulTemp;
    }

    VariantInit(&varIndex);
    V_VT(&varIndex) = VT_I4;

    for(bl = FALSE; ((*pCeltFetched < celt) && (!bl)); ++(*pCeltFetched))
    {
        do
        {
            VariantClear(&rgVar[*pCeltFetched]);
#ifndef _MSC_VER
            V_I4(&varIndex) = InterlockedCompareExchangeAcquire(&m_lCursor, -1, -1);
#else
            // Microsoft uses Acquire semantics on volatile variables.
            V_I4(&varIndex) = m_lCursor;
#endif

            if(FAILED((hr = m_pDevices->get_Item(varIndex, &rgVar[*pCeltFetched]))))
            {
                if(hr == E_INVALIDARG)
                {
                    bl = TRUE;
                }
                else if(S_OK != Skip(1))
                {
                    bl = TRUE;
                }

                --(*pCeltFetched);
                break;
            }

        } while(InterlockedCompareExchangeRelease(&m_lCursor, V_I4(&varIndex) + 1, V_I4(&varIndex)) != V_I4(&varIndex));
    }

    VariantClear(&varIndex);
    return((*pCeltFetched < celt) ? S_FALSE : S_OK);
}

STDMETHODIMP
CEnumVARIANT::Skip(THIS_ ULONG celt)
{
    HRESULT hr;
    LONG    lNumElements, lCurrentValue, lNewValue;

    if(!celt)
        return(S_OK);

    do
    {
#ifndef _MSC_VER
        lCurrentValue = InterlockedCompareExchangeAcquire(&m_lCursor, -1, -1);
#else
        // Microsoft uses Acquire semantics on volatile variables.
        lCurrentValue = m_lCursor;
#endif
        lNewValue = static_cast<LONG>(lCurrentValue + celt);

        if(FAILED(m_pDevices->get_Count(&lNumElements)))
            return(S_FALSE);

        if(static_cast<ULONG_PTR>(lNewValue) > static_cast<ULONG_PTR>(m_lLBound + lNumElements))
        {
            lNewValue = m_lLBound + lNumElements;

            if(static_cast<ULONG>(lNewValue) < static_cast<ULONG>(lCurrentValue))
                lNewValue = lCurrentValue;

            hr = S_FALSE;
        }
        else
        {
            hr = S_OK;
        }

    } while(InterlockedCompareExchangeRelease(&m_lCursor, lNewValue, lCurrentValue) != lCurrentValue);

    return(hr);
}

STDMETHODIMP
CEnumVARIANT::Reset(THIS)
{
    InterlockedExchange(&m_lCursor, m_lLBound);
    return(S_OK);
}

STDMETHODIMP
CEnumVARIANT::Clone(THIS_ IEnumVARIANT** ppEnum)
{
    CEnumVARIANT*   pEnum;
    HRESULT         hr;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumVARIANT(m_pDevices)))
        return(E_OUTOFMEMORY);

#ifndef _MSC_VER
    pEnum->m_lCursor = InterlockedCompareExchangeAcquire(&m_lCursor, -1, -1);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    pEnum->m_lCursor = m_lCursor;
#endif

    hr = pEnum->QueryInterface(IID_IEnumVARIANT, (void**)ppEnum);
    pEnum->Release();
    return(hr);
}
