#include "precomp.h"

HRESULT
CIntelHexFile::Load(LPCWSTR filename, SAFEARRAY** ppsa)
{
    HRESULT hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
    RECORD_INFO ri;
    FILE* file;
    char line[528];
    char* psz;
    SAFEARRAYBOUND sab;
    long lExtendedLinearAddress = 0, lExtendedSegmentAddress = 0, tmp;
    bool blIsSegmented = false, blEof = false;
    BYTE bCheckSum;
    unsigned cnt;
    BYTE* pData;

    *ppsa = NULL;

    if(NULL == (file = _wfopen(filename, L"r")))
        return(HresultFromErrno());

    while(!feof(file))
    {
        if(!fgets(line, _countof(line), file))
        {
            hr = HresultFromErrno();
            break;
        }

        if(NULL != (psz = strchr(line, '\n')))
            *psz = '\0';

        if(FAILED((hr = GetRecordInfo(line, &ri))))
            break;

        if(strlen(line) < (ri.len + 11))
        {
            hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
            break;
        }

        bCheckSum = LOBYTE(ri.typ + LOBYTE(ri.offset) + HIBYTE(ri.offset) + ri.len);

        switch(ri.typ)
        {
            case Data:
                if(blIsSegmented)
                    tmp = static_cast<long>((lExtendedSegmentAddress << 4) + ri.offset);
                else
                    tmp = static_cast<long>((lExtendedLinearAddress << 16) + ri.offset);

                if(SUCCEEDED((hr = RecreateArray(tmp, &ri, ppsa, &sab, &pData))))
                {
                    for(cnt = 0; cnt < ri.len; ++cnt)
                    {
                        if(1 != sscanf(&line[cnt * 2 + 9], "%2x", &tmp))
                        {
                            hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
                            break;
                        }
                        *pData++ = LOBYTE(tmp);
                        bCheckSum += LOBYTE(tmp);
                    }
                }
                break;
            case EndOfFile:
                if(ri.len || ri.offset)
                    hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                blEof = true;
                break;
            case ExtendedSegmentAddress:
                if((1 != sscanf(&line[9], "%4x", &lExtendedSegmentAddress)) || (ri.len != 2) || (ri.offset))
                    hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
                else
                    blIsSegmented = true;

                bCheckSum += LOBYTE(LOBYTE(lExtendedSegmentAddress) + HIBYTE(lExtendedSegmentAddress));
                break;
            case ExtendedLinearAddress:
                if((1 != sscanf(&line[9], "%4x", &lExtendedLinearAddress)) || (ri.len != 2) || (ri.offset))
                    hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                bCheckSum += LOBYTE(LOWORD(lExtendedLinearAddress) + HIBYTE(LOWORD(lExtendedLinearAddress)));
                break;
            case StartSegmentAddress:
            case StartLinearAddress:
                if((1 != sscanf(&line[9], "%4x", &tmp)) || (ri.len != 4) || (ri.offset))
                    hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);

                for(cnt = 0; cnt < 4; ++cnt)
                {
                    bCheckSum += LOBYTE(tmp);
                    tmp >>= 8;
                }

                break;
            default:
                break;
        }

        if(1 != sscanf(&line[ri.len * 2 + 9], "%2x", &tmp))
        {
            hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
        }
        else if(static_cast<BYTE>(tmp + bCheckSum))
        {
            hr = HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT);
        }

        if(blEof)
            break;

        if(FAILED(hr))
        {
            if(*ppsa)
            {
                SafeArrayDestroy(*ppsa);
                *ppsa = NULL;
            }
            break;
        }
    }

    fclose(file);
    return(hr);
}

HRESULT
CIntelHexFile::GetRecordInfo(const char* line, RECORD_INFO* pri)
{
    int typ;

    if(*line != ':')
        return(HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    typ = MAKEWORD(line[8], line[7]) - 0x3030;

    if(typ > StartLinearAddress)
        return(HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    if(sscanf(&line[1], "%2x%4x", &pri->len, &pri->offset) < 2)
        return(HRESULT_FROM_WIN32(ERROR_FILE_CORRUPT));

    pri->typ = static_cast<RecordType>(typ);
    return(S_OK);
}

HRESULT
CIntelHexFile::RecreateArray(long lStartAddress, const RECORD_INFO* pri, SAFEARRAY** ppsa, SAFEARRAYBOUND* psab, BYTE** ppData)
{
    HRESULT hr;
    long cnt, lFillAddress;

    if(!*ppsa)
    {
        psab->cElements = pri->len;
        psab->lLbound = lStartAddress;

        if(NULL == (*ppsa = SafeArrayCreate(VT_UI1, 1, psab)))
            return(E_OUTOFMEMORY);

        return(SafeArrayPtrOfIndex(*ppsa, &lStartAddress, reinterpret_cast<void**>(ppData)));
    }

    if(lStartAddress < psab->lLbound)
    {
        cnt = psab->lLbound - lStartAddress;

        if(pri->len > (psab->cElements = (cnt + psab->cElements)))
            psab->cElements = pri->len;

        psab->lLbound = lFillAddress = lStartAddress;
    }
    else if(0 <= (cnt = static_cast<long>(lStartAddress - (psab->lLbound + psab->cElements))))
    {
        psab->cElements += cnt + pri->len;
        lFillAddress = lStartAddress - cnt;
    }
    else
    {
        if(0 < (cnt = static_cast<long>((lStartAddress + pri->len) - (psab->lLbound + psab->cElements))))
            psab->cElements += cnt;

        cnt = lFillAddress = 0;
    }

    if(FAILED((hr = SafeArrayRedim(*ppsa, psab))))
        return(hr);

    if(cnt)
    {
        if(FAILED((hr = SafeArrayPtrOfIndex(*ppsa, &lFillAddress, reinterpret_cast<void**>(ppData)))))
            return(hr);

        FillMemory(*ppData, static_cast<size_t>(cnt), 0xFF);
    }

    return(SafeArrayPtrOfIndex(*ppsa, &lStartAddress, reinterpret_cast<void**>(ppData)));
}
