#ifndef CLIST_H
#define CLIST_H

template<typename T>
class CList
{
public:
    CList() throw() : m_pList(NULL), m_uSize(0) { }
    ~CList() throw() { while(Remove(0)); }

    BOOL Add(__in const T& val) throw() { return(Insert(m_uSize, val)); }

    BOOL Insert(__in UINT_PTR uIndex, __in const T& val) throw()
    {
        CListElement* pNew;

        if(NULL == (pNew = new (std::nothrow) CListElement(val)))
        {
            SetLastError(ERROR_OUTOFMEMORY);
            return(FALSE);
        }

        if(!m_pList)
        {
            m_pList = m_pLast = m_cursor.pElement = pNew;
        }
        else if(uIndex >= m_uSize)
        {
            m_pLast->m_pNext = pNew;
            m_pLast = pNew;
        }
        else if(!uIndex)
        {
            pNew->m_pNext = m_pList;
            m_pList = pNew;
            ++m_cursor.idx;
        }
        else
        {
            if(m_cursor.idx >= uIndex)
            {
                m_cursor.idx = 0;
                m_cursor.pElement = m_pList;
            }

            uIndex -= m_cursor.idx;
            m_cursor.idx += uIndex;

            for(; --uIndex; m_cursor.pElement = m_cursor.pElement->m_pNext)
                ;

            pNew->m_pNext = m_cursor.pElement->m_pNext;
            m_cursor.pElement->m_pNext = pNew;
        }

        ++m_uSize;
        return(TRUE);
    }

    BOOL Remove(__in UINT_PTR uIndex, __out_opt T* pval = NULL) throw()
    {
        CListElement* pRemove;
        CListElement* pPrev;

        if(uIndex >= m_uSize)
        {
            SetLastError(ERROR_INVALID_INDEX);
            return(FALSE);
        }

        if(!uIndex)
        {
            pRemove = m_pList;
            m_pList = pRemove->m_pNext;

            if(!m_cursor.idx)
            {
                m_cursor.pElement = m_pList;
            }
            else
            {
                --m_cursor.idx;
            }
        }
        else
        {
            if(m_cursor.idx >= uIndex)
            {
                m_cursor.idx = 0;
                m_cursor.pElement = m_pList;
            }

            uIndex -= m_cursor.idx;
            m_cursor.idx += uIndex;

            for(pPrev = m_cursor.pElement; --uIndex; pPrev = pPrev->m_pNext)
                ;

            pRemove = pPrev->m_pNext;

            if(NULL == (pPrev->m_pNext = m_cursor.pElement = pRemove->m_pNext))
            {
                m_pLast = m_cursor.pElement = pPrev;
                --m_cursor.idx;
            }
        }

        if(pval)
            *pval = pRemove->m_val;

        delete pRemove;
        --m_uSize;
        return(TRUE);
    }

    BOOL Get(__in UINT_PTR uIndex, __deref_out T* pval) const throw()
    {
        assert(pval);

#ifdef _DEBUG
        if(!pval)
        {
            SetLastError(ERROR_INVALID_PARAMETER);
            return(FALSE);
        }
#endif

        if(uIndex >= m_uSize)
        {
            SetLastError(ERROR_INVALID_PARAMETER);
            return(FALSE);
        }
        else if(uIndex == (m_uSize - 1))
        {
            *pval = m_pLast->m_val;
        }
        else
        {
            if(m_cursor.idx > uIndex)
            {
                m_cursor.idx = 0;
                m_cursor.pElement = m_pList;
            }

            uIndex -= m_cursor.idx;
            m_cursor.idx += uIndex;

            for(; uIndex; m_cursor.pElement = m_cursor.pElement->m_pNext)
                --uIndex;

            *pval = m_cursor.pElement->m_val;
        }

        return(TRUE);
    }

    template<typename KEY>
    BOOL Find(__in const KEY& key, __out_opt T* pval = NULL, __out_opt UINT_PTR* pidx = NULL) const throw()
    {
        CListElement* pElement = m_pList;

        for(UINT_PTR idx = 0; pElement; ++idx)
        {
            if(Equals(key, pElement->m_val))
            {
                if(pidx)
                    *pidx = idx;

                if(pval)
                    *pval = pElement->m_val;

                return(TRUE);
            }
            pElement = pElement->m_pNext;
        }

        return(FALSE);
    }

    UINT_PTR Size(void) const throw() { return(m_uSize); }

private:
    template<typename KEY, typename EQ> bool Equals(__in const KEY& key, __in EQ* pval) const throw() { return(*pval == key); }
    template<typename KEY, typename EQ> bool Equals(__in const KEY& key, __in EQ& val) const throw() { return(val == key); }

    CList(const CList&) throw();
    CList& operator=(const CList&) throw();

protected:
    class CListElement
    {
    public:
        CListElement(const T& val) throw() : m_val(val), m_pNext(NULL) { };
        ~CListElement() throw() { };

        CListElement* m_pNext;
        T m_val;

        CListElement(const CListElement&);
    };

    mutable struct CListCursor
    {
        CListElement* pElement;
        UINT_PTR idx;
        CListCursor() : idx(0) { }
    } m_cursor;

    CListElement* m_pList;
    CListElement* m_pLast;
    UINT_PTR      m_uSize;
};

#endif
