#ifndef CSMARTICE04_H
#define CSMARTICE04_H

enum StatusFlags
{
    TargetVoltagePresent = 0x01,
    DebugMode = 0x02
};

#include <pshpack1.h>

struct REPORT1
{
    BYTE    bReportId;
    BYTE    bChipId;
    BYTE    bVersion;
    BYTE    bStatus;
    WORD    wTargetVoltage;
};

#define CMDID_DEBUGMODE         0x01    /* param1 = 0xFFFF -> Enter, param1 = 0x0000 -> Leave                                                               */
#define CMDID_DEBUGCONFIG       0x02    /* param1 = 0x00xx -> xx = config data                                                                              */
#define CMDID_RESUME            0x03    /* ** no parameters **                                                                                              */
#define CMDID_HALT              0x04    /* ** no parameters **                                                                                              */
#define CMDID_CHIP_ERASE        0x05    /* ** no parameters                                                                                                 */
#define CMDID_DEBUG_INSTR       0x06    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3                                                  */
#define CMDID_STEP_REPLACE      0x07    /* param1 low -> byte1, param1 high -> byte2, param2 low -> byte 3, param2 high -> number of bytes used             */
#define CMDID_STEP_INSTR        0x08    /* ** no parameters **                                                                                              */
#define CMDID_SET_HW_BRKPNT     0x09    /* param1 -> Breakpoint address[0..15], param2 low -> Breakpoint address[16..23], param2 high -> Breakpoint Info    */

typedef struct
{
    BYTE    bReportId;
    BYTE    bCmd;
    WORD    param1;
    WORD    param2;
} REPORT2;

typedef struct
{
    BYTE            bReportId;
    BYTE            bSegment;
    unsigned long   uAddress;
    unsigned long   uLen;
    BYTE            blRead;
} REPORT3;

typedef struct
{
    BYTE    bReportId;
    BYTE    bLength;
    BYTE    bData[62];
} REPORT4;

typedef struct
{
    BYTE    bReportId;
    BYTE    bInDbgMode;
    BYTE    bConfig;
    BYTE    bStatus;
    BYTE    bResult;
} REPORT5;

typedef struct
{
    BYTE    bReportId;
    WORD    pc;
} REPORT6;

typedef struct
{
    BYTE    bReportId;
    BYTE    bLockBits[1];
    BYTE    bReserved[15];
} REPORT7;

typedef struct
{
    BYTE    bBank;
} REPORT8;

union IOREPORT
{
    REPORT1 rep1;
    REPORT2 rep2;
    REPORT3 rep3;
    REPORT4 rep4;
    REPORT5 rep5;
    REPORT6 rep6;
    REPORT7 rep7;
    REPORT8 rep8;
};

#include <poppack.h>

#define CSMARTICE04_STATUS_GETMEMORY    0x00000001
#define CSMARTICE04_STATUS_SETMEMORY    0x00000002
#define CSMARTICE04_STATUS_VERIFY       0x00000004

class CSmartICE04Devices;

class CSmartICE04 : public ISmartICE04, public IConnectionPointContainer, public IProvideClassInfo2, public ISupportErrorInfo
{
protected:
    inline IUnknown* GetUnknown() throw() { return(static_cast<ISmartICE04*>(this));   }
    inline HRESULT RaiseCOMException(DISPID dispid, HRESULT hr) throw() { return(::RaiseCOMException(this, dispid, hr));  }
    inline WORD getPID() const { return(m_wPID); }

    CSmartICE04(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, HRESULT* phr) throw();

public:
    // IUnknown
    STDMETHOD(QueryInterface) (THIS_ REFIID riid, void** ppvObject);
    STDMETHOD_(ULONG, AddRef) (THIS);
    STDMETHOD_(ULONG, Release)(THIS);

    // IDispatch
    STDMETHOD(GetTypeInfoCount)(THIS_ UINT* pctinfo);
    STDMETHOD(GetTypeInfo)(THIS_ UINT iTInfo, LCID lcid, ITypeInfo** ppTInfo);
    STDMETHOD(GetIDsOfNames)(THIS_ REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId);
    STDMETHOD(Invoke)(THIS_ DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr);

    // ISmartICE04
    STDMETHOD(get_Parent)(THIS_ IDispatch** ppParent);
    STDMETHOD(get_ProductId)(THIS_ long* PID);
    STDMETHOD(get_SerialNumber)(THIS_ BSTR* SerNum);
    STDMETHOD(get_Icon)(THIS_ IDispatch** Picture);
    STDMETHOD(get_SmallIcon)(THIS_ IDispatch** Picture);
    STDMETHOD(get_ChipId)(THIS_ enum ChipId* cid);
    STDMETHOD(get_ChipVersion)(THIS_ long* vers);
    STDMETHOD(get_TargetVoltage)(THIS_ long* targetVoltage);
    STDMETHOD(EnterDebugMode)(THIS_ VARIANT_BOOL enableDebug);
    STDMETHOD(get_IsInDebugMode)(THIS_ VARIANT_BOOL* isInDbgMode);
    STDMETHOD(get_DebugConfig)(THIS_ long* configFlags);
    STDMETHOD(put_DebugConfig)(THIS_ long configFlags);
    STDMETHOD(ChipErase)(THIS);
    STDMETHOD(Halt)(THIS);
    STDMETHOD(Resume)(THIS);
    STDMETHOD(ReadStatus)(THIS_ long* statusFlags);
    STDMETHOD(DebugInstr)(THIS_ long cmd, long* result);
    STDMETHOD(StepReplace)(THIS_ long cmd, long* result);
    STDMETHOD(StepInstr)(THIS_ long* result);
    STDMETHOD(SetHwBreakpoint)(THIS_ long brkpntNumber, VARIANT_BOOL enable, long address);
    STDMETHOD(GetPc)(THIS_ long* pc);
    STDMETHOD(ReadLockBits)(THIS_ long* lockbits);
    STDMETHOD(WriteLockBits)(THIS_ long lockbits);
    STDMETHOD(GetMemory)(THIS_ enum MemorySegment seg, long startAddress, long len, VARIANT* progressItf, SAFEARRAY** data);
    STDMETHOD(SetMemory)(THIS_ enum MemorySegment seg, long startAddress, SAFEARRAY** data, VARIANT_BOOL verify, VARIANT* progressItf);
    STDMETHOD(DownloadFile)(THIS_ BSTR filename, enum FileType ft, VARIANT_BOOL verify, VARIANT* progressItf);
    STDMETHOD(GetBm)(THIS_ long* bm);

    // IConnectionPointContainer
    STDMETHOD(EnumConnectionPoints)(THIS_ IEnumConnectionPoints** ppEnum);
    STDMETHOD(FindConnectionPoint)(THIS_ REFIID riid, IConnectionPoint** ppCP);

    // IProvideClassInfo2
    STDMETHOD(GetClassInfo)(THIS_ ITypeInfo** ppTInfo);
    STDMETHOD(GetGUID)(THIS_ DWORD dwGuidKind, GUID* pGUID);

    // ISupportErrorInfo
    STDMETHOD(InterfaceSupportsErrorInfo)(THIS_ REFIID riid);

    static NTSTATUS WINAPI HidP_InitializeReportForID(IN HIDP_REPORT_TYPE, IN UCHAR ReportID, IN PHIDP_PREPARSED_DATA, IN OUT PCHAR Report, IN ULONG ReportLength) throw();
    static HRESULT Create(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, CSmartICE04** ppDev) throw();

    void OnDetached() throw();
    inline bool operator == (__in_z LPCTSTR pszName) const throw() { return(CSTR_EQUAL == CompareString(MAKELCID(MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US), SORT_DEFAULT), NORM_IGNORECASE, pszName, -1, m_pszDeviceName, -1)); }
    inline bool operator == (__in BSTR SerialNumber) const throw() { return(VARCMP_EQ == VarBstrCmp(SerialNumber, m_bstrSerialNumber, MAKELCID(MAKELANGID(LANG_ENGLISH, SUBLANG_DEFAULT), SORT_DEFAULT), NORM_IGNORECASE)); }

#pragma warning(suppress: 6031)
    inline void Refresh(void) { if(!(m_ObjectStatus & (CSMARTICE04_STATUS_GETMEMORY | CSMARTICE04_STATUS_SETMEMORY))) { IOREPORT report; ReadReport1(&report); }}

private:
    virtual ~CSmartICE04();
    CSmartICE04(const CSmartICE04&);
    CSmartICE04& operator=(const CSmartICE04&);

    __checkReturn HRESULT ReadReport1(__in IOREPORT* pReport) throw();
    __checkReturn HRESULT ReadReport5(__in IOREPORT* pReport) const throw();
    __checkReturn HRESULT ReadReport6(__in IOREPORT* pReport) const throw();
    __checkReturn HRESULT ReadReport7(__in IOREPORT* pReport) const throw();
    __checkReturn HRESULT ReadReport8(__in IOREPORT* pReport) const throw();
    __checkReturn HRESULT WriteReport(__in const IOREPORT* pReport) const throw();
    static __checkReturn HRESULT ExtractProgressItf(__in VARIANT* progressItf, __in CSmartICE04ProgressDummy* pDummy, __out ISmartICE04Progress** ppProgressItf) throw();

private:
    CSmartICE04Devices* m_pParent;
    CDSmartICE04Events  m_DispEvents;
    CSmartICE04Events   m_Events;
    ITypeInfo*          m_pTypeInfo;
    HANDLE              m_hDevice;
    HANDLE              m_hMutex;
    LPTSTR              m_pszDeviceName;
    BYTE*               m_pFeatureReport;
    BYTE*               m_pInputReport;
    BSTR                m_bstrSerialNumber;
    BOOL                m_blDetached;
    BOOL                m_blInDebugMode;
    USHORT              m_usFeatureReportLength;
    USHORT              m_usInputReportLength;
    ULONG               m_lRefCount;
    ULONG               m_ObjectStatus;
    REPORT1             m_report1;
    WORD                m_wPID;
    BYTE                m_bDebugConfig;
    BYTE bPadding[3]; // C4820
};

#endif