#include "precomp.h"

CSmartICE04Devices::CSmartICE04Devices(HRESULT* phr) throw()
    : m_lRefCount(1), m_hDevNotify(NULL), m_pTypeInfo(NULL), m_hWnd(NULL), m_blRefreshing(FALSE)
{
    SP_DEVICE_INTERFACE_DETAIL_DATA*    pspdidd;
    DEV_BROADCAST_DEVICEINTERFACE       itf = { 0 };
    SP_DEVICE_INTERFACE_DATA            spdid;
    WNDCLASSEX                          wc, wcTemp = { 0 };
    ITypeLib*                           pTlb = NULL;
    HDEVINFO                            hDevInfo;
    DWORD                               dwIndex, dwSize;

    ::LockServer();
    m_Events.SetContainer(this);
    m_DispEvents.SetContainer(this);

    wc.cbSize           = sizeof(wc);
    wc.style            = 0;
    wc.lpfnWndProc      = WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = g_hInst;
    wc.hIcon            = NULL;
    wc.hCursor          = NULL;
    wc.hbrBackground    = NULL;
    wc.lpszMenuName     = NULL;
    wc.lpszClassName    = TEXT("--{FEFE1540-2F65-46F3-9796-E49BDD1D2126}--");
    wc.hIconSm          = NULL;

    wcTemp.cbSize = sizeof(wcTemp);

    if(!GetClassInfoEx(wc.hInstance, wc.lpszClassName, &wcTemp))
    {
        if(!RegisterClassEx(&wc))
        {
            *phr = HRESULT_FROM_WIN32(GetLastError());
            return;
        }
    }

    if(NULL == (m_hWnd = CreateWindowEx(0, wc.lpszClassName, wc.lpszClassName, 0, 0, 0, 0, 0, NULL, NULL, g_hInst, this)))
    {
        *phr =  HRESULT_FROM_WIN32(GetLastError());
        return;
    }
    else if(!SetTimer(m_hWnd, 1234, 2500, NULL))
    {
        *phr =  HRESULT_FROM_WIN32(GetLastError());
        return;
    }

    itf.dbcc_size       = sizeof(itf);
    itf.dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE;
    HidD_GetHidGuid(&itf.dbcc_classguid);

    if(NULL == (m_hDevNotify = RegisterDeviceNotification(m_hWnd, &itf, DEVICE_NOTIFY_WINDOW_HANDLE)))
    {
        *phr = HRESULT_FROM_WIN32(GetLastError());
        return;
   }

    if(SUCCEEDED((*phr = LoadRegTypeLib(LIBID_SmartICE04Lib, 1, 0, LOCALE_USER_DEFAULT, &pTlb))))
    {
        if(SUCCEEDED((*phr = pTlb->GetTypeInfoOfGuid(IID_ISmartICE04Devices, &m_pTypeInfo))))
        {
            ZeroMemory(&spdid, sizeof(spdid));
            spdid.cbSize = sizeof(spdid);

            if(INVALID_HANDLE_VALUE == (hDevInfo = SetupDiGetClassDevs(&itf.dbcc_classguid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT)))
            {
                *phr = HRESULT_FROM_SETUPAPI(GetLastError());
            }
            else
            {
                for(dwIndex = 0; (SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &itf.dbcc_classguid, dwIndex, &spdid)); ++dwIndex)
                {
                    dwSize = 0;
                    SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid, NULL, 0, &dwSize, NULL);

                    if(dwSize)
                    {
                        if(NULL == (pspdidd = reinterpret_cast<PSP_DEVICE_INTERFACE_DETAIL_DATA>(new (std::nothrow) BYTE[dwSize])))
                            continue;

                        ZeroMemory(pspdidd, dwSize);
                        pspdidd->cbSize = sizeof(*pspdidd);

                        if(SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid, pspdidd, dwSize, &dwSize, NULL))
                            Add(pspdidd->DevicePath);

                        delete [] reinterpret_cast<BYTE*>(pspdidd);
                    }
                }
                SetupDiDestroyDeviceInfoList(hDevInfo);
            }
        }
        pTlb->Release();
    }
}

CSmartICE04Devices::~CSmartICE04Devices()
{
    CSmartICE04* pDev;

    if(m_hDevNotify)
        UnregisterDeviceNotification(m_hDevNotify);

    if(m_hWnd)
    {
        KillTimer(m_hWnd, 1234);
        DestroyWindow(m_hWnd);
    }

    while(m_Devices.Remove(0, &pDev))
        pDev->Release();

    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    ::UnlockServer();
}

STDMETHODIMP
CSmartICE04Devices::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04Devices))
            *ppvObject = static_cast<ISmartICE04Devices*>(this);
        else if(IsEqualIID(riid, IID_IConnectionPointContainer))
            *ppvObject = static_cast<IConnectionPointContainer*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo2))
            *ppvObject = static_cast<IProvideClassInfo2*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04Devices::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CSmartICE04Devices::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CSmartICE04Devices::GetTypeInfoCount(THIS_ UINT* pctinfo)
{
    if(IsBadWritePtrAuto(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Devices::GetTypeInfo(THIS_ UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = NULL;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Devices::GetIDsOfNames(THIS_ REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

STDMETHODIMP
CSmartICE04Devices::Invoke(THIS_ DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04Devices*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

STDMETHODIMP
CSmartICE04Devices::get_Item(const VARIANT varIndex, VARIANT* pVal)
{
    CSmartICE04*    pDev = NULL;
    HRESULT         hr;
    VARIANT         var;

    if(IsBadWritePtrAuto(pVal))
        return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, E_POINTER));

    VariantInit(pVal);

    if(V_VT(&varIndex) == VT_BSTR)
    {
        if(!V_BSTR(&varIndex))
            return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, E_INVALIDARG));

        if(m_Devices.Find(V_BSTR(&varIndex), &pDev))
        {
            if(SUCCEEDED((hr = pDev->QueryInterface(IID_IDispatch, (void**)&V_DISPATCH(pVal)))))
            {
                V_VT(pVal) = VT_DISPATCH;
                return(S_OK);
            }

            return(RaiseCOMException(DISPID_SMARTICE04_LISTITEM, hr));
        }
    }

    VariantInit(&var);

    if(SUCCEEDED((hr = VariantChangeTypeEx(&var, const_cast<VARIANT*>(&varIndex), CThreadLocale::getLCID(), 0, VT_I4))))
    {
        if(!m_Devices.Get(static_cast<UINT_PTR>(V_I4(&var)), &pDev))
        {
            hr = E_INVALIDARG;
        }
        else if(SUCCEEDED((hr = pDev->QueryInterface(IID_IDispatch, (void**)&V_DISPATCH(pVal)))))
        {
            V_VT(pVal) = VT_DISPATCH;
        }
    }
    else if(V_VT(&varIndex) == VT_BSTR)
    {
        hr = E_INVALIDARG;
    }

    VariantClear(&var);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LISTITEM, hr));
}

STDMETHODIMP
CSmartICE04Devices::get_Count(long* pVal)
{
    HRESULT hr = getCount(pVal);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LISTCOUNT, hr));
}

STDMETHODIMP
CSmartICE04Devices::get_length(long* pVal)
{
    HRESULT hr = getCount(pVal);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_COLLCOUNT, hr));
}

STDMETHODIMP
CSmartICE04Devices::get__NewEnum(THIS_ IUnknown** ppVal)
{
    CEnumVARIANT*   pEnum;
    HRESULT         hr;

    if(IsBadWritePtrAuto(ppVal))
        return(RaiseCOMException(DISPID_SMARTICE04__NEWENUM, E_POINTER));

    *ppVal = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumVARIANT(this)))
        return(RaiseCOMException(DISPID_SMARTICE04__NEWENUM, E_OUTOFMEMORY));

    hr = pEnum->QueryInterface(IID_IUnknown, (void**)ppVal);
    pEnum->Release();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04__NEWENUM, hr));
}

STDMETHODIMP
CSmartICE04Devices::EnumConnectionPoints(IEnumConnectionPoints** ppEnum)
{
    CEnumConnectionPoints*  pEnum;
    HRESULT                 hr;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumConnectionPoints))
        return(E_OUTOFMEMORY);

    if(SUCCEEDED((hr = pEnum->Add(m_DispEvents.GetUnknown()))))
        if(SUCCEEDED((hr = pEnum->Add(m_Events.GetUnknown()))))
            hr = pEnum->QueryInterface(IID_IEnumConnectionPoints, (void**)ppEnum);

    pEnum->Release();
    return(hr);
}

STDMETHODIMP
CSmartICE04Devices::FindConnectionPoint(REFIID riid, IConnectionPoint** ppCP)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04DevicesEvents))
            return(m_Events.QueryInterface(IID_IConnectionPoint, (void**)ppCP));
        else if(IsEqualIID(riid, DIID__DISmartICE04DevicesEvents))
            return(m_DispEvents.QueryInterface(IID_IConnectionPoint, (void**)ppCP));

        *ppCP = NULL;
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_POINTER);
    }

    return(CONNECT_E_NOCONNECTION);
}

STDMETHODIMP
CSmartICE04Devices::GetClassInfo(ITypeInfo** ppTI)
{
    ITypeLib*   pTlb = NULL;
    HRESULT     hr;
    UINT        idx;

    if(IsBadWritePtrAuto(ppTI))
        return(E_POINTER);

    *ppTI = NULL;

    if(SUCCEEDED((hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &idx))))
    {
        hr = pTlb->GetTypeInfoOfGuid(CLSID_SmartICE04Devices, ppTI);
        pTlb->Release();
    }

    return(hr);
}

STDMETHODIMP
CSmartICE04Devices::GetGUID(DWORD dwGuidKind, GUID* pGUID)
{
    if(IsBadWritePtrAuto(pGUID))
        return(E_POINTER);

    if(dwGuidKind != GUIDKIND_DEFAULT_SOURCE_DISP_IID)
    {
        *pGUID = GUID_NULL;
        return(E_INVALIDARG);
    }

    *pGUID = DIID__DISmartICE04DevicesEvents;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Devices::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04Devices))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

HRESULT
CSmartICE04Devices::getCount(long* pVal) const throw()
{
    if(IsBadWritePtrAuto(pVal))
        return(E_POINTER);

    *pVal = static_cast<long>(m_Devices.Size());
    return(S_OK);
}

void
CSmartICE04Devices::Refresh(void) throw()
{
    CSmartICE04*    pDev;
    UINT            x;

    if(!m_blRefreshing)
    {
        m_blRefreshing = TRUE;

        for(x = 0; m_Devices.Get(x, &pDev); ++x)
            pDev->Refresh();

        m_blRefreshing = FALSE;
    }
}

HRESULT
CSmartICE04Devices::Add(LPCTSTR pszName) throw()
{
    typedef HRESULT (* CREATION_FUNC)(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, CSmartICE04** ppDev);

    static const CREATION_FUNC pfnCreate[] =
    {
        CSmartICE04::Create
    };

    HIDD_ATTRIBUTES attr = { 0 };
    CSmartICE04*    pDev;
    ITypeLib*       pTlb = NULL;
    HRESULT         hr;
    HANDLE          hDevice;
    UINT            x;

    if(m_Devices.Find(pszName))
        return(S_OK);

    if(INVALID_HANDLE_VALUE == (hDevice = CreateFile(pszName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL)))
        return(HRESULT_FROM_WIN32(GetLastError()));

    attr.Size = sizeof(attr);
    
    if(!HidD_GetAttributes(hDevice, &attr))
    {
        hr = HRESULT_FROM_WIN32(GetLastError());
    }
    else if(attr.VendorID != 0x14AB)
    {
        hr = E_INVALIDARG;
    }
    else
    {
        if(SUCCEEDED((hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &x))))
        {
            for(x = 0; x < _countof(pfnCreate); ++x)
            {
                if(SUCCEEDED((hr = pfnCreate[x](pTlb, this, hDevice, pszName, attr.ProductID, &pDev))))
                {
                    hDevice = INVALID_HANDLE_VALUE; // pDev took ownership

                    if(!m_Devices.Add(pDev))
                    {
                        hr = HRESULT_FROM_WIN32(GetLastError());
                        pDev->Release();
                    }
                    else
                    {
                        m_Events.FireEvent(DISPID_SMARTICE04_ONDEVICEATTACHED, pDev);
                        m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEVICEATTACHED, pDev);
                    }
                    break;
                }
            }
            pTlb->Release();
        }
    }

    if(INVALID_HANDLE_VALUE != hDevice)
         CloseHandle(hDevice);

    return(hr);
}

HRESULT
CSmartICE04Devices::Add(const DEV_BROADCAST_DEVICEINTERFACE* pItf) throw()
{
    GUID    guid = { 0 };
    HidD_GetHidGuid(&guid);

    if(!IsEqualGUID(guid, pItf->dbcc_classguid))
        return(E_INVALIDARG);

#ifdef UNICODE
    if(!IsWindowUnicode(m_hWnd))
    {
        BSTR bstr = BSTRFromStringA(reinterpret_cast<const DEV_BROADCAST_DEVICEINTERFACE_A*>(pItf)->dbcc_name);

        if(!bstr)
            return(E_OUTOFMEMORY);

        HRESULT hr = Add(bstr);
        SysFreeString(bstr);
        return(hr);
    }
#endif

    return(Add(pItf->dbcc_name));
}

HRESULT
CSmartICE04Devices::Remove(LPCTSTR pszName) throw()
{
    CSmartICE04*    pDev;
    UINT_PTR        x;

    if(m_Devices.Find(pszName, &pDev, &x))
    {
        m_Devices.Remove(x);
        m_Events.FireEvent(DISPID_SMARTICE04_ONDEVICEDETACHED, pDev);
        m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEVICEDETACHED, pDev);
        pDev->OnDetached();
        pDev->Release();
        return(S_OK);
    }

    return(E_INVALIDARG);
}

HRESULT
CSmartICE04Devices::Remove(const DEV_BROADCAST_DEVICEINTERFACE* pItf) throw()
{
#ifdef UNICODE
    if(!IsWindowUnicode(m_hWnd))
    {
        BSTR bstr = BSTRFromStringA(reinterpret_cast<const DEV_BROADCAST_DEVICEINTERFACE_A*>(pItf)->dbcc_name);

        if(!bstr)
            return(E_OUTOFMEMORY);

        HRESULT hr = Remove(bstr);
        SysFreeString(bstr);
        return(hr);
    }
#endif

    return(Remove(pItf->dbcc_name));
}

LRESULT CALLBACK
CSmartICE04Devices::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) throw()
{
    CSmartICE04Devices* pDevs = reinterpret_cast<CSmartICE04Devices*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));

    switch(uMsg)
    {
        case WM_CREATE:
            SetWindowLongPtr(hWnd, GWLP_USERDATA, reinterpret_cast<LONG_PTR>(reinterpret_cast<CREATESTRUCT*>(lParam)->lpCreateParams));
            return(0);
        case WM_TIMER:
            if(wParam == 1234)
                pDevs->Refresh();

            return(0);
        case WM_DEVICECHANGE:
            if(wParam == DBT_DEVICEARRIVAL)
            {
                if(reinterpret_cast<DEV_BROADCAST_HDR*>(lParam)->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
                    pDevs->Add(reinterpret_cast<DEV_BROADCAST_DEVICEINTERFACE*>(lParam));
            }
            else if(wParam == DBT_DEVICEREMOVECOMPLETE)
            {
                if(reinterpret_cast<DEV_BROADCAST_HDR*>(lParam)->dbch_devicetype == DBT_DEVTYP_DEVICEINTERFACE)
                    pDevs->Remove(reinterpret_cast<DEV_BROADCAST_DEVICEINTERFACE*>(lParam));
            }
            return(TRUE);
        case WM_CLOSE:
            DestroyWindow(hWnd);
            return(0);
        default:
            break;
    }

    return(DefWindowProc(hWnd, uMsg, wParam, lParam));
}
