#include "precomp.h"

CSmartICE04DevicesEventsSink::CSmartICE04DevicesEventsSink()
    : m_pCP(NULL), m_dwCookie(0)
{
}

CSmartICE04DevicesEventsSink::~CSmartICE04DevicesEventsSink()
{
    if(m_pCP)
    {
        if(m_dwCookie)
            m_pCP->Unadvise(m_dwCookie);

        m_pCP->Release();
    }
}

STDMETHODIMP
CSmartICE04DevicesEventsSink::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04DevicesEvents))
            *ppvObject = static_cast<ISmartICE04DevicesEvents*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04DevicesEventsSink::AddRef()
{
    return(2);
}

STDMETHODIMP_(ULONG)
CSmartICE04DevicesEventsSink::Release()
{
    return(1);
}

STDMETHODIMP
CSmartICE04DevicesEventsSink::OnDeviceAttached(IDispatch*, IDispatch* device)
{
    ISmartICE04* pDevice = NULL;
    if(SUCCEEDED(device->QueryInterface(IID_ISmartICE04, (void**)&pDevice)))
    {
        AddDevice(pDevice);
        pDevice->Release();
    }
    return(S_OK);
}

STDMETHODIMP
CSmartICE04DevicesEventsSink::OnDeviceDetached(IDispatch*, IDispatch* device)
{
    ISmartICE04* pDevice = NULL;
    if(SUCCEEDED(device->QueryInterface(IID_ISmartICE04, (void**)&pDevice)))
    {
        RemoveDevice(pDevice);
        pDevice->Release();
    }
    return(S_OK);
}

HRESULT
CSmartICE04DevicesEventsSink::Connect()
{
    IConnectionPointContainer*  pCPC = NULL;
    ISmartICE04Devices*         pDevices;
    IConnectionPoint*           pCP = NULL;
    HRESULT                     hr = S_OK;
    DWORD                       dwCookie;

    if(NULL == (pDevices = new (std::nothrow) CSmartICE04Devices(&hr)))
        return(E_OUTOFMEMORY);
    
    if(SUCCEEDED(hr))
    {
        if(SUCCEEDED((hr = pDevices->QueryInterface(IID_IConnectionPointContainer, (void**)&pCPC))))
        {
            if(SUCCEEDED((hr = pCPC->FindConnectionPoint(IID_ISmartICE04DevicesEvents, &pCP))))
            {
                if(SUCCEEDED((hr = pCP->Advise(GetUnknown(), &dwCookie))))
                {
                    Disconnect();
                    m_dwCookie = dwCookie;
                    m_pCP = pCP;
                    m_pCP->AddRef();
                }
                pCP->Release();
            }
            pCPC->Release();
        }
    }

    pDevices->Release();
    return(hr);
}

HRESULT
CSmartICE04DevicesEventsSink::Disconnect()
{
    HRESULT hr = S_FALSE;

    if(m_pCP)
    {
        if(m_dwCookie)
        {
            hr = m_pCP->Unadvise(m_dwCookie);
            m_dwCookie = 0;
        }
        m_pCP->Release();
        m_pCP = NULL;
    }

    return(hr);
}

HRESULT
CSmartICE04DevicesEventsSink::EnumDevices()
{
    IConnectionPointContainer*  pCPC = NULL;
    ISmartICE04Devices*         pDevices = NULL;
    VARIANT                     varIndex, varDevice;
    HRESULT                     hr;

    if(!m_pCP)
        return(E_FAIL);

    if(FAILED((hr = m_pCP->GetConnectionPointContainer(&pCPC))))
        return(hr);

    if(SUCCEEDED((hr = pCPC->QueryInterface(IID_ISmartICE04Devices, (void**)&pDevices))))
    {
        VariantInit(&varIndex);
        VariantInit(&varDevice);
        V_VT(&varIndex) = VT_I4;

        for(V_I4(&varIndex) = 0; SUCCEEDED(pDevices->get_Item(varIndex, &varDevice)); ++V_I4(&varIndex))
        {
            if(SUCCEEDED(VariantChangeTypeEx(&varDevice, &varDevice, CThreadLocale::getLCID(), 0, VT_DISPATCH)))
                if(V_DISPATCH(&varDevice))
                    OnDeviceAttached(pDevices, V_DISPATCH(&varDevice));

            VariantClear(&varDevice);
        }

        pDevices->Release();
    }

    pCPC->Release();
    return(hr);
}