#include "precomp.h"

STDMETHODIMP
CSmartICE04Events::GetConnectionInterface(IID* pIID)
{
    if(IsBadWritePtrAuto(pIID))
        return(E_POINTER);

    *pIID = IID_ISmartICE04Events;
    return(S_OK);
}

void
CSmartICE04Events::FireEvent(DISPID dispidEvt, const VARIANT* param) const
{
    CONNECTDATA cd;
    IDispatch*  pSender = NULL;

    m_pCPC->QueryInterface(IID_IDispatch, (void**)&pSender);

    for(UINT x = 0; m_Connections.Get(x, &cd); ++x)
    {
        switch(dispidEvt)
        {
            case DISPID_SMARTICE04_ONTARGETVOLTAGECHANGED:
                static_cast<ISmartICE04Events*>(cd.pUnk)->OnTargetVoltageChanged(pSender, V_BOOL(param));
                break;
            case DISPID_SMARTICE04_ONTARGETCHANGED:
                static_cast<ISmartICE04Events*>(cd.pUnk)->OnTargetChanged(pSender, static_cast<enum ChipId>(V_I2(param)));
                break;
            case DISPID_SMARTICE04_ONDEBUGMODECHANGED:
                static_cast<ISmartICE04Events*>(cd.pUnk)->OnDebugModeChanged(pSender, V_BOOL(param));
                break;
            default:
#if defined(_MSC_VER) && !defined(_DEBUG)
                __assume(0);
#else
#pragma warning(suppress: 4365)
                assert(!"Unhandled case in switch");
                break;
#endif
        }
    }
    
    if(pSender)
        pSender->Release();
}