#include "precomp.h"

CSmartICE04EventsSink::CSmartICE04EventsSink(CDeviceList* pDevList)
    : m_pDevList(pDevList), m_pCP(NULL), m_dwCookie(0)
{
}

CSmartICE04EventsSink::~CSmartICE04EventsSink()
{
    if(m_pCP)
    {
        if(m_dwCookie)
            m_pCP->Unadvise(m_dwCookie);

        m_pCP->Release();
    }
}

STDMETHODIMP
CSmartICE04EventsSink::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_ISmartICE04Events))
            *ppvObject = static_cast<ISmartICE04Events*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04EventsSink::AddRef()
{
    return(2);
}

STDMETHODIMP_(ULONG)
CSmartICE04EventsSink::Release()
{
    return(1);
}

STDMETHODIMP
CSmartICE04EventsSink::OnTargetVoltageChanged(IDispatch* sender, VARIANT_BOOL)
{
    ISmartICE04* pDevice = NULL;
    long voltage = 0;
    TCHAR sz[32];

    if(SUCCEEDED(sender->QueryInterface(IID_ISmartICE04, (void**)&pDevice)))
    {
        pDevice->get_TargetVoltage(&voltage);
        wsprintf(sz, TEXT("%u mV"), voltage);
        m_pDevList->SetTargetVoltage(pDevice, sz);
        pDevice->Release();
    }
    return(S_OK);
}

STDMETHODIMP
CSmartICE04EventsSink::OnTargetChanged(IDispatch* sender, enum ChipId id)
{
    ISmartICE04* pDevice = NULL;
    ITypeInfo* pTypeInfo = NULL;
    TYPEATTR* pAttr = NULL;
    VARDESC* pDesc;
    VARIANT var;
    UINT idx, cnt;
    BSTR bstr;
    TCHAR sz[32];

    if(FAILED(sender->QueryInterface(IID_ISmartICE04, (void**)&pDevice)))
        return(S_OK);

    if(id == UnknownChip)
    {
        if(LoadString(g_hInst, IDS_NONE, sz, _countof(sz)))
        {
            m_pDevList->SetTarget(pDevice, sz);
            pDevice->Release();
            return(S_OK);
        }
    }

    if(SUCCEEDED(::GetTypeInfoOfChipIdEnumeration(pDevice, &pTypeInfo)))
    {
        if(SUCCEEDED(pTypeInfo->GetTypeAttr(&pAttr)))
        {
            if(pAttr->typekind == TKIND_ENUM)
            {
                VariantInit(&var);

                for(idx = 0; idx < pAttr->cVars; ++idx)
                {
                    pDesc = NULL;

                    if(SUCCEEDED(pTypeInfo->GetVarDesc(idx, &pDesc)))
                    {
                        if(SUCCEEDED(VariantChangeTypeEx(&var, pDesc->lpvarValue, CThreadLocale::getLCID(), 0, VT_I4)))
                        {
                            if(V_I4(&var) == id)
                            {
                                bstr = NULL;
                                if(SUCCEEDED(pTypeInfo->GetNames(pDesc->memid, &bstr, 1, &cnt)))
                                {
                                    m_pDevList->SetTarget(pDevice, bstr);
                                    SysFreeString(bstr);
                                }
                            }
                        }
                        pTypeInfo->ReleaseVarDesc(pDesc);
                        VariantClear(&var);
                    }
                }
            }
            pTypeInfo->ReleaseTypeAttr(pAttr);
        }
        pTypeInfo->Release();
    }

    pDevice->Release();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04EventsSink::OnDebugModeChanged(IDispatch* sender, VARIANT_BOOL isInDebugMode)
{
    ISmartICE04* pDevice = NULL;
    if(SUCCEEDED(sender->QueryInterface(IID_ISmartICE04, (void**)&pDevice)))
    {
        TCHAR sz[32];

        if(LoadString(g_hInst, static_cast<UINT>((isInDebugMode) ? IDS_DEBUGMODE : IDS_IDLE), sz, _countof(sz)))
            m_pDevList->SetDebugMode(pDevice, sz);

        pDevice->Release();
    }
    return(S_OK);
}

HRESULT
CSmartICE04EventsSink::ConnectTo(IUnknown* pUnk)
{
    IConnectionPointContainer*  pCPC = NULL;
    IConnectionPoint*           pCP = NULL;
    HRESULT                     hr = S_OK;
    DWORD                       dwCookie = 0;

    if(SUCCEEDED((hr = pUnk->QueryInterface(IID_IConnectionPointContainer, (void**)&pCPC))))
    {
        if(SUCCEEDED((hr = pCPC->FindConnectionPoint(IID_ISmartICE04Events, &pCP))))
        {
            if(SUCCEEDED((hr = pCP->Advise(GetUnknown(), &dwCookie))))
            {
                Disconnect();
                m_dwCookie = dwCookie;
                m_pCP = pCP;
                m_pCP->AddRef();
            }
            pCP->Release();
        }
        pCPC->Release();
    }

    return(hr);
}

HRESULT
CSmartICE04EventsSink::Disconnect()
{
    HRESULT hr = S_FALSE;

    if(m_pCP)
    {
        if(m_dwCookie)
        {
            hr = m_pCP->Unadvise(m_dwCookie);
            m_dwCookie = 0;
        }
        m_pCP->Release();
        m_pCP = NULL;
    }

    return(hr);
}
