#include "precomp.h"

const TCHAR CSmartICE04Progress::m_szSmartICE04Progress[] = TEXT("E1D20737-7D34-4868-A671-2033AAC3FDFE");

CSmartICE04Progress::CSmartICE04Progress(ITypeLib* pTlb, ISmartICE04ProgressDialog* pDlg, HWND hWndParent, HRESULT* phr)
    : m_lRefCount(1), m_pTypeInfo(NULL), m_pDispTypeInfo(NULL), m_hWnd(NULL), m_wndproc(NULL), m_pTaskbarList(NULL), m_uFlags(0)
{
    INITCOMMONCONTROLSEX    iccex;
    HANDLE                  hThread;
    DWORD                   dw;
    TCHAR                   szText[48];
    HWND                    hwndAncestor;
    BSTR                    bstr;
    LCID                    lcid = CThreadLocale::getLCID();
    int                     wPrimaryLangID;

    ::LockServer();
    *phr = S_OK;
    pDlg->get_LCID(&lcid);

    if(!hWndParent)
    {
        m_hWndParent = GetDesktopWindow();
    }
    else if(!IsWindow(hWndParent))
    {
        *phr = E_INVALIDARG;
    }
    else
    {
        m_hWndParent = hWndParent;

        if(NULL != (hwndAncestor = GetAncestor(m_hWndParent, GA_ROOT)))
        {
            m_uFlags = (IsWindowEnabled(hwndAncestor)) ? FLAG_PARENTENABLED : 0U;
            
            if(!GetProp(hwndAncestor, m_szSmartICE04Progress))
            {
                if(SetProp(hwndAncestor, m_szSmartICE04Progress, reinterpret_cast<HANDLE>(this)))
                {
                    m_uFlags |= FLAG_PROPERTYVALID;
                    m_wndproc = reinterpret_cast<WNDPROC>(SetWindowLongPtr(hwndAncestor, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(SubClassProc)));
                }
            }
        }
    }

    if(SUCCEEDED(*phr))
    {
        iccex.dwSize = sizeof(iccex);
        iccex.dwICC = ICC_PROGRESS_CLASS;
        InitCommonControlsEx(&iccex);
        
        if(NULL == (hThread = CreateThread(NULL, 0, ThreadProc, this, 0, &dw)))
        {
            dw = GetLastError();
            *phr = HRESULT_FROM_WIN32(dw);
        }
        else
        {
            while(!IsWindow(m_hWnd))
            {
                if(WAIT_TIMEOUT == WaitForSingleObject(hThread, 1))
                    continue;

                GetExitCodeThread(hThread, &dw);
                *phr = HRESULT_FROM_WIN32(dw);
                break;
            }
        
            CloseHandle(hThread);
        }

        if(IsWindow(m_hWnd))
        {
            wPrimaryLangID = PRIMARYLANGID(lcid);

            if(wPrimaryLangID == LANG_GERMAN)
            {
                CThreadLocale loc(MAKELCID(MAKELANGID(wPrimaryLangID, SUBLANG_DEFAULT), SORT_DEFAULT));

                if(LoadString(g_hInst, IDS_CANCEL, szText, _countof(szText)))
                    SetDlgItemText(m_hWnd, IDCANCEL, szText);
            }
            
            bstr = NULL;
            if(SUCCEEDED((*phr = pDlg->get_Line1(&bstr))))
            {
// m_hWnd has been verified with IsWindow, so the first
// argument is actually *not* 0
#pragma warning(suppress: 6387)
                SetDlgItemTextW(m_hWnd, IDC_LINE1, bstr);
                SysFreeString(bstr);
                bstr = NULL;
                
                if(SUCCEEDED((*phr = pDlg->get_Line2(&bstr))))
                {
// m_hWnd has been verified with IsWindow, so the first
// argument is actually *not* 0
#pragma warning(suppress: 6387)
                    SetDlgItemTextW(m_hWnd, IDC_LINE2, bstr);
                    SysFreeString(bstr);
                    bstr = NULL;
                    
                    if(SUCCEEDED((*phr = pDlg->get_Title(&bstr))))
                    {
// m_hWnd has been verified with IsWindow, so the first
// argument is actually *not* 0
#pragma warning(suppress: 6387)
                        SetWindowTextW(m_hWnd, bstr);
                        SysFreeString(bstr);
                    }
                }
            }
        }
        else if(SUCCEEDED(*phr))
        {
            *phr = E_UNEXPECTED;
        }
    }

    if(SUCCEEDED(*phr))
    {
        if(SUCCEEDED((*phr = pTlb->GetTypeInfoOfGuid(IID_ISmartICE04Progress, &m_pTypeInfo))))
        {
            if(SUCCEEDED((*phr = pTlb->GetTypeInfoOfGuid(DIID_ISmartICE04ProgressDisp, &m_pDispTypeInfo))))
            {
                if(FAILED(CoCreateInstance(CLSID_TaskbarList, NULL, CLSCTX_SERVER, __uuidof(ITaskbarList3), (void**)&m_pTaskbarList)))
                {
                    m_pTaskbarList = NULL;
                }
            }
        }
    }
}

CSmartICE04Progress::~CSmartICE04Progress()
{
    HWND hwndAncestor = GetAncestor(m_hWndParent, GA_ROOT);

    if(m_uFlags & FLAG_PARENTENABLED)
    {
        if(hwndAncestor)
            EnableWindow(hwndAncestor, TRUE);
    }

    if(IsWindow(m_hWnd))
    {
        if(m_pTaskbarList)
            m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NOPROGRESS);

        PostMessage(m_hWnd, WM_CLOSE, 0, 0);
    }

    if(m_uFlags & FLAG_PROPERTYVALID)
    {
        if(hwndAncestor)
        {
            if(m_wndproc)
                SetWindowLongPtr(hwndAncestor, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(m_wndproc));

            RemoveProp(hwndAncestor, m_szSmartICE04Progress);
        }
    }

    if(m_pTaskbarList)
        m_pTaskbarList->Release();
    if(m_pTypeInfo)
        m_pTypeInfo->Release();
    if(m_pDispTypeInfo)
        m_pDispTypeInfo->Release();
    if(IsWindow(m_hWnd))
        PostMessage(m_hWnd, WM_CLOSE, 0, 0);
    
    ::UnlockServer();
}

STDMETHODIMP
CSmartICE04Progress::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, DIID_ISmartICE04ProgressDisp))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04Progress))
            *ppvObject = static_cast<ISmartICE04Progress*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04Progress::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CSmartICE04Progress::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CSmartICE04Progress::GetTypeInfoCount(THIS_ UINT* pctinfo)
{
    if(IsBadWritePtrAuto(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::GetTypeInfo(THIS_ UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = NULL;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pDispTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::GetIDsOfNames(THIS_ REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pDispTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

STDMETHODIMP
CSmartICE04Progress::Invoke(THIS_ DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale   loc(lcid);
    UINT            uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04Progress*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

STDMETHODIMP
CSmartICE04Progress::Start(THIS)
{
    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_START, E_UNEXPECTED));

    DisableParent();
    EnableWindow(GetDlgItem(m_hWnd, IDCANCEL), TRUE);
    SendDlgItemMessage(m_hWnd, IDC_PROGRESSBAR, PBM_SETRANGE32, 0, 100);
    SendDlgItemMessage(m_hWnd, IDC_PROGRESSBAR, PBM_SETPOS, 0, 0);
    ShowWindow(m_hWnd, SW_SHOW);

    if(m_pTaskbarList)
    {
        m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NORMAL);
        m_pTaskbarList->SetProgressValue(m_hWnd, 0, 100);
    }

    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::Stop(THIS)
{
    EnableParent();

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressState(m_hWnd, TBPF_NOPROGRESS);

    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_STOP, E_UNEXPECTED));

    ShowWindow(m_hWnd, SW_HIDE);
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::SetProgress(THIS_ long lCompleted, long lTotal)
{
    HWND hWnd;

    if(!IsWindow(m_hWnd))
        return(RaiseCOMException(DISPID_SMARTICE04_SETPROGRESS, E_UNEXPECTED));

    if(m_pTaskbarList)
        m_pTaskbarList->SetProgressValue(m_hWnd, static_cast<unsigned long>(lCompleted), static_cast<unsigned long>(lTotal));

    if(IsWindowVisible(m_hWnd))
    {
        MSG msg;

        while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    hWnd = GetDlgItem(m_hWnd, IDC_PROGRESSBAR);
    PostMessage(hWnd, PBM_SETRANGE32, 0, lTotal);
    PostMessage(hWnd, PBM_SETPOS, static_cast<WPARAM>(lCompleted), 0);
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::get_Canceled(THIS_ VARIANT_BOOL* pblCanceled)
{
    if(IsBadWritePtrAuto(pblCanceled))
        return(RaiseCOMException(DISPID_SMARTICE04_CANCELED, E_POINTER));

    if(!IsWindow(m_hWnd))
    {
        *pblCanceled = VARIANT_FALSE;
        return(RaiseCOMException(DISPID_SMARTICE04_CANCELED, E_UNEXPECTED));
    }

    if(IsWindowVisible(m_hWnd))
    {
        MSG msg;

        while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    *pblCanceled = (IsWindowEnabled(GetDlgItem(m_hWnd, IDCANCEL))) ? VARIANT_FALSE : VARIANT_TRUE;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04Progress::InterfaceSupportsErrorInfo(THIS_ REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, DIID_ISmartICE04ProgressDisp))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

VOID
CSmartICE04Progress::EnableParent(VOID)
{
    if(m_uFlags & FLAG_PARENTENABLED)
    {
        HWND hWnd = GetAncestor(m_hWndParent, GA_ROOT);

        if(hWnd)
            EnableWindow(hWnd, TRUE);
    }
}

VOID
CSmartICE04Progress::DisableParent(VOID)
{
    if(m_uFlags & FLAG_PARENTENABLED)
    {
        HWND hWnd = GetAncestor(m_hWndParent, GA_ROOT);

        if(hWnd)
            EnableWindow(hWnd, FALSE);
    }
}

DWORD WINAPI
CSmartICE04Progress::ThreadProc(LPVOID lpParameter)
{
    CSmartICE04Progress*    pProgress = reinterpret_cast<CSmartICE04Progress*>(lpParameter);
    RECT                    rc;
    MSG                     msg;

    if(NULL == (pProgress->m_hWnd = CreateDialogParam(g_hInst, MAKEINTRESOURCE(IDD_PROGRESS), NULL, pProgress->DlgProc, (LPARAM)pProgress)))
        return((DWORD)HRESULT_FROM_WIN32(GetLastError()));

    if(GetWindowRect(pProgress->m_hWndParent, &rc))
        SetWindowPos(pProgress->m_hWnd, NULL, rc.left + GetSystemMetrics(SM_CXBORDER), rc.top + GetSystemMetrics(SM_CYBORDER), 0, 0, SWP_NOOWNERZORDER | SWP_NOSIZE | SWP_NOZORDER);

    while(GetMessage(&msg, NULL, 0, 0))
    {
        if(!IsDialogMessage(pProgress->m_hWnd, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage (&msg);
        }
    }

    return(0);
}

INT_PTR CALLBACK
CSmartICE04Progress::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM)
{
    switch(uMsg)
    {
        case WM_INITDIALOG:
            return(TRUE);
        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case IDCANCEL:
                    EnableWindow(GetDlgItem(hWnd, IDCANCEL), FALSE);
                    return(TRUE);
                default:
                    break;
            }
            break;
        case WM_SYSCOMMAND:
            if(wParam == SC_CLOSE)
            {
                EnableWindow(GetDlgItem(hWnd, IDCANCEL), FALSE);
                return(TRUE);
            }
            break;
        case WM_CLOSE:
            DestroyWindow(hWnd);
            return(TRUE);
        case WM_DESTROY:
            PostQuitMessage(0);
            return(TRUE);
        default:
            break;
    }

    return(FALSE);
}

LRESULT CALLBACK
CSmartICE04Progress::SubClassProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    CSmartICE04Progress* pThis = reinterpret_cast<CSmartICE04Progress*>(GetProp(hWnd, m_szSmartICE04Progress));

    if(!pThis)
        return(DefWindowProc(hWnd, uMsg, wParam, lParam));

    if(IsWindowVisible(pThis->m_hWnd))
    {
        if((uMsg == WM_ACTIVATE) || (wParam != WA_INACTIVE))
        {
            SetForegroundWindow(pThis->m_hWnd);
        }
        else if((uMsg == WM_ACTIVATEAPP) && (wParam))
        {
            SetForegroundWindow(pThis->m_hWnd);
        }
    }

    return(CallWindowProc(pThis->m_wndproc, hWnd, uMsg, wParam, lParam));
}
