#include "precomp.h"

CSmartICE04ProgressDialog::CSmartICE04ProgressDialog(HRESULT* phr)
    : m_lRefCount(1), m_pTypeInfo(NULL), m_bstrTitle(NULL), m_bstrLine1(NULL), m_bstrLine2(NULL), m_lcid(LOCALE_NEUTRAL)
{
    ITypeLib*   pTlb = NULL;
    HRESULT     hr;

    ::LockServer();
    m_bstrTitle = SysAllocString(L"SmartICE04");
    m_bstrLine1 = SysAllocString(L"");
    m_bstrLine2 = SysAllocString(L"");
    m_lcid = CThreadLocale::getLCID();

    if(!m_bstrTitle || !m_bstrLine1 || !m_bstrLine2)
    {
        *phr = E_OUTOFMEMORY;
    }
    else if(SUCCEEDED((hr = LoadRegTypeLib(LIBID_SmartICE04Lib, 1, 0, LOCALE_USER_DEFAULT, &pTlb))))
    {
        hr = pTlb->GetTypeInfoOfGuid(IID_ISmartICE04ProgressDialog, &m_pTypeInfo);
        pTlb->Release();
    }
}

CSmartICE04ProgressDialog::~CSmartICE04ProgressDialog()
{
    SysFreeString(m_bstrLine2);
    SysFreeString(m_bstrLine1);
    SysFreeString(m_bstrTitle);

    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    ::UnlockServer();
}

STDMETHODIMP
CSmartICE04ProgressDialog::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04ProgressDialog))
            *ppvObject = static_cast<ISmartICE04ProgressDialog*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04ProgressDialog::AddRef(THIS)
{
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CSmartICE04ProgressDialog::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

STDMETHODIMP
CSmartICE04ProgressDialog::GetTypeInfoCount(THIS_ UINT* pctinfo)
{
    if(IsBadWritePtrAuto(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04ProgressDialog::GetTypeInfo(THIS_ UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = NULL;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04ProgressDialog::GetIDsOfNames(THIS_ REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

STDMETHODIMP
CSmartICE04ProgressDialog::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04ProgressDialog*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::get_Title(BSTR *Title)
{
    HRESULT hr = CopyBSTR(Title, m_bstrTitle);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_TITLE, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::put_Title(BSTR Title)
{
    HRESULT hr = ChangeBSTR(&m_bstrTitle, Title);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_TITLE, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::get_Line1(BSTR *line)
{
    HRESULT hr = CopyBSTR(line, m_bstrLine1);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE1, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::put_Line1(BSTR line)
{
    HRESULT hr = ChangeBSTR(&m_bstrLine1, line);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE1, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::get_Line2(BSTR *line)
{
    HRESULT hr = CopyBSTR(line, m_bstrLine2);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE2, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::put_Line2(BSTR line)
{
    HRESULT hr = ChangeBSTR(&m_bstrLine2, line);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_LINE2, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::get_LCID(LCID *lcid)
{
    if(IsBadWritePtrAuto(lcid))
        return(RaiseCOMException(DISPID_SMARTICE04_LCID, E_POINTER));

    *lcid = m_lcid;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04ProgressDialog::put_LCID(LCID lcid)
{
    CThreadLocale loc(lcid);
    m_lcid = loc.getLCID();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04ProgressDialog::Create(OAHWND hWndParent, IDispatch **ppProgress)
{
    CSmartICE04Progress*    pProgress;
    ITypeLib*               pTlb = NULL;
    HRESULT                 hr;
    UINT                    uIndex;

    if(IsBadWritePtrAuto(ppProgress))
        return(RaiseCOMException(DISPID_SMARTICE04_CREATE, E_POINTER));

    *ppProgress = NULL;

    if(FAILED((hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &uIndex))))
        return(RaiseCOMException(DISPID_SMARTICE04_CREATE, hr));
    
    if(NULL == (pProgress = new (std::nothrow) CSmartICE04Progress(pTlb, this, reinterpret_cast<HWND>(hWndParent), &hr)))
    {
        hr = E_OUTOFMEMORY;
    }
    else
    {
        if(SUCCEEDED(hr))
            hr = pProgress->QueryInterface(IID_IDispatch, (void**)ppProgress);

        pProgress->Release();
    }

    pTlb->Release();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_CREATE, hr));
}

STDMETHODIMP
CSmartICE04ProgressDialog::GetClassInfo(ITypeInfo** ppTInfo)
{
    ITypeLib*   pTlb = NULL;
    HRESULT     hr;
    UINT        idx;

    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    *ppTInfo = NULL;

    if(SUCCEEDED((hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &idx))))
    {
        hr = pTlb->GetTypeInfoOfGuid(CLSID_SmartICE04ProgressDialog, ppTInfo);
        pTlb->Release();
    }

    return(hr);
}

STDMETHODIMP
CSmartICE04ProgressDialog::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04ProgressDialog))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

HRESULT
CSmartICE04ProgressDialog::ChangeBSTR(BSTR *pbstrOld, BSTR bstrNew)
{
    BSTR bstr;

    if(!bstrNew)
        return(E_INVALIDARG);

    if(NULL == (bstr = SysAllocString(bstrNew)))
        return(E_OUTOFMEMORY);

    SysFreeString(*pbstrOld);
    *pbstrOld = bstr;
    return(S_OK);
}

HRESULT
CSmartICE04ProgressDialog::CopyBSTR(BSTR *pbstrDest, BSTR bstrSrc)
{
    if(IsBadWritePtrAuto(pbstrDest))
        return(E_POINTER);

    if(NULL == (*pbstrDest = SysAllocString(bstrSrc)))
        return(E_OUTOFMEMORY);

    return(S_OK);
}