#include "precomp.h"

CTargetSetup::CTargetSetup() throw()
    : m_hWnd(NULL)
{
    m_Sink.SetParent(this);
    m_Sink.Connect();
}

CTargetSetup::~CTargetSetup() throw()
{
}

HRESULT
CTargetSetup::DoModal(HWND hWndParent)
{
    CTargetSetup dlg;
    CActCtx ctx(IDR_MANIFEST);
    INT_PTR n = DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_TARGETSETUP), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg));
    return((n == -1) ? HRESULT_FROM_WIN32(GetLastError()) : ((n == IDOK) ? S_OK : S_FALSE));
}

INT_PTR
CTargetSetup::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    int idx;

    m_hWnd = hWnd;
// MonConf.Opt = FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY | CACHE_DATA | CACHE_XDATA | CACHE_CODE;
    CheckDlgButton(hWnd, IDC_CACHE_DATA, static_cast<UINT>((MonConf.Opt & CACHE_DATA) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_CACHE_XDATA, static_cast<UINT>((MonConf.Opt & CACHE_XDATA) ? BST_CHECKED : BST_UNCHECKED));
    CheckDlgButton(hWnd, IDC_CACHE_CODE, static_cast<UINT>((MonConf.Opt & CACHE_CODE) ? BST_CHECKED : BST_UNCHECKED));
    CheckRadioButton(hWnd, IDC_DA_USEFIRST, IDC_DA_SPECIFIC, ((MonConf.Opt & DA_SPECIFIC) ? IDC_DA_SPECIFIC : ((MonConf.Opt & DA_ASKUSER) ? IDC_DA_ASKUSER : IDC_DA_USEFIRST)));
    SetDlgItemTextA(hWnd, IDC_DA, MonConf.idDevice);
    EnableWindow(GetDlgItem(hWnd, IDC_DA), (BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC)));
    m_Sink.EnumDevices();

    if(0 <= (idx = static_cast<int>(SendDlgItemMessageA(hWnd, IDC_DA, CB_FINDSTRINGEXACT, 0, reinterpret_cast<LPARAM>(MonConf.idDevice)))))
        SendDlgItemMessage(hWnd, IDC_DA, CB_SETCURSEL, static_cast<WPARAM>(idx), 0);

    return(TRUE);
}

INT_PTR
CTargetSetup::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDC_DA_USEFIRST:
        case IDC_DA_ASKUSER:
        case IDC_DA_SPECIFIC:
            if(HIWORD(wParam) == BN_CLICKED)
                EnableWindow(GetDlgItem(hWnd, IDC_DA), (BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC)));

            break;
        case IDCANCEL:
            OnWmInitDialog(hWnd, 0, reinterpret_cast<LPARAM>(this));
        case IDOK:
            MonConf.Opt &= ~(CACHE_DATA | CACHE_XDATA | CACHE_CODE | DA_ASKUSER | DA_SPECIFIC);

            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_DATA))
                MonConf.Opt |= CACHE_DATA;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_XDATA))
                MonConf.Opt |= CACHE_XDATA;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_CACHE_CODE))
                MonConf.Opt |= CACHE_CODE;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_ASKUSER))
                MonConf.Opt |= DA_ASKUSER;
            if(BST_CHECKED == IsDlgButtonChecked(hWnd, IDC_DA_SPECIFIC))
                MonConf.Opt |= DA_SPECIFIC;

            GetDlgItemTextA(hWnd, IDC_DA, MonConf.idDevice, _countof(MonConf.idDevice));
            EndDialog(hWnd, LOWORD(wParam));
            break;
        default:
            return(FALSE);
    }

    return(TRUE);
}

INT_PTR CALLBACK
CTargetSetup::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
#pragma warning(push)
#pragma warning(disable: 4820)
    static const struct
    {
        UINT uMsg;
        INT_PTR (CTargetSetup::*pfn)(HWND hWnd, WPARAM wParam, LPARAM lParam);
    }
    map[] =
    {
        {   WM_COMMAND,     &OnWmCommand    }
    };
#pragma warning(pop)

    for(unsigned n = 0; n < _countof(map); ++n)
        if(map[n].uMsg == uMsg)
            return((reinterpret_cast<CTargetSetup*>(GetWindowLongPtr(hWnd, DWLP_USER))->*map[n].pfn)(hWnd, wParam, lParam));

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return(reinterpret_cast<CTargetSetup*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam));
    }

    return(FALSE);
}

void
CTargetSetup::AddDevice(ISmartICE04* pDevice)
{
    BSTR sernum = NULL;

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    if(0 > SendDlgItemMessage(m_hWnd, IDC_DA, CB_FINDSTRINGEXACT, static_cast<WPARAM>(-1), reinterpret_cast<LPARAM>(sernum)))
        SendDlgItemMessageW(m_hWnd, IDC_DA, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(sernum));

    SysFreeString(sernum);
}

void
CTargetSetup::RemoveDevice(ISmartICE04* pDevice)
{
    BSTR sernum = NULL;
    int idx;

    if(FAILED(pDevice->get_SerialNumber(&sernum)))
        return;

    if(CB_ERR != (idx = static_cast<int>(SendDlgItemMessageW(m_hWnd, IDC_DA, CB_FINDSTRINGEXACT, static_cast<WPARAM>(-1), reinterpret_cast<LPARAM>(sernum)))))
        SendDlgItemMessageW(m_hWnd, IDC_DA, CB_DELETESTRING, static_cast<WPARAM>(idx), 0);

    SysFreeString(sernum);
}
