#include "precomp.h"

HRESULT RaiseCOMException(IDispatch* pDispatch, DISPID dispid, HRESULT hr) throw()
{
    IProvideClassInfo*  pPCI        = NULL;
    ICreateErrorInfo*   pCreateErr  = NULL;
    IErrorInfo*         pError      = NULL;
    ITypeInfo*          pInfo       = NULL;
    TYPEATTR*           ptattr      = NULL;
    LPOLESTR            pSource     = NULL;
    DWORD               dwHelpContext;
    DWORD               dwMessageLen = 0;
    TCHAR               szError[1024];
    BSTR                bstrHelpFile;

    if((HRESULT_FACILITY(hr) == FACILITY_ITF) && (HRESULT_CODE(hr) >= 0x0200))
    {
        CThreadLocale loc(MAKELCID(MAKELANGID(PRIMARYLANGID(CThreadLocale::getLANGID()), SUBLANG_DEFAULT), SORT_DEFAULT));
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_HMODULE, g_hInst, static_cast<DWORD>(hr), 0, szError, _countof(szError), NULL);
    }
    else
    {
        dwMessageLen = FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM, NULL, static_cast<DWORD>(hr), 0, szError, _countof(szError), NULL);
    }

    if(!dwMessageLen)
    {
        #ifdef _DLL
          #ifdef UNICODE
            swprintf(szError, _countof(szError) - 1, L"Unknown error: 0x%08X", hr);
          #else
            sprintf(szError, "Unknown error: 0x%08X", hr);
          #endif
        #else
          wsprintf(szError, TEXT("Unknown error: 0x%08X"), hr);
        #endif
    }

    if(SUCCEEDED(CreateErrorInfo(&pCreateErr)))
    {
        #ifdef UNICODE
          pCreateErr->SetDescription(szError);
        #else
          WCHAR wszError[_countof(szError)];
          MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szError, -1, wszError, _countof(wszError));
          pCreateErr->SetDescription(wszError);
        #endif

        if(SUCCEEDED(pDispatch->QueryInterface(IID_IProvideClassInfo, (void**)&pPCI)))
        {
            if(SUCCEEDED(pPCI->GetClassInfo(&pInfo)))
            {
                if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
                {
                    if(ptattr->wTypeFlags & TYPEFLAG_FCANCREATE)
                    {
                        if(SUCCEEDED(ProgIDFromCLSID(ptattr->guid, &pSource)))
                        {
                            pCreateErr->SetSource(pSource);
                            CoTaskMemFree(pSource);
                        }
                    }
                    pInfo->ReleaseTypeAttr(ptattr);
                    ptattr = NULL;
                }
                pInfo->Release();
                pInfo = NULL;
            }
            pPCI->Release();
        }

        if(SUCCEEDED(pDispatch->GetTypeInfo(0, CThreadLocale::getLCID(), &pInfo)))
        {
            if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
            {
                pCreateErr->SetGUID(ptattr->guid);
                pInfo->ReleaseTypeAttr(ptattr);
            }

            if(SUCCEEDED(pInfo->GetDocumentation(dispid, NULL, NULL, &dwHelpContext, &bstrHelpFile)))
            {
                pCreateErr->SetHelpContext(dwHelpContext);
                pCreateErr->SetHelpFile(bstrHelpFile);
                SysFreeString(bstrHelpFile);
            }

            pInfo->Release();
        }

        if(SUCCEEDED(pCreateErr->QueryInterface(IID_IErrorInfo, (void**)&pError)))
        {
            SetErrorInfo(0, pError);
            pError->Release();
        }

        pCreateErr->Release();
    }

    return(hr);
}
