#include "precomp.h"

// The documention for this function can be found at
// http://msdn.microsoft.com/en-us/library/bb776404(VS.85).aspx
STDAPI DllGetVersion(DLLVERSIONINFO2* pdvi2)
{
    VS_FIXEDFILEINFO*   pvsffi = NULL;
    HRESULT             hr;
    DWORD               dwHandle = 0, dwSize, dwError;
    TCHAR               szFileName[MAX_PATH * 2];
    PVOID               pMem = NULL;
    UINT                uLen;

    if(IsBadReadPtr(&pdvi2->info1.cbSize, sizeof(pdvi2->info1.cbSize)))
        return(E_INVALIDARG);
    if((pdvi2->info1.cbSize != sizeof(*pdvi2)) && (pdvi2->info1.cbSize != sizeof(pdvi2->info1)))
        return(E_INVALIDARG);
    if(IsBadWritePtr(pdvi2, pdvi2->info1.cbSize))
        return(E_INVALIDARG);

    ZeroMemory(&pdvi2->info1.dwMajorVersion, (pdvi2->info1.cbSize - sizeof(pdvi2->info1.cbSize)));
    szFileName[0] = TEXT('\0');

    if(!GetModuleFileName(g_hInst, szFileName, _countof(szFileName)))
        return(E_UNEXPECTED);
    if(0 == (dwSize = GetFileVersionInfoSize(szFileName, &dwHandle)))
        return(E_UNEXPECTED);
    if(NULL == (pMem = new (std::nothrow) BYTE[dwSize]))
        return(E_OUTOFMEMORY);
    if(GetFileVersionInfo(szFileName, dwHandle, dwSize, pMem))
        VerQueryValue(pMem, TEXT("\\"), (PVOID*)&pvsffi, &uLen);

    if(!pvsffi)
    {
        dwError = GetLastError();
        hr = HRESULT_FROM_WIN32(dwError);
    }
    else
    {
        pdvi2->info1.dwMajorVersion = HIWORD(pvsffi->dwFileVersionMS);
        pdvi2->info1.dwMinorVersion = LOWORD(pvsffi->dwFileVersionMS);
        pdvi2->info1.dwBuildNumber  = HIWORD(pvsffi->dwFileVersionLS);
        pdvi2->info1.dwPlatformID   = static_cast<DWORD>((pvsffi->dwFileOS == VOS__WINDOWS32) ? DLLVER_PLATFORM_WINDOWS : DLLVER_PLATFORM_NT);
        hr                          = S_OK;

        if(pdvi2->info1.cbSize >= sizeof(DLLVERSIONINFO2))
        {
            pdvi2->dwFlags    = 0;
            pdvi2->ullVersion = MAKEDLLVERULL(pdvi2->info1.dwMajorVersion, pdvi2->info1.dwMinorVersion, pdvi2->info1.dwBuildNumber, LOWORD(pvsffi->dwFileVersionLS));
        }
    }
    delete [] pMem;
    return(hr);
}
