#include "precomp.h"

struct dbgblk     *pdbg;      // startup values
struct MonConf  MonConf;      // holds target-setup values 
BYTE           NotInMon;      // 1:=Remote Setup Mode via Options-Debug

/*
 * Analyze the command arguments 'pArg' and initialize 'MonConf'
 */

void AnalyzeMonParms (char * /* pPath */, char *pArgs)  {
    char* pToken;
    int tmp;
    char* ptmp =_strdup(pArgs);

    ZeroMemory(&MonConf, sizeof(MonConf));
    MonConf.Opt = FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY | CACHE_CODE;

    for(pToken = strtok(((ptmp) ? ptmp : pArgs), "/"); pToken; pToken = strtok(NULL, "/"))
    {
        switch(*pToken++)
        {
            case 'D':
                TrimA(pToken);

                if(strlen(pToken) < _countof(MonConf.idDevice))
                    strcpy(MonConf.idDevice, pToken);

                break;
            case 'O':                              // some Options
                if(StrToIntExA(pToken, STIF_DEFAULT, &tmp))
                    MonConf.Opt = static_cast<DWORD>(tmp);

                break;
            default:
                break;
        }
    }

    free(ptmp);
}

/*
 * Create an argument string out of current 'MonConf'.  This string
 * will be registered with the current project.
 * Note: the maximum length of the string must not exceed 'ValSize-1'
 *       characters. ValSize is a member of QDLL (in Bom.h). The current
 *       maximum is 1024 characters including the zero terminator.
 */

void WriteMonParms (char *pArgs)  {
  sprintf (pArgs, "/D%s /O%d", MonConf.idDevice, MonConf.Opt);
}


/*
 * Start the Configuration-Parameter Setup Dialog.
 * Note: This dialog *MUST* be modal, modeless is not allowed!
 */

int DoDlgSetup (HWND hWndParent)  {
    return((S_OK == CTargetSetup::DoModal(hWndParent)) ? IDOK : IDCANCEL);
}

/*
 * Start the Configuration-Parameter Flash Download Setup Dialog.
 * Note: this dialog *MUST* be modal, modeless is not allowed !
 */

int DoFDlgSetup (HWND hWndParent)  {
    return((S_OK == CDownloadSetup::DoModal(hWndParent)) ? IDOK : IDCANCEL);
}


#ifdef __cplusplus
 extern "C"  {  // must avoid C++ mangled names here !
#endif

/*
 * Checker Entry for Target-Dll loader
 *  Note: When the uVision2 debugger is startet in target mode, this
 *        function is called first to check the validity of the DLL.
 *
 * The name of this function depends on architecture:
 *        80167: EnumUv3167()
 *        8051:  EnumUv351()
 *        80251: EnumUv3251()
 */

int _EXPO_ EnumUv351 (void *p, DWORD nCode)  {
  switch (nCode)  {
    case 0:          
      break;
    case 1:          // not used
      break;
    case 2:          // register debug block
      pdbg = (struct dbgblk *) p;  // defined in ComTyp.h
      AnalyzeMonParms (pdbg->TargDll, pdbg->TargArgs);  
      return (8051);                     // Ok.
  }
  return (0);                            // Ok.
}

/*
 * This function is required for Remote-Setup by Target-Options-Debug 
 * and Target-Options-Utilities Sheet.
 */

const char szTFlWar[] = {"Flash Download Warning"};
const char szNoFunc[] = {"Nothing to do. All Download Functions are now disabled."};


int _EXPO_ DllUv3Cap (DWORD nCode, void *p)  {
  QDLL    *pQ;
  int     i = 0;
  
  switch (nCode)  {
    case 2:                          // match family
      i = 8051;                      // identify as 8051 Target driver
      break;

    case 100:                        // has Flash-DownLoad Capabilities
      return (1);                    // say 'Yes'

    case 1:                          // Cpu/Target-DLL Settings
      NotInMon = 1;                  // we are just doing remote setup
      pQ = (QDLL *) p;               // refer to Bom.H for 'QDLL'
      AnalyzeMonParms (pQ->pathUv3, pQ->value);
      i = DoDlgSetup(pQ->parent);    // start Target Setup Dialog...
      if (i == IDOK)  {              // take the values...
        WriteMonParms (pQ->value);   // create an ASCII string out of MonConf
        i = 1;
      }
      else i = 0;                    // don't register string in project.
      NotInMon = 0;
      break;

    case 110:                        // Cpu/Target/Flash-DLL Settings
      pQ = (QDLL *) p;
      AnalyzeMonParms (pQ->pathUv3, pQ->value);
      i = DoFDlgSetup(pQ->parent);   // Flash Download Setup Dialog...
      if (i == IDOK)  {
        WriteMonParms (pQ->value);
        if (!(MonConf.Opt & (FLASH_ERASE | FLASH_PROGRAM | FLASH_VERIFY)))
          MessageBoxA(hMfrm, szNoFunc, szTFlWar, MB_OK|MB_ICONEXCLAMATION);
        i = 1;
      }
      else i = 0;                    // don't register MonConf changes...
      break;
  }
  return (i);                        // return value
}


#ifdef __cplusplus
 }              // End 'extern "C"'
#endif
