#include "precomp.h"

CSmartICE04::CSmartICE04(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, HRESULT* phr) throw()
    : m_lRefCount(1), m_pTypeInfo(NULL), m_pParent(pParent), m_wPID(wProductId), m_bstrSerialNumber(NULL), m_pFeatureReport(NULL), m_pInputReport(NULL), m_hDevice(INVALID_HANDLE_VALUE), m_usInputReportLength(0), m_blDetached(FALSE), m_blInDebugMode(FALSE), m_bDebugConfig(0), m_hMutex(NULL), m_ObjectStatus(0)
{
    DWORD   dwError;
    WCHAR   wsz[256];

    m_Events.SetContainer(this);
    m_DispEvents.SetContainer(this);

    if(NULL == (m_pszDeviceName = new (std::nothrow) TCHAR[_tcslen(pszDeviceName) + 1]))
    {
        *phr = E_OUTOFMEMORY;
    }
    else
    {
        _tcscpy(m_pszDeviceName, pszDeviceName);
        *phr = pTlb->GetTypeInfoOfGuid(IID_ISmartICE04, &m_pTypeInfo);
    }

    if(SUCCEEDED(*phr))
    {
        if(!HidD_GetSerialNumberString(hDevice, wsz, sizeof(wsz)))
        {
            if(ERROR_INVALID_USER_BUFFER == (dwError = GetLastError()))
                *phr = E_UNEXPECTED;
            else
                *phr = HRESULT_FROM_WIN32(dwError);
        }

        wsz[_countof(wsz) - 1] = L'\0';

        if(NULL == (m_bstrSerialNumber = SysAllocString(wsz)))
        {
            *phr = E_OUTOFMEMORY;
        }
        else if(NULL == (m_hMutex = OpenMutex(SYNCHRONIZE, FALSE, TEXT("Global\\AAD17982-BD7D-4EFC-8235-E471232296A6"))))
        {
            if(NULL == (m_hMutex = CreateMutex(NULL, FALSE, TEXT("Global\\AAD17982-BD7D-4EFC-8235-E471232296A6"))))
            {
                dwError = GetLastError();
                *phr = HRESULT_FROM_WIN32(dwError);
            }
        }
    }

    if(SUCCEEDED(*phr))
    {
        HidP_InitializeReportForID(HidP_Input, 1, NULL, reinterpret_cast<CHAR*>(&m_report1), sizeof(m_report1));
        m_hDevice = hDevice;
        Refresh();
    }
}

CSmartICE04::~CSmartICE04()
{
    delete [] m_pFeatureReport;
    delete [] m_pInputReport;
    delete [] m_pszDeviceName;
    SysFreeString(m_bstrSerialNumber);

    if(m_hMutex)
    {
        if(m_blInDebugMode)
            ReleaseMutex(m_hMutex);

        CloseHandle(m_hMutex);
    }

    if(INVALID_HANDLE_VALUE != m_hDevice)
        CloseHandle(m_hDevice);

    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    if(m_blDetached)
        m_pParent->Release();
}

STDMETHODIMP
CSmartICE04::QueryInterface(THIS_ REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = NULL;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISmartICE04))
            *ppvObject = static_cast<ISmartICE04*>(this);
        else if(IsEqualIID(riid, IID_IConnectionPointContainer))
            *ppvObject = static_cast<IConnectionPointContainer*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo2))
            *ppvObject = static_cast<IProvideClassInfo2*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            reinterpret_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

STDMETHODIMP_(ULONG)
CSmartICE04::AddRef(THIS)
{
    m_pParent->AddRef();
    return(InterlockedIncrement(&m_lRefCount));
}

STDMETHODIMP_(ULONG)
CSmartICE04::Release(THIS)
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
    {
        delete this;
        return(0);
    }

    m_pParent->Release();
    return(ulResult);
}

STDMETHODIMP
CSmartICE04::GetTypeInfoCount(THIS_ UINT* pctinfo)
{
    if(IsBadWritePtrAuto(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::GetTypeInfo(THIS_ UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = NULL;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::GetIDsOfNames(THIS_ REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

STDMETHODIMP
CSmartICE04::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale   loc(lcid);
    IErrorInfo*     pError;
    VARIANT         var[4];
    HRESULT         hr;
    UINT            idx, uArgErr;

    if(dispIdMember == DISPID_SMARTICE04_SETMEMORY)
    {
        if(!IsEqualIID(riid, IID_NULL))
            return(DISP_E_UNKNOWNINTERFACE);

        if(wFlags != DISPATCH_METHOD)
            return(DISP_E_MEMBERNOTFOUND);

        if(pDispParams->cNamedArgs)
            return(DISP_E_NONAMEDARGS);

        if((pDispParams->cArgs < (_countof(var) - 1)) || (pDispParams->cArgs > (_countof(var) + 1)))
            return(DISP_E_BADPARAMCOUNT);

        for(idx = 0; idx < _countof(var); ++idx)
            VariantInit(&var[idx]);

        if(FAILED(VariantChangeTypeEx(&var[0], &pDispParams->rgvarg[pDispParams->cArgs - 1], CThreadLocale::getLCID(), 0, VT_I4)))
            return(DISP_E_TYPEMISMATCH);

        if(FAILED(VariantChangeTypeEx(&var[1], &pDispParams->rgvarg[pDispParams->cArgs - 2], CThreadLocale::getLCID(), 0, VT_I4)))
            return(DISP_E_TYPEMISMATCH);

        if(V_VT(&pDispParams->rgvarg[pDispParams->cArgs - 3]) == (VT_VARIANT | VT_BYREF))
            var[2] = *V_VARIANTREF(&pDispParams->rgvarg[pDispParams->cArgs - 3]);
        else
            var[2] = pDispParams->rgvarg[pDispParams->cArgs - 3];

        if((!(V_VT(&var[2]) & VT_ARRAY)) && (V_VT(&var[2]) != VT_SAFEARRAY))
            return(DISP_E_TYPEMISMATCH);

        SetErrorInfo(0, NULL);

        if(pDispParams->cArgs >= _countof(var))
        {
            if(FAILED(VariantChangeTypeEx(&var[3], &pDispParams->rgvarg[pDispParams->cArgs - 4], CThreadLocale::getLCID(), 0, VT_BOOL)))
            {
                if(V_VT(&pDispParams->rgvarg[pDispParams->cArgs - 4]) != VT_ERROR)
                    return(DISP_E_TYPEMISMATCH);

                V_VT(&var[3]) = VT_BOOL;
                V_BOOL(&var[3]) = VARIANT_TRUE;
            }

            hr = SetMemory(static_cast<MemorySegment>(V_I4(&var[0])), V_I4(&var[1]), &V_ARRAY(&var[2]), V_BOOL(&var[3]), ((pDispParams->cArgs > _countof(var)) ? &pDispParams->rgvarg[pDispParams->cArgs - 5] : NULL));
        }
        else
        {
            hr = SetMemory(static_cast<MemorySegment>(V_I4(&var[0])), V_I4(&var[1]), &V_ARRAY(&var[2]), VARIANT_TRUE, NULL);
        }

        if(FAILED(hr))
        {
            pError = NULL;

            if(SUCCEEDED(GetErrorInfo(0, &pError)))
            {
                if(pExcepInfo)
                {
                    ZeroMemory(pExcepInfo, sizeof(*pExcepInfo));
                    pError->GetSource(&pExcepInfo->bstrSource);
                    pError->GetDescription(&pExcepInfo->bstrDescription);
                    pError->GetHelpFile(&pExcepInfo->bstrHelpFile);
                    pError->GetHelpContext(&pExcepInfo->dwHelpContext);
                    pExcepInfo->scode = hr;
                }

                pError->Release();
                hr = DISP_E_EXCEPTION;
            }
        }

        return(hr);
    }

    return(m_pTypeInfo->Invoke(static_cast<ISmartICE04*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

STDMETHODIMP
CSmartICE04::get_Parent(IDispatch** ppParent)
{
    HRESULT hr = m_pParent->QueryInterface(IID_IDispatch, (void**)ppParent);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_PARENT, hr));
}

STDMETHODIMP
CSmartICE04::get_ProductId(long* PID)
{
    if(IsBadWritePtrAuto(PID))
        return(RaiseCOMException(DISPID_SMARTICE04_PRODUCTID, E_POINTER));

    *PID = getPID();
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::get_SerialNumber(BSTR* SerNum)
{
    if(IsBadWritePtrAuto(SerNum))
        return(RaiseCOMException(DISPID_SMARTICE04_SERIALNUMBER, E_POINTER));
    
    *SerNum = SysAllocString(m_bstrSerialNumber);
    return((*SerNum) ? S_OK : RaiseCOMException(DISPID_SMARTICE04_SERIALNUMBER, E_OUTOFMEMORY));
}

STDMETHODIMP
CSmartICE04::get_Icon(IDispatch** Picture)
{
    PICTDESC    picdesc;
    HRESULT     hr;

    if(IsBadWritePtrAuto(Picture))
        return(RaiseCOMException(DISPID_SMARTICE04_ICON, E_POINTER));

    *Picture = NULL;

    if(!SetupDiLoadClassIcon(&GUID_DEVCLASS_USB, &picdesc.icon.hicon, NULL))
        return(RaiseCOMException(DISPID_SMARTICE04_ICON, HRESULT_FROM_SETUPAPI(GetLastError())));

    picdesc.cbSizeofstruct  = sizeof(picdesc);
    picdesc.picType         = PICTYPE_ICON;

    if(FAILED((hr = OleCreatePictureIndirect(&picdesc, IID_IDispatch, TRUE, (void**)Picture))))
    {
        DestroyIcon(picdesc.icon.hicon);
        return(RaiseCOMException(DISPID_SMARTICE04_ICON, hr));
    }

    return(S_OK);
}

STDMETHODIMP
CSmartICE04::get_SmallIcon(IDispatch** Picture)
{
    SP_CLASSIMAGELIST_DATA  spimld = { 0 };
    PICTDESC                picdesc;
    HRESULT                 hr;
    INT                     nIndex;

    if(IsBadWritePtrAuto(Picture))
        return(RaiseCOMException(DISPID_SMARTICE04_SMALLICON, E_POINTER));

    *Picture        = NULL;
    spimld.cbSize   = sizeof(spimld);

    if(!SetupDiGetClassImageList(&spimld))
        return(RaiseCOMException(DISPID_SMARTICE04_SMALLICON, HRESULT_FROM_SETUPAPI(GetLastError())));

    if(SetupDiGetClassImageIndex(&spimld, &GUID_DEVCLASS_USB, &nIndex))
    {
        if(NULL != (picdesc.icon.hicon = ImageList_ExtractIcon(NULL, spimld.ImageList, nIndex)))
        {
            picdesc.cbSizeofstruct  = sizeof(picdesc);
            picdesc.picType         = PICTYPE_ICON;

            if(FAILED((hr = OleCreatePictureIndirect(&picdesc, IID_IDispatch, TRUE, (void**)Picture))))
                DestroyIcon(picdesc.icon.hicon);
        }
        else
        {
            hr = HRESULT_FROM_WIN32(GetLastError());
        }
    }
    else
    {
        hr = HRESULT_FROM_SETUPAPI(GetLastError());
    }

    SetupDiDestroyClassImageList(&spimld);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_SMALLICON, hr));
}

STDMETHODIMP
CSmartICE04::get_ChipId(enum ChipId* cid)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(cid))
        return(RaiseCOMException(DISPID_SMARTICE04_CHIPID, E_POINTER));

    if(FAILED((hr = ReadReport1(&report))))
    {
        *cid = UnknownChip;
        return(RaiseCOMException(DISPID_SMARTICE04_CHIPID, hr));
    }

    *cid = static_cast<enum ChipId>(report.rep1.bChipId);
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::get_ChipVersion(long* vers)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(vers))
        return(RaiseCOMException(DISPID_SMARTICE04_CHIPVERSION, E_POINTER));

    if(FAILED((hr = ReadReport1(&report))))
    {
        *vers = 0;
        return(RaiseCOMException(DISPID_SMARTICE04_CHIPVERSION, hr));
    }

    *vers = report.rep1.bVersion;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::get_TargetVoltage(long* targetVoltage)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(targetVoltage))
        return(RaiseCOMException(DISPID_SMARTICE04_TARGETVOLTAGE, E_POINTER));

    if(FAILED((hr = ReadReport1(&report))))
    {
        *targetVoltage = 0;
        return(RaiseCOMException(DISPID_SMARTICE04_TARGETVOLTAGE, hr));
    }

    *targetVoltage = report.rep1.wTargetVoltage;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::EnterDebugMode(VARIANT_BOOL enableDebug)
{
    IOREPORT    report;
    HRESULT     hr;
    VARIANT     var;
    DWORD       dwWait;

    if(enableDebug)
    {
        if(m_blInDebugMode)
            return(S_OK);

        if(WAIT_FAILED == (dwWait = WaitForSingleObject(m_hMutex, 0)))
            return(RaiseCOMException(DISPID_SMARTICE04_ENTERDEBUGMODE, HRESULT_FROM_WIN32(GetLastError())));
        else if(dwWait == WAIT_TIMEOUT)
            return(RaiseCOMException(DISPID_SMARTICE04_ENTERDEBUGMODE, SMARTICE04_E_ALREADYUSED));

        if(!(m_report1.bStatus & TargetVoltagePresent))
        {
            hr = SMARTICE04_E_NOTARGETVOLTAGE;
        }
        else if(m_report1.bChipId == UnknownChip)
        {
            hr = SMARTICE04_E_NOTARGET;
        }
        else if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        {
            report.rep2.bCmd    = CMDID_DEBUGMODE;
            report.rep2.param1  = static_cast<WORD>(VARIANT_TRUE);
            if(SUCCEEDED((hr = WriteReport(&report))))
            {
                report.rep2.bCmd = CMDID_HALT;

                if(SUCCEEDED((hr = WriteReport(&report))))
                {
                    m_blInDebugMode = TRUE;
                    VariantInit(&var);
                    V_VT(&var) = VT_BOOL;
                    V_BOOL(&var) = VARIANT_TRUE;
                    m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEBUGMODECHANGED, &var);
                    m_Events.FireEvent(DISPID_SMARTICE04_ONDEBUGMODECHANGED, &var);
                }
                else
                {
                    report.rep2.param1 = static_cast<WORD>(VARIANT_FALSE);
#pragma warning(suppress: 6031)
                    WriteReport(&report);
                }
            }
        }

        if(FAILED(hr))
            ReleaseMutex(m_hMutex);
    }
    else if(m_blInDebugMode)
    {
        if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        {
            report.rep2.bCmd    = CMDID_DEBUGMODE;
            report.rep2.param1  = VARIANT_FALSE;
            
            if(SUCCEEDED((hr = WriteReport(&report))))
            {
                m_blInDebugMode = FALSE;
                VariantInit(&var);
                V_VT(&var) = VT_BOOL;
                V_BOOL(&var) = VARIANT_FALSE;
                m_DispEvents.FireEvent(DISPID_SMARTICE04_ONDEBUGMODECHANGED, &var);
                m_Events.FireEvent(DISPID_SMARTICE04_ONDEBUGMODECHANGED, &var);
                ReleaseMutex(m_hMutex);
            }
        }
    }
    else
    {
        hr = S_OK;
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_ENTERDEBUGMODE, hr));
}

STDMETHODIMP
CSmartICE04::get_IsInDebugMode(VARIANT_BOOL* isInDbgMode)
{
    if(IsBadWritePtrAuto(isInDbgMode))
        return(RaiseCOMException(DISPID_SMARTICE04_ISINDEBUGMODE, E_POINTER));

    *isInDbgMode = (m_blInDebugMode) ? VARIANT_TRUE : VARIANT_FALSE;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::get_DebugConfig(long *configFlags)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(configFlags))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGCONFIG, E_POINTER));

    *configFlags = m_bDebugConfig;

    if(m_blInDebugMode)
    {
        if(FAILED((hr = ReadReport5(&report))))
            return(RaiseCOMException(DISPID_SMARTICE04_DEBUGCONFIG, hr));

        if(!report.rep5.bInDbgMode)
            EnterDebugMode(VARIANT_FALSE);
        else
            *configFlags = report.rep5.bConfig;
    }

    return(S_OK);
}

STDMETHODIMP
CSmartICE04::put_DebugConfig(long configFlags)
{
    IOREPORT    report;
    HRESULT     hr;

    if(configFlags & ~0xFF)
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGCONFIG, E_INVALIDARG));

    if(!m_blInDebugMode)
    {
        hr = S_OK;
    }
    else if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
    {
        report.rep2.bCmd    = CMDID_DEBUGCONFIG;
        report.rep2.param1  = static_cast<WORD>(configFlags);
        hr = WriteReport(&report);
    }

    if(FAILED(hr))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGCONFIG, hr));

    m_bDebugConfig = LOBYTE(configFlags);
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::ChipErase()
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
    {
        hr = SMARTICE04_E_NOTINPGMMODE;
    }
    else if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
    {
        report.rep2.bCmd = CMDID_CHIP_ERASE;
        hr = WriteReport(&report);
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_CHIPERASE, hr));
}

STDMETHODIMP
CSmartICE04::Halt()
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
    {
        hr = SMARTICE04_E_NOTINPGMMODE;
    }
    else if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
    {
        report.rep2.bCmd = CMDID_HALT;
        hr = WriteReport(&report);
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_HALT, hr));
}

STDMETHODIMP
CSmartICE04::Resume()
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
    {
        hr = SMARTICE04_E_NOTINPGMMODE;
    }
    else if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
    {
        report.rep2.bCmd = CMDID_RESUME;
        hr = WriteReport(&report);
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_RESUME, hr));
}

STDMETHODIMP
CSmartICE04::ReadStatus(long *statusFlags)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(statusFlags))
    {
        hr = E_POINTER;
    }
    else
    {
        if(!m_blInDebugMode)
        {
            *statusFlags = 0;
            hr = SMARTICE04_E_NOTINPGMMODE;
        }
        else if(SUCCEEDED((hr = ReadReport5(&report))))
        {
            *statusFlags = report.rep5.bStatus;
        }
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_READSTATUS, hr));
}

STDMETHODIMP
CSmartICE04::DebugInstr(long cmd, long *result)
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGINSTR, SMARTICE04_E_NOTINPGMMODE));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGINSTR, hr));

    report.rep2.bCmd    = CMDID_DEBUG_INSTR;
    report.rep2.param1  = LOWORD(cmd);
    report.rep2.param2  = HIWORD(cmd);

    if((HIBYTE(report.rep2.param2) < 1) || (HIBYTE(report.rep2.param2) > 3))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGINSTR, E_INVALIDARG));

    if(FAILED((hr = WriteReport(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGINSTR, hr));

    if(FAILED((hr = ReadReport5(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_DEBUGINSTR, hr));

    if(result)
        *result = report.rep5.bResult;

    return(S_OK);
}

STDMETHODIMP
CSmartICE04::StepReplace(long cmd, long *result)
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
        return(RaiseCOMException(DISPID_SMARTICE04_STEPREPLACE, SMARTICE04_E_NOTINPGMMODE));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPREPLACE, hr));

    report.rep2.bCmd    = CMDID_STEP_REPLACE;
    report.rep2.param1  = LOWORD(cmd);
    report.rep2.param2  = HIWORD(cmd);

    if((HIBYTE(report.rep2.param2) < 1) || (HIBYTE(report.rep2.param2) > 3))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPREPLACE, E_INVALIDARG));

    if(FAILED((hr = WriteReport(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPREPLACE, hr));

    if(FAILED((hr = ReadReport5(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPREPLACE, hr));

    if(result)
        *result = report.rep5.bResult;

    return(S_OK);
}

STDMETHODIMP
CSmartICE04::StepInstr(long *result)
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
        return(RaiseCOMException(DISPID_SMARTICE04_STEPINSTR, SMARTICE04_E_NOTINPGMMODE));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPINSTR, hr));

    report.rep2.bCmd = CMDID_STEP_INSTR;

    if(FAILED((hr = WriteReport(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPINSTR, hr));

    if(FAILED((hr = ReadReport5(&report))))
        return(RaiseCOMException(DISPID_SMARTICE04_STEPINSTR, hr));

    if(result)
        *result = report.rep5.bResult;

    return(S_OK);
}

STDMETHODIMP
CSmartICE04::SetHwBreakpoint(long brkpntNumber, VARIANT_BOOL enable, long address)
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
        return(RaiseCOMException(DISPID_SMARTICE04_SETHWBREAKPOINT, SMARTICE04_E_NOTINPGMMODE));

    if((address & ~0x0003FFFF) || (static_cast<unsigned long>(brkpntNumber) > 3))
        return(RaiseCOMException(DISPID_SMARTICE04_SETHWBREAKPOINT, E_INVALIDARG));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 2, NULL, reinterpret_cast<CHAR*>(&report.rep2), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETHWBREAKPOINT, hr));

    report.rep2.bCmd    = CMDID_SET_HW_BRKPNT;
    report.rep2.param1  = LOWORD(address);
    report.rep2.param2  = MAKEWORD(HIWORD(address), (brkpntNumber | ((enable) ? 0x80 : 0x00)));
    hr = WriteReport(&report);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_SETHWBREAKPOINT, hr));
}

STDMETHODIMP
CSmartICE04::GetPc(long *pc)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(pc))
    {
        hr = E_POINTER;
    }
    else
    {
        if(!m_blInDebugMode)
        {
            *pc = 0;
            hr = SMARTICE04_E_NOTINPGMMODE;
        }
        else if(SUCCEEDED((hr = ReadReport6(&report))))
        {
            *pc = report.rep6.pc;
        }
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_GETPC, hr));
}

STDMETHODIMP
CSmartICE04::ReadLockBits(long *lockbits)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(lockbits))
    {
        hr = E_POINTER;
    }
    else
    {
        if(!m_blInDebugMode)
        {
            *lockbits = -1;
            hr = SMARTICE04_E_NOTINPGMMODE;
        }
        else if(SUCCEEDED((hr = ReadReport7(&report))))
        {
            *lockbits = report.rep7.bLockBits[0];
        }
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_READLOCKBITS, hr));
}

STDMETHODIMP
CSmartICE04::WriteLockBits(long lockbits)
{
    IOREPORT    report;
    HRESULT     hr;

    if(!m_blInDebugMode)
        return(RaiseCOMException(DISPID_SMARTICE04_WRITELOCKBITS, SMARTICE04_E_NOTINPGMMODE));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 7, NULL, reinterpret_cast<CHAR*>(&report.rep7), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_WRITELOCKBITS, hr));

    report.rep7.bLockBits[0] = LOBYTE(lockbits);
    FillMemory(report.rep7.bReserved, sizeof(report.rep7.bReserved), 0xFF);
    hr = WriteReport(&report);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_WRITELOCKBITS, hr));
}

STDMETHODIMP
CSmartICE04::GetMemory(MemorySegment seg, long startAddress, long len, VARIANT* progressItf, SAFEARRAY **data)
{
    CSmartICE04ProgressDummy    progressDummy;
    ISmartICE04Progress*        pProgress = NULL;
    SAFEARRAYBOUND              sab;
    VARIANT_BOOL                vblCanceled = VARIANT_FALSE;
    IOREPORT                    report;
    HRESULT                     hr;
    DWORD                       dw;
    BYTE*                       pData;
    long                        status;

    if(IsBadWritePtrAuto(data))
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, E_POINTER));

    *data = NULL;

    if(len < 0)
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, E_INVALIDARG));

    if(FAILED((hr = ReadStatus(&status))))
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, SMARTICE04_E_NOTINPGMMODE));

    if(!(status & CpuHalted))
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, SMARTICE04_E_TARGETNOTHALTED));

    if(status & DebugLocked)
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, SMARTICE04_E_TARGETLOCKED));

    if(static_cast<unsigned long>(seg) > segIData)
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, E_INVALIDARG));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 3, NULL, reinterpret_cast<CHAR*>(&report.rep3), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, hr));

    if(FAILED((hr = ExtractProgressItf(progressItf, &progressDummy, &pProgress))))
        return(RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, hr));

    report.rep3.bSegment    = static_cast<BYTE>(seg);
    report.rep3.uAddress    = static_cast<unsigned long>(startAddress);
    report.rep3.uLen        = static_cast<unsigned long>(len);
    report.rep3.blRead      = TRUE;

    m_ObjectStatus |= CSMARTICE04_STATUS_GETMEMORY;

    if(!(m_ObjectStatus & CSMARTICE04_STATUS_VERIFY))
        pProgress->Start();

    if(SUCCEEDED((hr = WriteReport(&report))))
    {
        sab.cElements   = static_cast<ULONG>(len);
        sab.lLbound     = 0;

        if(m_ObjectStatus & CSMARTICE04_STATUS_VERIFY)
            len *= 2;

        if(NULL == (*data = SafeArrayCreate(VT_UI1, 1, &sab)))
        {
            hr = E_OUTOFMEMORY;
        }
        else
        {
            if(SUCCEEDED((hr = SafeArrayAccessData(*data, (void**)&pData))))
            {
                while(sab.cElements && SUCCEEDED(hr))
                {
                    if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 4, NULL, reinterpret_cast<CHAR*>(&report.rep3), sizeof(report))))))
                    {
                        if(!ReadFile(m_hDevice, &report.rep4, sizeof(report.rep4), &dw, NULL))
                        {
                            dw = GetLastError();
                            hr = HRESULT_FROM_WIN32(dw);
                        }
                        else if(dw != sizeof(report.rep4))
                        {
                            hr = E_FAIL;
                        }
                        else if(report.rep4.bReportId == 0x04)
                        {
                            if(report.rep4.bLength == 0xFF)
                            {
                                hr = SMARTICE04_E_COMMUNICATIONFAILURE;
                            }
                            else
                            {
                                dw = min(report.rep4.bLength, sab.cElements);
                                CopyMemory(pData, report.rep4.bData, dw);
                                pData += dw;
                                sab.cElements -= dw;
                                pProgress->SetProgress(static_cast<long>(len - sab.cElements), len);

                                if((dw < sizeof(report.rep4.bData)) && (sab.cElements))
                                {
                                    hr = E_FAIL;
                                }
                                else
                                {
                                    pProgress->SetProgress(static_cast<long>(len - sab.cElements), len);
                                    pProgress->get_Canceled(&vblCanceled);

                                    if(vblCanceled)
                                    {
                                        report.rep3.bSegment    = static_cast<BYTE>(seg);
                                        report.rep3.uAddress    = static_cast<unsigned long>(startAddress);
                                        report.rep3.uLen        = 0;
                                        report.rep3.blRead      = TRUE;
                                        HidP_InitializeReportForID(HidP_Output, 3, NULL, reinterpret_cast<CHAR*>(&report.rep3), sizeof(report));
#pragma warning(suppress: 6031)
                                        WriteReport(&report);
                                        hr = HRESULT_FROM_WIN32(ERROR_CANCELLED);
                                    }
                                }
                            }
                        }
                    }
                }

                SafeArrayUnaccessData(*data);
            }

            if(FAILED(hr))
            {
                SafeArrayDestroy(*data);
                *data = NULL;
            }
        }
    }

    if(!(m_ObjectStatus & CSMARTICE04_STATUS_VERIFY))
        pProgress->Stop();

    pProgress->Release();
    m_ObjectStatus &= ~CSMARTICE04_STATUS_GETMEMORY;
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_GETMEMORY, hr));
}

STDMETHODIMP
CSmartICE04::SetMemory(MemorySegment seg, long startAddress, SAFEARRAY **data, VARIANT_BOOL verify, VARIANT* progressItf)
{
    CSmartICE04ProgressDummy    progressDummy;
    ISmartICE04Progress*        pProgress = NULL;
    VARIANT_BOOL                vblCanceled = VARIANT_FALSE;
    SAFEARRAY*                  psa;
    IOREPORT                    report;
    VARTYPE                     vt;
    HRESULT                     hr;
    VARIANT                     var;
    void*                       pData1;
    void*                       pData2;
    LONG                        lLBound, lUBound;
    long                        status, len, total;

    if(IsBadReadPtrAuto(data))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, E_POINTER));

    if(FAILED((hr = ReadStatus(&status))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, SMARTICE04_E_NOTINPGMMODE));

    if(!(status & CpuHalted))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, SMARTICE04_E_TARGETNOTHALTED));

    if(status & DebugLocked)
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, SMARTICE04_E_TARGETLOCKED));

    if(static_cast<unsigned long>(seg) > segIData)
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, E_INVALIDARG));

    if(FAILED((hr = SafeArrayGetLBound(*data, 1, &lLBound))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));

    if(FAILED((hr = SafeArrayGetUBound(*data, 1, &lUBound))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));

    if(lLBound > lUBound)
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, E_INVALIDARG));

    if(FAILED((hr = SafeArrayGetVartype(*data, &vt))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));

    if(FAILED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 3, NULL, reinterpret_cast<CHAR*>(&report.rep3), sizeof(report))))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));

    if(FAILED((hr = ExtractProgressItf(progressItf, &progressDummy, &pProgress))))
        return(RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));

    lUBound -= lLBound - 1;
    len = lUBound;

    report.rep3.bSegment    = static_cast<BYTE>(seg);
    report.rep3.uAddress    = static_cast<unsigned long>(startAddress);
    report.rep3.uLen        = static_cast<unsigned long>(lUBound);
    report.rep3.blRead      = FALSE;

    if(verify)
    {
        m_ObjectStatus |= (CSMARTICE04_STATUS_SETMEMORY | CSMARTICE04_STATUS_VERIFY);
        total = len * 2;
    }
    else
    {
        m_ObjectStatus |= CSMARTICE04_STATUS_SETMEMORY;
        total = len;
    }

    pProgress->Start();

    if(SUCCEEDED((hr = WriteReport(&report))))
    {
        if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Output, 4, NULL, reinterpret_cast<CHAR*>(&report.rep4), sizeof(report))))))
        {
            if(SUCCEEDED((hr = SafeArrayAccessData(*data, &pData1))))
            {
                pData2 = pData1;

                if((vt == VT_UI1) || (vt == VT_I1))
                {
                    for(; lUBound; lUBound -= report.rep4.bLength)
                    {
                        report.rep4.bLength = static_cast<BYTE>(min(sizeof(report.rep4.bData), lUBound));
                        CopyMemory(report.rep4.bData, pData1, report.rep4.bLength);
                        pData1 = reinterpret_cast<BYTE*>(pData1) + report.rep4.bLength;

                        if(FAILED((hr = WriteReport(&report))))
                            break;

                        pProgress->SetProgress(len - lUBound, total);
                        pProgress->get_Canceled(&vblCanceled);

                        if(vblCanceled)
                        {
                            hr = HRESULT_FROM_WIN32(ERROR_CANCELLED);
                            break;
                        }
                    }

                    if((verify) && (SUCCEEDED(hr)))
                    {
                        if(SUCCEEDED((hr = GetMemory(seg, startAddress, len, progressItf, &psa))))
                        {
                            if(SUCCEEDED((hr = SafeArrayAccessData(psa, &pData1))))
                            {
                                hr = (memcmp(pData1, pData2, static_cast<size_t>(len))) ? SMARTICE04_E_VERIFYFAILED : S_OK;
                                SafeArrayUnaccessData(psa);
                            }
                            SafeArrayDestroy(psa);
                        }
                    }
                }
                else if(vt == VT_VARIANT)
                {
                    VariantInit(&var);

                    while((SUCCEEDED(hr)) && (lUBound))
                    {
                        for(report.rep4.bLength = 0; ((lUBound) && (report.rep4.bLength < sizeof(report.rep4.bData))); --lUBound)
                        {
                            if(FAILED((hr = VariantChangeTypeEx(&var, reinterpret_cast<VARIANT*>(pData1), CThreadLocale::getLCID(), 0, VT_UI1))))
                                break;

                            pData1 = reinterpret_cast<VARIANT*>(pData1) + 1;
                            report.rep4.bData[report.rep4.bLength++] = V_UI1(&var);
                        }

                        if(SUCCEEDED(hr))
                        {
                            if(SUCCEEDED((hr = WriteReport(&report))))
                            {
                                pProgress->SetProgress(len - lUBound, total);
                                pProgress->get_Canceled(&vblCanceled);

                                if(vblCanceled)
                                    hr = HRESULT_FROM_WIN32(ERROR_CANCELLED);
                            }
                        }
                    }

                    if((verify) && (SUCCEEDED(hr)))
                    {
                        if(SUCCEEDED((hr = GetMemory(seg, startAddress, len, progressItf, &psa))))
                        {
                            if(SUCCEEDED((hr = SafeArrayAccessData(psa, &pData1))))
                            {
                                while(len--)
                                {
                                    if(FAILED((hr = VariantChangeTypeEx(&var, reinterpret_cast<VARIANT*>(pData2), CThreadLocale::getLCID(), 0, VT_UI1))))
                                    {
                                        hr = E_UNEXPECTED;
                                        break;
                                    }

                                    if(V_UI1(&var) != *reinterpret_cast<BYTE*>(pData1))
                                    {
                                        hr = SMARTICE04_E_VERIFYFAILED;
                                        break;
                                    }

                                    pData2 = reinterpret_cast<VARIANT*>(pData2) + 1;
                                    pData1 = reinterpret_cast<BYTE*>(pData1) + 1;
                                }
                                SafeArrayUnaccessData(psa);
                            }
                            SafeArrayDestroy(psa);
                        }
                    }
                }
                else
                {
                    hr = E_INVALIDARG;
                }
                SafeArrayUnaccessData(*data);
            }
        }
    }

    pProgress->Stop();
    pProgress->Release();
    m_ObjectStatus &= ~(CSMARTICE04_STATUS_SETMEMORY | CSMARTICE04_STATUS_VERIFY);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_SETMEMORY, hr));
}

STDMETHODIMP
CSmartICE04::DownloadFile(BSTR filename, FileType ft, VARIANT_BOOL verify, VARIANT* progressItf)
{
    static const struct
    {
        LPCWSTR pwszPattern;
        HRESULT (* fnHandler)(LPCWSTR filename, SAFEARRAY** ppsa);
    }
    map[] =
    {
        {   L"*.bin",   CBinaryFile::Load   },
        {   L"*.hex",   CIntelHexFile::Load }
    };

    SAFEARRAY*  psa;
    HRESULT     hr;
    long        lLBound;

    if((!filename) || (static_cast<unsigned long>(ft) > _countof(map)))
        return(RaiseCOMException(DISPID_SMARTICE04_DOWNLOADFILE, E_INVALIDARG));

    if(ft == fileAutoDetect)
    {
        ft = fileBinary;

        for(lLBound = 0; lLBound < _countof(map); )
        {
            if(PathMatchSpecW(filename, map[lLBound++].pwszPattern))
            {
                ft = static_cast<enum FileType>(lLBound);
                break;
            }
        }
    }

    if(FAILED((hr = map[ft - 1].fnHandler(filename, &psa))))
        return(RaiseCOMException(DISPID_SMARTICE04_DOWNLOADFILE, hr));

    if(SUCCEEDED((hr = SafeArrayGetLBound(psa, 1, &lLBound))))
        hr = SetMemory(segCode, lLBound, &psa, verify, progressItf);

    SafeArrayDestroy(psa);
    return(hr);    
}

STDMETHODIMP
CSmartICE04::GetBm(long* bm)
{
    IOREPORT    report;
    HRESULT     hr;

    if(IsBadWritePtrAuto(bm))
    {
        hr = E_POINTER;
    }
    else
    {
        *bm = 0;

        if(!m_blInDebugMode)
        {
            hr = SMARTICE04_E_NOTINPGMMODE;
        }
        else if(SUCCEEDED((hr = ReadReport8(&report))))
        {
            *bm = report.rep8.bBank;
        }
    }

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SMARTICE04_GETBM, hr));
}

STDMETHODIMP
CSmartICE04::EnumConnectionPoints(IEnumConnectionPoints** ppEnum)
{
    CEnumConnectionPoints*  pEnum;
    HRESULT                 hr;

    if(IsBadWritePtrAuto(ppEnum))
        return(E_POINTER);

    *ppEnum = NULL;

    if(NULL == (pEnum = new (std::nothrow) CEnumConnectionPoints))
        return(E_OUTOFMEMORY);

    if(SUCCEEDED((hr = pEnum->Add(m_DispEvents.GetUnknown()))))
        if(SUCCEEDED((hr = pEnum->Add(m_Events.GetUnknown()))))
            hr = pEnum->QueryInterface(IID_IEnumConnectionPoints, (void**)ppEnum);

    pEnum->Release();
    return(hr);
}

STDMETHODIMP
CSmartICE04::FindConnectionPoint(REFIID riid, IConnectionPoint** ppCP)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04Events))
            return(m_Events.QueryInterface(IID_IConnectionPoint, (void**)ppCP));
        else if(IsEqualIID(riid, DIID__DISmartICE04Events))
            return(m_DispEvents.QueryInterface(IID_IConnectionPoint, (void**)ppCP));

        *ppCP = NULL;
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_POINTER);
    }

    return(CONNECT_E_NOCONNECTION);
}

STDMETHODIMP
CSmartICE04::GetClassInfo(ITypeInfo** ppTInfo)
{
    ITypeLib*   pTlb = NULL;
    HRESULT     hr;
    UINT        idx;

    if(IsBadWritePtrAuto(ppTInfo))
        return(E_POINTER);

    *ppTInfo = NULL;

    if(SUCCEEDED((hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &idx))))
    {
        hr = pTlb->GetTypeInfoOfGuid(CLSID_SmartICE04, ppTInfo);
        pTlb->Release();
    }

    return(hr);
}

STDMETHODIMP
CSmartICE04::GetGUID(DWORD dwGuidKind, GUID* pGUID)
{
    if(IsBadWritePtrAuto(pGUID))
        return(E_POINTER);

    if(dwGuidKind != GUIDKIND_DEFAULT_SOURCE_DISP_IID)
    {
        *pGUID = GUID_NULL;
        return(E_INVALIDARG);
    }

    *pGUID = DIID__DISmartICE04Events;
    return(S_OK);
}

STDMETHODIMP
CSmartICE04::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISmartICE04))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}

void
CSmartICE04::OnDetached() throw()
{
    m_blDetached = TRUE;
    m_pParent->AddRef();
}

NTSTATUS WINAPI
CSmartICE04::HidP_InitializeReportForID(IN HIDP_REPORT_TYPE, IN UCHAR ReportID, IN PHIDP_PREPARSED_DATA, IN OUT PCHAR Report, IN ULONG ReportLength) throw()
{
    ZeroMemory(Report, ReportLength);
    Report[0] = static_cast<CHAR>(ReportID);
    return(HIDP_STATUS_SUCCESS);
}

HRESULT
CSmartICE04::Create(ITypeLib* pTlb, CSmartICE04Devices* pParent, HANDLE hDevice, LPCTSTR pszDeviceName, WORD wProductId, CSmartICE04** ppDev) throw()
{
    static const WORD wPID[] =
    {
        0xCC11
    };

    HRESULT hr = E_INVALIDARG;

    for(UINT x = 0; x < _countof(wPID); ++x)
    {
        if(wPID[x] == wProductId)
        {
            if(NULL == (*ppDev = new (std::nothrow) CSmartICE04(pTlb, pParent, hDevice, pszDeviceName, wProductId, &hr)))
                return(E_OUTOFMEMORY);

            if(FAILED(hr))
                (*ppDev)->Release();

            break;
        }
    }

    return(hr);
}

HRESULT
CSmartICE04::ReadReport1(IOREPORT* pReport) throw()
{
    VARIANT var;
    HRESULT hr;
    DWORD   dw = 0;
    BOOL    blDebugMode;

    do
    {
        if(SUCCEEDED((hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 1, NULL, reinterpret_cast<CHAR*>(&pReport->rep1), sizeof(*pReport))))))
        {
            if(!ReadFile(m_hDevice, &pReport->rep1, sizeof(*pReport), &dw, NULL))
            {
                dw = GetLastError();
                hr = HRESULT_FROM_WIN32(dw);
            }
            else if(dw != sizeof(*pReport))
            {
                hr = E_FAIL;
            }
            else if(pReport->rep1.bReportId == 0x01)
            {
                if(memcmp(&pReport->rep1, &m_report1, sizeof(m_report1)))
                {
                    VariantInit(&var);
                    blDebugMode = TRUE;

                    if((pReport->rep1.bStatus & TargetVoltagePresent) != (m_report1.bStatus & TargetVoltagePresent))
                    {
                        m_report1.bStatus = pReport->rep1.bStatus;
                        V_VT(&var) = VT_BOOL;
                        V_BOOL(&var) = (pReport->rep1.bStatus & TargetVoltagePresent) ? VARIANT_TRUE : VARIANT_FALSE;
                        m_DispEvents.FireEvent(DISPID_SMARTICE04_ONTARGETVOLTAGECHANGED, &var);
                        m_Events.FireEvent(DISPID_SMARTICE04_ONTARGETVOLTAGECHANGED, &var);
                        blDebugMode = FALSE;
                    }

                    if(pReport->rep1.bChipId != m_report1.bChipId)
                    {
                        m_report1.bChipId = pReport->rep1.bChipId;
                        V_VT(&var) = VT_I2;
                        V_I2(&var) = pReport->rep1.bChipId;
                        m_DispEvents.FireEvent(DISPID_SMARTICE04_ONTARGETCHANGED, &var);
                        m_Events.FireEvent(DISPID_SMARTICE04_ONTARGETCHANGED, &var);
                        blDebugMode = FALSE;
                    }

                    m_report1 = pReport->rep1;

                    if(!blDebugMode)
                        EnterDebugMode(VARIANT_FALSE);
                }
                break;
            }
        }
    } while(SUCCEEDED(hr));

    return(hr);
}

HRESULT
CSmartICE04::ReadReport5(IOREPORT* pReport) const throw()
{
    HRESULT hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 5, NULL, reinterpret_cast<CHAR*>(&pReport->rep5), sizeof(*pReport)));
    DWORD   dw;

    if(FAILED(hr))
        return(hr);

    if(!HidD_GetInputReport(m_hDevice, &pReport->rep5, sizeof(*pReport)))
    {
        dw = GetLastError();
        return(HRESULT_FROM_WIN32(dw));
    }

    return(S_OK);
}

HRESULT
CSmartICE04::ReadReport6(IOREPORT* pReport) const throw()
{
    HRESULT hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 6, NULL, reinterpret_cast<CHAR*>(&pReport->rep6), sizeof(*pReport)));
    DWORD   dw;

    if(FAILED(hr))
        return(hr);

    if(!HidD_GetInputReport(m_hDevice, &pReport->rep6, sizeof(*pReport)))
    {
        dw = GetLastError();
        return(HRESULT_FROM_WIN32(dw));
    }

    return(S_OK);
}

HRESULT
CSmartICE04::ReadReport7(IOREPORT* pReport) const throw()
{
    HRESULT hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 7, NULL, reinterpret_cast<CHAR*>(&pReport->rep7), sizeof(*pReport)));
    DWORD   dw;

    if(FAILED(hr))
        return(hr);

    if(!HidD_GetInputReport(m_hDevice, &pReport->rep7, sizeof(*pReport)))
    {
        dw = GetLastError();
        return(HRESULT_FROM_WIN32(dw));
    }

    return(S_OK);
}

HRESULT
CSmartICE04::ReadReport8(IOREPORT* pReport) const throw()
{
    HRESULT hr = HresultFromHidErrorCode(HidP_InitializeReportForID(HidP_Input, 8, NULL, reinterpret_cast<CHAR*>(&pReport->rep6), sizeof(*pReport)));
    DWORD   dw;

    if(FAILED(hr))
        return(hr);

    if(!HidD_GetInputReport(m_hDevice, &pReport->rep6, sizeof(*pReport)))
    {
        dw = GetLastError();
        return(HRESULT_FROM_WIN32(dw));
    }

    return(S_OK);
}

HRESULT
CSmartICE04::WriteReport(const IOREPORT* pReport) const throw()
{
    HRESULT hr;
    DWORD   dw;

    if(!WriteFile(m_hDevice, pReport, sizeof(*pReport), &dw, NULL))
    {
        dw = GetLastError();
        hr = HRESULT_FROM_WIN32(dw);
    }
    else if(dw != sizeof(*pReport))
    {
        hr = E_FAIL;
    }
    else
    {
        hr = S_OK;
    }

    return(hr);
}

HRESULT
CSmartICE04::ExtractProgressItf(VARIANT *progressItf, CSmartICE04ProgressDummy *pDummy, ISmartICE04Progress **ppProgressItf) throw()
{
    IDispatch*  pDispatch;
    VARIANT     var;
    HRESULT     hr = S_OK;

    if(!progressItf)
    {
        *ppProgressItf = pDummy;
    }
    else if((V_VT(progressItf) == VT_ERROR) || (V_VT(progressItf) == VT_EMPTY))
    {
        *ppProgressItf = pDummy;
    }
    else
    {
        VariantInit(&var);

        if(SUCCEEDED((hr = VariantChangeTypeEx(&var, progressItf, CThreadLocale::getLCID(), 0, VT_UNKNOWN))))
        {
            if(!V_UNKNOWN(&var))
            {
                *ppProgressItf = pDummy;
            }
            else if(FAILED(V_UNKNOWN(&var)->QueryInterface(IID_ISmartICE04Progress, (void**)ppProgressItf)))
            {
                pDispatch = NULL;

                if(SUCCEEDED((hr = V_UNKNOWN(&var)->QueryInterface(DIID_ISmartICE04ProgressDisp, (void**)&pDispatch))))
                {
                    *ppProgressItf = pDummy;
                    pDummy->SetDispatch(pDispatch);
                    pDispatch->Release();
                }
            }
        }

        VariantClear(&var);
    }

    return(hr);
}