#include "precomp.h"

HRESULT HresultFromHidErrorCode(NTSTATUS stat) throw()
{
    // Note: HRESULT_FROM_WIN32 is an inline function, hence you get C4640 when
    // compiling with /Wall. This is why __HRESULT_FROM_WIN32 has been used instead.
    static const struct
    {
        NTSTATUS    stat;
        HRESULT     hr;
    }
    Map[] =
    {
        {   HIDP_STATUS_SUCCESS,            S_OK                                            },
        {   HIDP_STATUS_BUFFER_TOO_SMALL,   __HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER) },
        {   HIDP_STATUS_INTERNAL_ERROR,     __HRESULT_FROM_WIN32(ERROR_INTERNAL_ERROR)      },
        {   HIDP_STATUS_NOT_IMPLEMENTED,    E_NOTIMPL                                       }
    };

    for(size_t x = 0; x < _countof(Map); ++x)
        if(stat == Map[x].stat)
            return(Map[x].hr);

    return(E_FAIL);
}

HRESULT HresultFromErrno(void) throw()
{
    // Note: HRESULT_FROM_WIN32 is an inline function, hence you get C4640 when
    // compiling with /Wall. This is why __HRESULT_FROM_WIN32 has been used instead.
    static const struct
    {
        int err;
        HRESULT hr;
    }
    map[] =
    {
        {   ENOENT, __HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND)      },
        {   EACCES, E_ACCESSDENIED                                  },
        {   ENOMEM, E_OUTOFMEMORY                                   },
        {   EINVAL, E_INVALIDARG                                    },
        {   EMFILE, __HRESULT_FROM_WIN32(ERROR_TOO_MANY_OPEN_FILES) }
    };

    int err = errno;

    for(size_t x = 0; x < _countof(map); ++x)
        if(map[x].err == err)
            return(map[x].hr);

    return(E_FAIL);
}

BSTR BSTRFromStringA(LPCSTR str) throw()
{
    BSTR    bstr;
    int     nLen = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, -1, NULL, 0) - 1;

    if(NULL != (bstr = SysAllocStringLen(NULL, static_cast<UINT>(nLen))))
        MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, str, nLen, bstr, nLen);

    return(bstr);
}

static WORD GetCharTypeA(const char* pch)
{
    WORD w = 0;
    GetStringTypeExA(CThreadLocale::getLCID(), CT_CTYPE1, pch, 1, &w);
    return(w);
}

void TrimA(char* psz)
{
    LPSTR pFirst, pLast;

    for(pFirst = psz; (GetCharTypeA(pFirst) & C1_SPACE); pFirst = CharNextA(pFirst))
        ;

    for(pLast = CharPrevA(pFirst, pFirst + strlen(pFirst)); (GetCharTypeA(pLast) & C1_SPACE); pLast = CharPrevA(pFirst, pLast))
        *pLast = TEXT('\0');

    if(pFirst > psz)
        MoveMemory(psz, pFirst, (CharNextA(pLast) - pFirst + 1) * sizeof(*psz));
}

HRESULT GetTypeInfoOfChipIdEnumeration(ISmartICE04* pDevice, ITypeInfo** ppTypeInfo)
{
    ITypeInfo* pTypeInfo = NULL;
    FUNCDESC* pDesc = NULL;
    HRESULT hr;
    UINT idx = 0;

    if(FAILED((hr = pDevice->GetTypeInfo(0, CThreadLocale::getLCID(), &pTypeInfo))))
        return(hr);

    for(UINT cnt = 0; SUCCEEDED((hr = pTypeInfo->GetFuncDesc(cnt, &pDesc))); ++cnt)
    {
        if(pDesc->memid == DISPID_SMARTICE04_CHIPID)
        {
            if((pDesc->invkind == INVOKE_PROPERTYGET) && (pDesc->elemdescFunc.tdesc.vt == VT_USERDEFINED) && (pDesc->elemdescFunc.tdesc.hreftype))
            {
                if(SUCCEEDED(hr = pTypeInfo->GetRefTypeInfo(pDesc->elemdescFunc.tdesc.hreftype, ppTypeInfo)))
                    idx = cnt;
            }
        }

        pTypeInfo->ReleaseFuncDesc(pDesc);
        pDesc = NULL;

        if(idx)
            break;
    }

    pTypeInfo->Release();
    return(hr);
}
