#include "precomp.h"

using namespace SELFREG_DATA;
typedef HRESULT (* GET_SPECIAL_DATA)(LPTSTR psz, UINT cchMax);

static HRESULT MyGetModuleFileName(LPTSTR psz, UINT cchMax)
{
    return((!GetModuleFileName(g_hInst, psz, cchMax)) ? E_UNEXPECTED : S_OK);
}

static const GET_SPECIAL_DATA gs_fnGetSpecialString[] =
{
    MyGetModuleFileName
};

static HRESULT AddValue(HKEY hKey, const SELFREG_STRUCT_KEYORVALUE* pKeyOrValue)
{
    if(pKeyOrValue->StructType == SELFREG_TYPE_VALUE_SZ)
    {
        LPCTSTR pValue = pKeyOrValue->Value.sz;
        TCHAR   szSpecial[MAX_PATH * 2];

        if(IS_INTRESOURCE(pKeyOrValue->Value.sz))
        {
            if(pKeyOrValue->Value.sz)
            {
                if((!pKeyOrValue->Value.dw) || (pKeyOrValue->Value.dw > _countof(gs_fnGetSpecialString)))
                    return(E_UNEXPECTED);

                if(FAILED(gs_fnGetSpecialString[pKeyOrValue->Value.dw - 1](szSpecial, _countof(szSpecial))))
                    return(E_UNEXPECTED);

                pValue = szSpecial;
            }
        }

        if(!pValue)
            return(E_UNEXPECTED);

        if(ERROR_SUCCESS != RegSetValueEx(hKey, pKeyOrValue->Name, 0, REG_SZ, reinterpret_cast<CONST BYTE*>(pValue), static_cast<DWORD>((_tcslen(pValue) + 1) * sizeof(*pValue))))
            return(E_UNEXPECTED);
    }
    else if(pKeyOrValue->StructType == SELFREG_TYPE_VALUE_DWORD)
    {
        if(ERROR_SUCCESS != RegSetValueEx(hKey, pKeyOrValue->Name, 0, REG_DWORD, reinterpret_cast<CONST BYTE*>(&pKeyOrValue->Value.dw), sizeof(pKeyOrValue->Value.dw)))
            return(E_UNEXPECTED);
    }

    return(S_OK);
}

static HRESULT DeleteValue(HKEY hKey, const SELFREG_STRUCT_KEYORVALUE* pKeyOrValue)
{
    LONG ret;

    if(ERROR_SUCCESS != (ret = RegDeleteValue(hKey, pKeyOrValue->Name)))
        return((ret == ERROR_FILE_NOT_FOUND) ? S_OK : E_UNEXPECTED);

    return(S_OK);
}

static HRESULT AddKey(HKEY hkeyParent, const SELFREG_STRUCT_KEYORVALUE** pKeyOrValue)
{
    HRESULT hr = S_OK;
    DWORD   dw;
    HKEY    hKey;

    if(ERROR_SUCCESS != RegCreateKeyEx(hkeyParent, (*pKeyOrValue)->Name, 0, NULL, REG_OPTION_NON_VOLATILE, STANDARD_RIGHTS_WRITE | KEY_SET_VALUE, NULL, &hKey, &dw))
        return(E_UNEXPECTED);

    for(++(*pKeyOrValue); (((*pKeyOrValue)->StructType != SELFREG_TYPE_NULL) && (SUCCEEDED(hr))); ++(*pKeyOrValue))
    {
#pragma warning(push)
#pragma warning(disable: 4061 4062)
        switch((*pKeyOrValue)->StructType)
        {
            case SELFREG_TYPE_KEY:
                hr = AddKey(hKey, pKeyOrValue);
                break;
            case SELFREG_TYPE_VALUE_SZ:
            case SELFREG_TYPE_VALUE_DWORD:
                hr = AddValue(hKey, *pKeyOrValue);
                break;
            default:
#if defined(_MSC_VER) && !defined(_DEBUG)
                __assume(0);
#else
#pragma warning(suppress: 4365)
                assert(!"Unhandled case in switch");
                break;
#endif
        }
    }
#pragma warning(pop)

    RegCloseKey(hKey);
    return(hr);
}

static HRESULT DeleteKey(HKEY hkeyParent, const SELFREG_STRUCT_KEYORVALUE** pKeyOrValue)
{
    HRESULT hr = S_OK, hrTmp;
    LPCTSTR pName = (*pKeyOrValue)->Name;
    LONG    dw;
    HKEY    hKey;
    BOOL    bRemoveable = (*pKeyOrValue)->bRemoveable;

    if(ERROR_SUCCESS != (dw = RegOpenKeyEx(hkeyParent, (*pKeyOrValue)->Name, 0, STANDARD_RIGHTS_WRITE | KEY_SET_VALUE, &hKey)))
        hKey = NULL;

    for(++(*pKeyOrValue); ((*pKeyOrValue)->StructType != SELFREG_TYPE_NULL); ++(*pKeyOrValue))
    {
#pragma warning(push)
#pragma warning(disable: 4061)
        switch((*pKeyOrValue)->StructType)
        {
            case SELFREG_TYPE_KEY:
                hrTmp = DeleteKey(hKey, pKeyOrValue);
                break;
            case SELFREG_TYPE_VALUE_SZ:
            case SELFREG_TYPE_VALUE_DWORD:
                if(bRemoveable)
                {
                    hrTmp = ((ERROR_SUCCESS == dw) ? DeleteValue(hKey, *pKeyOrValue) : S_OK);
                    break;
                }
            default:
#if defined(_MSC_VER) && !defined(_DEBUG)
                __assume(0);
#else
#pragma warning(suppress: 4365)
                assert(!"Unhandled case in switch");
                hrTmp = S_OK;
                break;
#endif
        }
#pragma warning(pop)

        if(FAILED(hrTmp))
            hr = hrTmp;
    }

    if(ERROR_SUCCESS == dw)
        RegCloseKey(hKey);

    if((hkeyParent) && (bRemoveable))
        if(ERROR_SUCCESS != (dw = RegDeleteKey(hkeyParent, pName)))
            return((dw == ERROR_FILE_NOT_FOUND) ? hr : E_UNEXPECTED);

    return(hr);
}

static HRESULT AddReg(const SELFREG_STRUCT_KEYORVALUE** pKeyOrValue, BOOL bRevoke)
{
    HRESULT hr = S_OK;
    HKEY    hkeyRoot = (HKEY)(*pKeyOrValue)->Name;

    for(++(*pKeyOrValue); (*pKeyOrValue)->StructType != SELFREG_TYPE_NULL; ++(*pKeyOrValue))
    {
        if((*pKeyOrValue)->StructType != SELFREG_TYPE_KEY)
            continue;

        if(bRevoke)
        {
            if(FAILED(DeleteKey(hkeyRoot, pKeyOrValue)))
                hr = E_UNEXPECTED;
        }
        else if(FAILED((hr = AddKey(hkeyRoot, pKeyOrValue))))
        {
            break;
        }
    }

    return(hr);
}

static SYSKIND GetSysKind(const SELFREG_STRUCT_TYPELIB* pTypeLib)
{
    TLIBATTR*   pAttr   = NULL;
    ITypeLib*   pTlb    = NULL;
    SYSKIND     sknd    = SYS_WIN32;

    if(FAILED(LoadRegTypeLib(*(pTypeLib->pGuid), pTypeLib->wVerMajor, pTypeLib->wVerMinor, pTypeLib->lcid, &pTlb)))
        return(sknd);

    if(SUCCEEDED(pTlb->GetLibAttr(&pAttr)))
    {
        sknd = pAttr->syskind;
        pTlb->ReleaseTLibAttr(pAttr);
    }

    pTlb->Release();
    return(sknd);
}

static HRESULT AddTypeLib(const SELFREG_STRUCT_TYPELIB* pTypeLib, BOOL bRevoke)
{
    HRESULT hr = E_UNEXPECTED;

    if(bRevoke)
    {
        #ifdef _WIN32_WCE
          typedef HRESULT (WINAPI * UNREG)(REFGUID, unsigned short, unsigned short, LCID, SYSKIND);
          HINSTANCE hDll = LoadLibrary(TEXT("oleaut32.dll"));
          if(hDll)
          {
              UNREG fn = reinterpret_cast<UNREG>(GetProcAddress(hDll, TEXT("UnRegisterTypeLib")));

              if(fn)
                hr = fn(*(pTypeLib->pGuid), pTypeLib->wVerMajor, pTypeLib->wVerMinor, pTypeLib->lcid, GetSysKind(pTypeLib));

              FreeLibrary(hDll);
          }
        #else
          hr = UnRegisterTypeLib(*(pTypeLib->pGuid), pTypeLib->wVerMajor, pTypeLib->wVerMinor, pTypeLib->lcid, GetSysKind(pTypeLib));
        #endif
    }
    else
    {
        TCHAR szFileName[MAX_PATH * 2];
        if(FAILED(gs_fnGetSpecialString[reinterpret_cast<UINT_PTR>(SELFREG_VALUE_FILENAME) - 1](szFileName, MAX_PATH * 2)))
            return(E_UNEXPECTED);

        ITypeLib* pTlb = NULL;
        #ifdef UNICODE
          #ifdef _WIN32_WCE
            if(SUCCEEDED((hr = LoadTypeLib(szFileName, &pTlb))))
            {
                hr = RegisterTypeLib(pTlb, szFileName, NULL);
                pTlb->Release();
            }
          #else
            if(SUCCEEDED((hr = LoadTypeLibEx(szFileName, REGKIND_REGISTER, &pTlb))))
                pTlb->Release();
          #endif
        #else
          WCHAR wszTemp[_countof(szFileName)];
          MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szFileName, -1, wszTemp, _countof(wszTemp));
          #ifdef _WIN32_WCE
            if(SUCCEEDED((hr = LoadTypeLib(wszTemp, &pTlb))))
            {
                hr = RegisterTypeLib(pTlb, wszTemp, NULL);
                pTlb->Release();
            }
          #else
            if(SUCCEEDED((hr = LoadTypeLibEx(wszTemp, REGKIND_REGISTER, &pTlb))))
                pTlb->Release();
          #endif
        #endif
    }

    return(hr);
}

STDAPI DllUnregisterServer(VOID)
{
    HRESULT hr = S_OK;
    const SELFREG_TYPE_ENUM** pEnum = reinterpret_cast<const SELFREG_TYPE_ENUM**>(SelfregMap);

    for(; (*pEnum); ++pEnum)
        ;

    for(--pEnum; (reinterpret_cast<const VOID**>(pEnum) >= SelfregMap); --pEnum)
    {
#pragma warning(push)
#pragma warning(disable: 6011)
        if(**pEnum == SELFREG_TYPE_REGISTRY_MAP)
        {
            for(const SELFREG_STRUCT_KEYORVALUE* pKeyOrValue = reinterpret_cast<const SELFREG_STRUCT_KEYORVALUE*>(*pEnum) + 1; pKeyOrValue->StructType != SELFREG_TYPE_NULL; ++pKeyOrValue)
            {
                if(pKeyOrValue->StructType == SELFREG_TYPE_ROOTKEY)
                    if(FAILED(AddReg(&pKeyOrValue, TRUE)))
                        hr = SELFREG_E_CLASS;
            }
        }

        if(**pEnum == SELFREG_TYPE_TYPELIB_MAP)
        {
            for(const SELFREG_STRUCT_TYPELIB* pTypeLib = reinterpret_cast<const SELFREG_STRUCT_TYPELIB*>(*pEnum) + 1; pTypeLib->StructType != SELFREG_TYPE_NULL; ++pTypeLib)
            {
                if(pTypeLib->StructType == SELFREG_TYPE_TYPELIB)
                    if(FAILED(AddTypeLib(pTypeLib, TRUE)))
                        hr = SELFREG_E_CLASS;
            }
        }

        if(**pEnum == SELFREG_TYPE_CALLBACK_MAP)
        {
            for(const SELFREG_STRUCT_CALLBACK* pCallback = reinterpret_cast<const SELFREG_STRUCT_CALLBACK*>(*pEnum) + 1; pCallback->StructType != SELFREG_TYPE_NULL; ++pCallback)
            {
                if(pCallback->StructType == SELFREG_TYPE_CALLBACK)
                    if(!pCallback->bOnRegister)
                        if(FAILED((hr = pCallback->pfn())))
                            hr = SELFREG_E_CLASS;
            }
        }
#pragma warning(pop)
    }

    return(hr);
}

STDAPI DllRegisterServer(VOID)
{
    HRESULT hr = S_OK;

    for(const SELFREG_TYPE_ENUM** pEnum = reinterpret_cast<const SELFREG_TYPE_ENUM**>(SelfregMap); (*pEnum); ++pEnum)
    {
        if(**pEnum == SELFREG_TYPE_REGISTRY_MAP)
        {
            for(const SELFREG_STRUCT_KEYORVALUE* pKeyOrValue = reinterpret_cast<const SELFREG_STRUCT_KEYORVALUE*>(*pEnum) + 1; pKeyOrValue->StructType != SELFREG_TYPE_NULL; ++pKeyOrValue)
            {
                if(pKeyOrValue->StructType == SELFREG_TYPE_ROOTKEY)
                    if(FAILED((hr = AddReg(&pKeyOrValue, FALSE))))
                        break;
            }
        }

        if(**pEnum == SELFREG_TYPE_TYPELIB_MAP)
        {
            for(const SELFREG_STRUCT_TYPELIB* pTypeLib = reinterpret_cast<const SELFREG_STRUCT_TYPELIB*>(*pEnum) + 1; pTypeLib->StructType != SELFREG_TYPE_NULL; ++pTypeLib)
            {
                if(pTypeLib->StructType == SELFREG_TYPE_TYPELIB)
                    if(FAILED((hr = AddTypeLib(pTypeLib, FALSE))))
                        break;
            }
        }

        if(**pEnum == SELFREG_TYPE_CALLBACK_MAP)
        {
            for(const SELFREG_STRUCT_CALLBACK* pCallback = reinterpret_cast<const SELFREG_STRUCT_CALLBACK*>(*pEnum) + 1; pCallback->StructType != SELFREG_TYPE_NULL; ++pCallback)
            {
                if(pCallback->StructType == SELFREG_TYPE_CALLBACK)
                    if(pCallback->bOnRegister)
                        if(FAILED((hr = pCallback->pfn())))
                            break;
            }
        }

        if(FAILED(hr))
        {
            DllUnregisterServer();
            return(SELFREG_E_CLASS);
        }
    }

    return(S_OK);
}
