#ifndef SELFREG_H
#define SELFREG_H

#include <pshpack1.h>

namespace SELFREG_DATA
{
    typedef enum
    {
        SELFREG_TYPE_NULL = 0,
        SELFREG_TYPE_ROOTKEY,
        SELFREG_TYPE_KEY,
        SELFREG_TYPE_VALUE_SZ,
        SELFREG_TYPE_VALUE_DWORD,
        SELFREG_TYPE_TYPELIB,
        SELFREG_TYPE_REGISTRY_MAP,
        SELFREG_TYPE_TYPELIB_MAP,
        SELFREG_TYPE_CALLBACK_MAP,
        SELFREG_TYPE_CALLBACK
    } SELFREG_TYPE_ENUM;

    typedef struct
    {
        SELFREG_TYPE_ENUM StructType;
        LPCTSTR Name;
        BOOL bRemoveable;
        union
        {
            LPCTSTR sz;
            DWORD   dw;
        } Value;
    } SELFREG_STRUCT_KEYORVALUE;

    typedef struct
    {
        SELFREG_TYPE_ENUM StructType;
        const GUID*       pGuid;
        unsigned short    wVerMajor;
        unsigned short    wVerMinor;
        LCID              lcid;
    } SELFREG_STRUCT_TYPELIB;

    typedef HRESULT (WINAPI * SELFREG_CALLBACKPROC)(void);

    typedef struct
    {
        SELFREG_TYPE_ENUM     StructType;
        SELFREG_CALLBACKPROC  pfn;
        BOOL                  bOnRegister;
    } SELFREG_STRUCT_CALLBACK;

    extern const void* SelfregMap[];
}

#define SELFREG_MAP_BEGIN           namespace SELFREG_DATA { const void* SelfregMap[] = {
#define SELFREG_MAP_END             , NULL };}

#define SELFREG_REGISTRY_BEGIN(nam) namespace SELFREG_DATA {  const SELFREG_STRUCT_KEYORVALUE nam##[] = {{ SELFREG_TYPE_REGISTRY_MAP, NULL, FALSE, NULL },
#define SELFREG_REGISTRY_END        { SELFREG_TYPE_NULL, 0, FALSE, 0 }};}
#define SELFREG_ROOT_BEGIN(hkey)    { SELFREG_TYPE_ROOTKEY, (LPCTSTR)(hkey), FALSE,  NULL },
#define SELFREG_KEY_BEGIN(nam)      { SELFREG_TYPE_KEY, (nam), TRUE, NULL     },
#define SELFREG_KEY_BEGIN_PERSIST(nam) { SELFREG_TYPE_KEY, (nam), FALSE, NULL     },
#define SELFREG_VALUE_SZ(nam, val)  { SELFREG_TYPE_VALUE_SZ, (nam), TRUE, { (val) }},
#define SELFREG_VALUE_DW(nam, val)  { SELFREG_TYPE_VALUE_DWORD, (nam), TRUE, { (LPCTSTR)(DWORD_PTR)(DWORD)(val) }},
#define SELFREG_END                 { SELFREG_TYPE_NULL, 0, FALSE, 0 },

#define SELFREG_TYPELIB_BEGIN(nam)  namespace SELFREG_DATA {  const SELFREG_STRUCT_TYPELIB nam##[] = {{ SELFREG_TYPE_TYPELIB_MAP, NULL, 0, 0, LOCALE_NEUTRAL },
#define SELFREG_TYPELIB(pGuid, vMj, vMn, lcid) { SELFREG_TYPE_TYPELIB, (pGuid), (vMj), (vMn), (lcid) },
#define SELFREG_TYPELIB_END         { SELFREG_TYPE_NULL, NULL, 0, 0, LOCALE_NEUTRAL }};}

#define SELFREG_CALLBACK_BEGIN(nam) namespace SELFREG_DATA {  const SELFREG_STRUCT_CALLBACK nam##[] = {{ SELFREG_TYPE_CALLBACK_MAP, NULL, FALSE },
#define SELFREG_CALLBACK_END        { SELFREG_TYPE_NULL, NULL, FALSE }};}
#define SELFREG_CALLBACK_ONREGSERVER(func)  { SELFREG_TYPE_CALLBACK, (func), TRUE  },
#define SELFREG_CALLBACK_ONUNREGSERVER(func){ SELFREG_TYPE_CALLBACK, (func), FALSE },

#define SELFREG_VALUE_FILENAME      MAKEINTRESOURCE(1)

#include <poppack.h>

#endif