#include "precomp.h"
#pragma intrinsic(memcmp)

WNDPROC CSmartRF04EBDialog::m_wndpEdit = NULL;

CSmartRF04EBDialog::CSmartRF04EBDialog()
{
    m_wndpEdit = NULL;
}

CSmartRF04EBDialog::~CSmartRF04EBDialog()
{
}

LRESULT CALLBACK
CSmartRF04EBDialog::EditSubclassProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    NCCALCSIZE_PARAMS*  pnccs;
    LRESULT             lResult;
    POINT               pt;
    RECT                rc;
    HWND                hWndBtn;

    lResult = CallWindowProc(m_wndpEdit, hWnd, uMsg, wParam, lParam);

    switch(uMsg)
    {
        case WM_NCCALCSIZE:
            if(wParam)
            {
                pnccs   = (NCCALCSIZE_PARAMS*)lParam;
                hWndBtn = GetWindow(hWnd, GW_HWNDNEXT);

                GetWindowRect(hWndBtn, &rc);

                rc.right  -= rc.left;
                rc.bottom  = pnccs->rgrc[0].bottom - pnccs->rgrc[0].top;

                pnccs->rgrc[0].right -= rc.right;

                rc.left    = pnccs->rgrc[0].right;
                rc.top     = pnccs->rgrc[0].top;

                MoveWindow(hWndBtn, rc.left, rc.top, rc.right, rc.bottom, TRUE);
            }
            break;
        case WM_NCHITTEST:
            if(lResult == HTNOWHERE)
            {
                GetWindowRect(GetWindow(hWnd, GW_HWNDNEXT), &rc);
                POINTSTOPOINT(pt, lParam);

                if(PtInRect(&rc, pt))
                    lResult = HTTRANSPARENT;
            }
            break;
        default:
            break;
    }

    return(lResult);
}

INT_PTR
CSmartRF04EBDialog::OnWmInitDialog(HWND hWnd, WPARAM, LPARAM)
{
    const char* pSerNum;
    DWORD       dwDevices = 0, dwCurrent;
    TCHAR       szPort[32];
    HWND        hEdit;
    int         idx;

    SetWindowLongPtr(hWnd, GWL_EXSTYLE, (GetWindowLongPtr(hWnd, GWL_EXSTYLE) | WS_EX_APPWINDOW));

    hEdit = GetDlgItem(hWnd, IDC_BOOTLOADER);
    m_wndpEdit = reinterpret_cast<WNDPROC>(SetWindowLongPtr(hEdit, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(EditSubclassProc)));
    SetWindowPos(hEdit, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);

    hEdit = GetDlgItem(hWnd, IDC_IMAGE);
    SetWindowLongPtr(hEdit, GWLP_WNDPROC, reinterpret_cast<LONG_PTR>(EditSubclassProc));
    SetWindowPos(hEdit, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);

    ::USBDebugDevices(&dwDevices);

    for(dwCurrent = 0; dwCurrent < dwDevices; ++dwCurrent)
    {
        if(FAILED(::GetUSBDeviceSN(dwCurrent, &pSerNum)))
            continue;

        if(0 > (idx = static_cast<int>(SendDlgItemMessageA(hWnd, IDC_DEBUGADAPTER, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(pSerNum)))))
            continue;

        SendDlgItemMessage(hWnd, IDC_DEBUGADAPTER, CB_SETITEMDATA, static_cast<WPARAM>(idx), reinterpret_cast<LPARAM>(pSerNum));
    }

    lstrcpy(szPort, TEXT("COM"));

    for(dwCurrent = 1; dwCurrent <= 1024; ++dwCurrent)
    {
        _itot(static_cast<int>(dwCurrent), &szPort[3], 10);
        QueryDosDevice(szPort, NULL, 0);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
            continue;

        if(0 > (idx = static_cast<int>(SendDlgItemMessage(hWnd, IDC_DEBUGADAPTER, CB_ADDSTRING, 0, reinterpret_cast<LPARAM>(szPort)))))
            continue;

        SendDlgItemMessage(hWnd, IDC_DEBUGADAPTER, CB_SETITEMDATA, static_cast<WPARAM>(idx), static_cast<LPARAM>(dwCurrent));
    }

    SendDlgItemMessage(hWnd, IDC_DEBUGADAPTER, CB_SETCURSEL, 0, 0);
    SendDlgItemMessage(hWnd, IDC_EBID, EM_LIMITTEXT, 4, 0);
    SetDlgItemInt(hWnd, IDC_EBID, 50, FALSE);
    return TRUE;
}

VOID
CSmartRF04EBDialog::DoDownload(HWND hWndParent)
{
    const char* pSerNum;
    CSiUtil*    pSiUtil;
    CCursor     cursor(IDC_WAIT);
    TCHAR       szId[8];
    UINT        ebid;
    char        szFileName[MAX_PATH * 2];
    int         idx;

    if(CB_ERR == (idx = static_cast<int>(SendDlgItemMessage(hWndParent, IDC_DEBUGADAPTER, CB_GETCURSEL, 0, 0))))
        return;

    pSerNum = reinterpret_cast<const char*>(SendDlgItemMessage(hWndParent, IDC_DEBUGADAPTER, CB_GETITEMDATA, static_cast<WPARAM>(idx), 0));
    GetDlgItemTextA(hWndParent, IDC_BOOTLOADER, szFileName, _countof(szFileName));

    CSiUtilEC2 ec2(PtrToInt(pSerNum));
    CSiUtilEC3 ec3(pSerNum);

    if(IS_INTRESOURCE(pSerNum))
        pSiUtil = &ec2;
    else
        pSiUtil = &ec3;

    if(FAILED(pSiUtil->Download(szFileName, TRUE)))
        return;

    cursor.Refresh();
    GetDlgItemTextA(hWndParent, IDC_IMAGE, szFileName, _countof(szFileName));

    if(FAILED(pSiUtil->Download(szFileName, FALSE)))
        return;

    cursor.Refresh();
    MarkDeviceAsBusPowered(pSiUtil);
    cursor.Refresh();

    if(GetDlgItemText(hWndParent, IDC_EBID, szId, _countof(szId)))
    {
        ebid = _tcstoul(szId, NULL, 16);
        pSiUtil->SetCodeMemory(reinterpret_cast<BYTE*>(&ebid), 0x008A, 2);
    }

    cursor.Refresh();
    pSiUtil->SetTargetGo();
}

VOID
CSmartRF04EBDialog::MarkDeviceAsBusPowered(CSiUtil* pSiUtil)
{
    // Buggy Descriptor (bmAttributes.7 is set to 0)
    static const BYTE bDefaultConfiguration1[] =
    {
        0x09, 0x02, 0x20, 0x00, 0x01, 0x01, 0x00, 0x40,
        0x32, 0x09, 0x04, 0x00, 0x00, 0x02, 0xFF, 0xFF,
        0xFF, 0x00, 0x07, 0x05, 0x82, 0x02, 0x40, 0x00,
        0x05, 0x07, 0x05, 0x02, 0x02, 0x40, 0x00, 0x05
    };

    // Correct Descriptor (bmAttributes.7 set to 1)
    static const BYTE bDefaultConfiguration2[] =
    {
        0x09, 0x02, 0x20, 0x00, 0x01, 0x01, 0x00, 0xC0,
        0x32, 0x09, 0x04, 0x00, 0x00, 0x02, 0xFF, 0xFF,
        0xFF, 0x00, 0x07, 0x05, 0x82, 0x02, 0x40, 0x00,
        0x05, 0x07, 0x05, 0x02, 0x02, 0x40, 0x00, 0x05
    };

    BYTE bMem[0x3E00];  // 16 KB minus reserved page
    unsigned x;

    if(FAILED(pSiUtil->GetCodeMemory(bMem, 0x0000, sizeof(bMem))))
        return;

    for(x = 0; x < (sizeof(bMem) - sizeof(bDefaultConfiguration1)); ++x)
    {
        if(!memcmp(&bMem[x], bDefaultConfiguration1, sizeof(bDefaultConfiguration1)))
        {
            x += 7;
            bMem[x] &= ~0x40;   // mark as bus powered
            bMem[x] |= 0x80;    // set bmAttributes.7
            pSiUtil->SetCodeMemory(&bMem[x], x, 1);
            return;
        }
    }

    // Buggy Descriptor not found, see if it has been fixed by TI
    for(x = 0; x < (sizeof(bMem) - sizeof(bDefaultConfiguration2)); ++x)
    {
        if(!memcmp(&bMem[x], bDefaultConfiguration2, sizeof(bDefaultConfiguration2)))
        {
            x += 7;
            bMem[x] &= ~0x40;   // mark as bus powered
            pSiUtil->SetCodeMemory(&bMem[x], x, 1);
            return;
        }
    }
}

INT_PTR
CSmartRF04EBDialog::OnWmCommand(HWND hWnd, WPARAM wParam, LPARAM)
{
    switch(LOWORD(wParam))
    {
        case IDC_BOOTLOADER_BROWSE:
        case IDC_IMAGE_BROWSE:
            AskForFileName(hWnd, LOWORD(wParam) - 1, IDS_FILTER, GUID_SmartRF04EB);
            return(TRUE);
        case IDOK:
            DoDownload(hWnd);
            return TRUE;
        case IDCANCEL:
            EndDialog(hWnd, 0);
            return TRUE;
        default:
            break;
    }

    return FALSE;
}

INT_PTR
CSmartRF04EBDialog::DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
#pragma warning(push)
#pragma warning(disable: 4820)
    static const struct
    {
        UINT uMsg;
        INT_PTR (CSmartRF04EBDialog::*pfn)(HWND hWnd, WPARAM wParam, LPARAM lParam);
    }
    map[] =
    {
        {   WM_COMMAND,     &OnWmCommand    }
    };
#pragma warning(pop)

    for(size_t x = 0; x < _countof(map); ++x)
        if(map[x].uMsg == uMsg)
            return (reinterpret_cast<CSmartRF04EBDialog*>(GetWindowLongPtr(hWnd, DWLP_USER))->*map[x].pfn)(hWnd, wParam, lParam);

    if(uMsg == WM_INITDIALOG)
    {
        SetWindowLongPtr(hWnd, DWLP_USER, lParam);
        return reinterpret_cast<CSmartRF04EBDialog*>(lParam)->OnWmInitDialog(hWnd, wParam, lParam);
    }

    return FALSE;
}

int
CSmartRF04EBDialog::DoModal(HWND hWndParent)
{
    CSmartRF04EBDialog dlg;
    return(static_cast<int>(DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_SMARTRF04EB), hWndParent, DlgProc, reinterpret_cast<LPARAM>(&dlg))));
}
